<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2009 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Jun 13, 2009
 * @id $Id: XMLRPCTest.php,v 1.7 2009/09/26 08:21:41 poehler Exp $
 */

/**
 * XMLRPC API Tests
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class API_XMLRPCTest extends IWorx_Framework_TestCase {
  protected $_xml = null;

  /**
   * setup function
   */
  public function setup() {
    exec( 'rm -rf /tmp/wsdl*' );

    exec( '/bin/hostname', $result );
    $this->_initApiObj( $result[ 0 ] );
  }

  /**
   * init api object
   *
   * @param string $hostname
   */
  protected function _initApiObj( $hostname ) {
    $url = "http://{$hostname}:2080/nodeworx/xmlrpc";
    $this->_xml = new Zend_XmlRpc_Client( $url );
  }

  /**
   * gets the installed api key
   *
   * @return string
   */
  private function _getGoodApiKey() {
    $sql = 'SELECT apikey FROM nodeworx WHERE nodeworx_id = ' . q( 1 );
    $key = IWorxDbUtil::db_getOne( IWorxDbUtil::getIWorxDB(), $sql );
    $this->assertTrue( trim( $key ) !== '' );
    $apikey = API_Key::db2pretty( 1,  $key );
    return $apikey;
  }

  /**
   * api call func
   *
   * @return array result of the api call
   */
  protected function _apiCall() {
    $args = func_get_args();
    $method = array_shift( $args );
    $result = $this->_xml->call( "iworx.{$method}", $args[0] );

    $this->_assertResponseLooksValid( $result );
    return $result;
  }

  /**
   * assert response looks valid
   *
   * @param mixed $resp
   */
  protected function _assertResponseLooksValid( $resp ) {
    $this->assertTrue( is_array( $resp ), 'response not an array' );
    $this->assertArrayHasKey( 'status',  $resp );
    $this->assertArrayHasKey( 'payload', $resp );
  }

  /**
   * assert response is "win" bundle
   *
   * @param mixed $resp
   */
  protected function _assertResponseIsWinBundle( $resp ) {
    $this->assertEquals( API::SUCCESS, $resp['status'] );
    $this->assertEquals( 'You Win', $resp['payload'] );
  }

  /**
   * assert response is "fail" bundle
   *
   * @param mixed $resp
   */
  protected function _assertResponseIsFailBundle( $resp ) {
    $this->assertEquals( API::GENERAL_FAILURE, $resp['status'] );
    $this->assertEquals( 'You Fail', $resp['payload'] );
  }

  /**
   * test
   */
  public function testNoParams() {
    $params = array( 'apikey'    => '',
                     'ctrl_name' => '',
                     'action'    => '',
                     'input'     => array() );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (ApiKey)', $result['payload'] );
  }

  /**
   * test
   */
  public function testNodeworxWithApiKey() {
    $input  = array();
    $params = array( 'apikey'    => $this->_getGoodApiKey(),
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * test
   */
  public function testNodeworxWithBadApiKey() {
    $input  = array();
    $params = array( 'apikey'    => 'hacker123',
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (ApiKey)', $result['payload'] );
  }

  /**
   * test
   */
  public function testNodeworxGetSessionWithApiKey() {

  }

  /**
   * test
   */
  public function testNodeworxGetSessionWithLogin() {
    $input = array();
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * test
   */
  public function testNodeworxSessionIdOnSiteworxController() {
    $input = array();
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (SiteWorx)', $result['payload'] );
  }

  /**
   * test
   */
  public function testSiteworxGetSessionWithLogin() {
    $input = array();
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => self::SW_PASS,
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * test
   */
  public function testSiteworxSessionIdOnNodeworxController() {
    $input = array();
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => self::SW_PASS,
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (NodeWorx)', $result['payload'] );
  }

  /**
   * test
   */
  public function testNodeworxWithLogin() {
    $input = array();
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * test
   */
  public function testNodeworxWithBadLogin() {
    $input = array();
    $key   = array( 'email'    => self::NW_USER . 'boo',
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (NodeWorx)', $result['payload'] );
  }

  /**
   * test
   */
  public function testSiteworxWithApiKeyFails() {
    $input  = array();
    $params = array( 'apikey'    => $this->_getGoodApiKey(),
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (SiteWorx)', $result['payload'] );
  }

  /**
   * test
   */
  public function testSiteworxWithLogin() {
    $input = array();
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => self::SW_PASS,
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * test
   */
  public function testSiteworxWithBadLogin() {
    $input = array();
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => 'WRONG',
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (SiteWorx)', $result['payload'] );
  }

  /**
   * test
   */
  public function testBasicFailure() {
    $input = array();
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'failCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsFailBundle( $result );
  }

  /**
   * test
   */
  public function testNodeworxUserEdit() {
    $input = array( 'id' => 1 );
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/users',
                     'action'    => 'edit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( API::SUCCESS, $result['status'] );
  }

  /**
   * test
   */
  public function testGetNextUidAndGidAction() {
    $key    = $this->_getGoodApiKey();
    $API    = new XMLRPC_API_Client( '127.0.0.1', $key );
    $result = $API->getNextUidAndGid();
    $this->assertArrayHasKey( 'uid', $result );
    $this->assertArrayHasKey( 'gid', $result );
  }
}
?>