<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Mar 6, 2009
 * @id $Id: AbstractTest.php,v 1.2 2009/06/02 17:06:33 poehler Exp $
 */

/**
 * Test Suite targeting logic contained in the Ctrl_Nodeworx_Abstract class
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_AbstractTest extends IWorx_Framework_TestCase {

  const CTRL_RSL_ALLOW     = 'Ctrl_Nodeworx_Overview';
  const CTRL_RSL_NOT_ALLOW = 'Ctrl_Nodeworx_Ip';
  const CTRL_RSL           = 'Ctrl_Nodeworx_Reseller';

  /**
   * test suite setup
   */
  public function testSetup() {
    $NW     = self::getMasterNodeWorx();
    $input  = $this->_getResellerInput();
    $result = IW::FC()->routeFromPHP( self::CTRL_RSL, 'add', $input, $NW );
  }

  /**
   * test assigned vars of index action
   */
  public function testResellerAllowedToAccess() {
    $RSL = self::getResellerNodeWorx();
    if( $RSL->getId() > 0 ) {
      $result = IW::FC()->routeFromPHP( self::CTRL_RSL_ALLOW,
                                        'index',
                                        array(),
                                        $RSL );
      $this->assertType( 'array', $result );
    } else {
      $this->markTestSkipped( 'could not get reseller for testing' );
    }
  }

  /**
   * test
   *
   * @expectedException IWorx_Exception
   */
  public function testResellerNotAllowedToAccessController() {
    $RSL = self::getResellerNodeWorx();
    if( $RSL->getId() > 0 ) {
      $result = IW::FC()->routeFromPHP( self::CTRL_RSL_NOT_ALLOW,
                                        'index',
                                        array(),
                                        $RSL );
    } else {
      $this->markTestSkipped( 'could not get reseller for testing' );
    }
  }

  /**
   * test
   */
  public function testResellerNotAllowedToAccessAction() {
    $RSL    = self::getResellerNodeWorx();
    if( $RSL->getId() > 0 ) {
      $result = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Dns',
                                        'zoneDefaults',
                                        array(),
                                        $RSL );
      $this->assertType( 'array', $result );
      $this->assertEquals( '##LG_ACTION_NOT_PERMITTED##', $result[0] );
    } else {
      $this->markTestSkipped( 'could not get reseller for testing' );
    }
  }

  /**
   * tests delete action return true
   */
  public function testDeleteActionReturnsTrue() {
    $RSL    = self::getResellerNodeWorx();
    $NW     = self::getMasterNodeWorx();
    if( $RSL->getId() > 0 ) {
      $input  = array( 'reseller_id' => $RSL->getId() );
      $result = IW::FC()->routeFromPHP( self::CTRL_RSL, 'delete', $input, $NW );
    } else {
      $this->markTestSkipped( 'could not get reseller for testing' );
    }
  }

}
?>