<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: BackupTest.php,v 1.4 2009/09/16 07:46:23 poehler Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Backup
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */

class Ctrl_Nodeworx_BackupTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Backup';

  /**
   * tests assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW    = self::getMasterNodeWorx();
    $input = array();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', $input, $NW );

    $this->assertArrayHasKey( 'iw_accounts_list', $result );
    $this->assertArrayHasKey( 'iw_user_email', $result );
  }

  /**
   * tests creating structure only backup
   */
  public function testBackupActionStructureOnlyBackup() {
    self::addSWAccountIfNecessary();
    $NW = self::getMasterNodeWorx();

    $input = array( 'input' => array( 'domains' => array( self::SW_DOMAIN ),
                                      'email' => '',
                                      'type' => 'structure_only' ) );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'backup', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests creating full backup
   */
  public function testBackupActionFullBackup() {
    self::addSWAccountIfNecessary();
    $NW = self::getMasterNodeWorx();

    $input = array( 'input' => array( 'domains' => array( self::SW_DOMAIN ),
                                      'email'   => '',
                                      'type'    => 'full_backup' ) );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'backup', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests assigned vars of manage action (when backups exists)
   */
  public function testManageActionAssignedVars() {
    $NW = self::getMasterNodeWorx();

    $input = array( 'account' => self::SW_DOMAIN );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'manage', $input, $NW );
    $this->assertArrayHasKey( 'iw_backups', $result );
  }

  /**
   * tests restore action (restores the first backup it can find)
   *
   * assumes there is one to restore.
   */
  public function testRestoreAction() {
    $NW = self::getMasterNodeWorx();

    $i = 0;
    do {
      // give it 1 minute...
      sleep( 1 );
      $filenames = $this->_getCompleteBackupFileNames();
      $first     = $filenames[0];
    } while( $i++ < 60 && $first != '' );
    $input = array( 'file'   => $first,
                    'domain' => self::SW_DOMAIN );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'restore', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * Tests the restoration of all backups
   */
  public function testRestoringAllBackups() {
    $NW = self::getMasterNodeWorx();

    $filenames = $this->_getCompleteBackupFileNames();
    $overall_result = true;

    foreach( $filenames as $first ) {
      $input = array( 'file'   => $first,
                      'domain' => self::SW_DOMAIN );

      $result = IW::FC()->routeFromPHP( self::CTRL, 'restore', $input, $NW );
      $overall_result = $result && $overall_result;
    }
    $this->assertTrue( $overall_result );
  }

  /**
   * tests deleting backups (deletes all it can find)
   */
  public function testDeleteActionReturnsTrue() {
    $NW = self::getMasterNodeWorx();

    $filenames = $this->_getCompleteBackupFileNames();

    $input = array( 'domain'  => self::SW_DOMAIN,
                    'backups' => $filenames );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests the vpopmail user directory hashing doesn't get screwed up on mail
   * restore
   */
  public function testVpopmailDirectoryHashingRestore() {
    $this->deleteSWAccountIfNecessary();
    self::addSWAccountIfNecessary();
    $this->_add300Mailboxes();
    $this->_delete300Mailboxes();
    $this->_addOneMailbox();
    $this->_addADummyMessageFileToInbox();
    $this->_createBackupInTmp();
    $this->deleteSWAccount();
    $this->_restoreBackup();
    $this->_checkMailboxLocation();
  }

  /**
   * helper
   */
  private function _add300Mailboxes() {
    $range  = range( 1, 300 );
    $domain = self::SW_DOMAIN;
    foreach( $range as $num ) {
      $cmd = "/var/vpopmail/bin/vadduser user{$num}@{$domain} password";
      IWorxExec::exec( $cmd );
    }
  }

  /**
   * helper
   */
  private function _delete300Mailboxes() {
    $range  = range( 1, 300 );
    $domain = self::SW_DOMAIN;
    foreach( $range as $num ) {
      $cmd = "/var/vpopmail/bin/vdeluser user{$num}@{$domain}";
      IWorxExec::exec( $cmd );
    }
  }

  /**
   * helper
   */
  private function _addOneMailbox() {
    $domain = self::SW_DOMAIN;
    $cmd = "/var/vpopmail/bin/vadduser testuser@{$domain} password";
    IWorxExec::exec( $cmd );
  }

  /**
   * helper
   */
  private function _addADummyMessageFileToInbox() {
    $domain = self::SW_DOMAIN;
    $cmd = "/bin/echo TESTING | mail testuser@{$domain}";
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_PIPE );
  }

  /**
   * helper
   */
  private function _createBackupInTmp() {
    $backup = Ini::get( 'iworx.bin', 'backup' );
    $domain = self::SW_DOMAIN;
    $tmp    = Ini::get( 'dir', 'tmp' ) . '/mailbackuptest';
    mkdir( $tmp );

    $cmd = "{$backup} --domains {$domain} --output-dir {$tmp}";
    IWorxExec::exec( $cmd, $result, $retval );
    $this->assertEquals( 0,
                         $retval,
                         'making backup failed: ' . implode( "\n", $result ) );
  }

  /**
   * helper
   */
  private function _restoreBackup() {
    $import = Ini::get( 'iworx.bin', 'import' );
    $domain = self::SW_DOMAIN;
    $tmp    = Ini::get( 'dir', 'tmp' ) . '/mailbackuptest';
    $array  = glob( "{$tmp}/{$domain}*.tgz" );
    $file   = $array[ 0 ];

    $this->assertTrue( is_file( $file ) );

    $cmd = "{$import} --archive={$file} --control-panel=siteworx";
    IWorxExec::exec( $cmd, $result, $retval );
    $this->assertEquals( 0,
                         $retval,
                         'restore backup failed: ' . implode( "\n", $result ) );

    IWorxFileSys::rm( $tmp, $recurse = true, $force = true );
  }

  /**
   * helper
   */
  private function _checkMailboxLocation() {
    $SW       = $this->getSWAccount();
    $dir      = $SW->getIncomingMailDriver()->getUserDir( 'testuser' );
    $hostname = IWorxSysInfo::hostname();
    $array    = glob( "{$dir}/Maildir/new/*{$hostname}*" );
    $this->assertTrue( is_file( $array[ 0 ] ), 'mail message not found' );
  }

  /**
   * helper to get existing backup file names under the account
   */
  private function _getCompleteBackupFileNames() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'account' => self::SW_DOMAIN );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'manage', $input, $NW );

    $filenames = array();
    foreach( $result['iw_backups'][ self::SW_DOMAIN ]['files'] as $file_array ) {
      if( $file_array['complete'] === true ) {
        $filenames[] = $file_array['filename'];
      }
    }
    return $filenames;
  }
}
?>