<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: CronTest.php,v 1.8 2009/09/16 08:15:52 poehler Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Cron
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_CronTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Cron';

  /**
   * index test
   */
  public function testIndexAssignedVars() {
    self::addSWAccountIfNecessary();
    $NodeWorx = self::getMasterNodeWorx();

    $data = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NodeWorx );

    $this->assertArrayHasKey( 'iw_job_actions', $data );
    $this->assertArrayHasKey( 'delete', $data['iw_job_actions'] );

    $this->assertArrayHasKey( 'iw_form', $data );
    $this->assertArrayHasKey( 'cron_options', $data['iw_form'] );

    $OptionsForm = $data['iw_form']['cron_options'];

    $this->assertTrue( $OptionsForm Instanceof Form_NW_Cron_Options );

    $this->assertArrayHasKey( 'cron_options', $data['iw_form'] );
    $this->assertArrayHasKey( 'user', $data['iw_cron'] );
    $this->assertArrayHasKey( 'crontab', $data['iw_cron'] );
  }

  /**
   * test - start from scratch
   */
  public function testResetCron() {
    $cmd = '/usr/bin/crontab -r -u ' . self::SW_UNIQNAME;
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::KILL_STDERR );
  }

  /**
   * test - start from scratch
   */
  public function testSetCronOptionDefaults() {
    $NodeWorx = self::getMasterNodeWorx();

    $input  = array( 'user'  => self::SW_UNIQNAME,
                     'shell' => '/bin/bash' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'options', $input, $NodeWorx );

    $this->assertTrue( $result );
  }


  /**
   * add action test
   */
  public function testSimpleAddAction() {
    $this->addSWAccountIfNecessary();

    $result = $this->_addCron( 'simple' );
    $this->assertTrue( $result );

    $cmd = '/usr/bin/crontab -l -u ' . self::SW_UNIQNAME;
    exec( $cmd, $result, $retval );
    $this->assertEquals( '* * * * * echo hi', $result[ 3 ] );
  }

  /**
   * tests adding with the advanced editor
   */
  public function testAdvancedAddAction() {
    $this->addSWAccountIfNecessary();

    $result = $this->_addCron( 'advanced' );
    $this->assertTrue( $result );

    $cmd = '/usr/bin/crontab -l -u ' . self::SW_UNIQNAME;
    exec( $cmd, $result, $retval );
    $this->assertEquals( '* * * * * echo hi', $result[ 3 ] );
    $this->assertEquals( '* * * * * echo hi', $result[ 4 ] );
  }

  /**
   * tests edit
   */
  public function testEditAction() {
    $this->addSWAccountIfNecessary();

    $NodeWorx    = self::getMasterNodeWorx();
    $SiteWorx    = $this->getSWAccount();
    $user        = $SiteWorx->getUniqname();
    $LineNumbers = new DataSource_Cron_LineNumbers( $user );

    $input = array( 'user'      => $user,
                    'job'       => $LineNumbers->getFirstValue(),
                    'minute'    => '*',
                    'hour'      => '*',
                    'day'       => '*',
                    'month'     => '*',
                    'dayofweek' => '*',
                    'script'    => 'echo hi' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NodeWorx );

    $this->assertTrue( $result );

    $cmd = '/usr/bin/crontab -l -u ' . self::SW_UNIQNAME;
    exec( $cmd, $result, $retval );
    $this->assertEquals( '* * * * * echo hi', $result[ 3 ] );
    $this->assertEquals( '* * * * * echo hi', $result[ 4 ] );
  }

  /**
   * tests delete
   */
  public function testDeleteAction() {
    $this->addSWAccountIfNecessary();

    $NodeWorx    = self::getMasterNodeWorx();
    $SiteWorx    = $this->getSWAccount();
    $user        = $SiteWorx->getUniqname();
    $LineNumbers = new DataSource_Cron_LineNumbers( $user );
    $input       = array( 'user' => $user,
                          'jobs' => $LineNumbers->getValues() );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NodeWorx );

    $this->assertTrue( $result );
  }


  /**
   * adds a cron job to the test SiteWorx user
   *
   * @param string $interface
   * @return boolean|array
   */
  private function _addCron( $interface = 'simple' ) {
    $NodeWorx = self::getMasterNodeWorx();
    $SiteWorx = $this->getSWAccount();

    $input = array( 'interface' => $interface,
                    'user'      => $SiteWorx->getUniqname(),
                    'minute'    => '*',
                    'hour'      => '*',
                    'day'       => '*',
                    'month'     => '*',
                    'dayofweek' => '*',
                    'script'    => 'echo hi' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NodeWorx );

    return $result;
  }

}
?>