<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: DnsZoneTest.php,v 1.4 2009/05/02 03:06:27 poehler Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_DnsZone
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_DnsZoneTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_DnsZone';

  const DNS_ZONE        = 'zone-added-by-test.com';
  const DUMMY_IP        = '1.2.3.4';
  const DUMMY_CONTACT   = 'nobody@nowhere.com';
  const DUMMY_CONTACT_2 = 'somebodyelse@nowhere.com';

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_zones', $result );
  }

  /**
   * test
   */
  public function testAddActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domain' => self::DNS_ZONE,
                     'ipaddr' => self::DUMMY_IP,
                     'contact' => self::DUMMY_CONTACT );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testEditActionReturnsTrue() {
    $this->assertTrue( Zone::exists( 'domain = ' . q( self::DNS_ZONE ) ) );
    $Zone    = new Zone( 'domain = ' . q( self::DNS_ZONE ) );
    $zone_id = $Zone->getId();
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'zone_id' => $zone_id,
                     'contact' => self::DUMMY_CONTACT_2 );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testDeleteActionReturnsTrue() {
    $this->assertTrue( Zone::exists( 'domain = ' . q( self::DNS_ZONE ) ) );
    $Zone    = new Zone( 'domain = ' . q( self::DNS_ZONE ) );
    $zone_id = $Zone->getId();
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'zone_id' => $zone_id );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testEditActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', array(), $NW );
    $this->assertIsFormErrorArray( $result );
  }

  /**
   * test
   */
  public function testDeleteActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $NW );
    $this->assertIsFormErrorArray( $result );
  }

  /**
   * test
   */
  public function testAddActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $NW );
    $this->assertIsFormErrorArray( $result );
  }
}