<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: FirewallTest.php,v 1.2 2009/09/01 21:22:10 jdaniel Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Firewall
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_FirewallTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Firewall';

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_firewall', $result );
    $this->assertArrayHasKey( 'iw_form', $result );

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'sob', $forms );
    $class_name = Form::NW_FIREWALL_START_ON_BOOT;
    $this->assertTrue( $forms['sob'] instanceof $class_name );

    $this->assertArrayHasKey( 'info', $forms );
    $class_name = Form::NW_FIREWALL_INFO;
    $this->assertTrue( $forms['info'] instanceof $class_name );

    $this->assertArrayHasKey( 'allowdeny', $forms );
    $class_name = Form::NW_FIREWALL_ALLOW_DENY_IPS;
    $this->assertTrue( $forms['allowdeny'] instanceof $class_name );
  }

  /**
   * test
   */
  public function testAddPortActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'addPort', array(), $NW );
    $this->assertType( 'array', $result );
    $this->assertEquals( '##LG_NOTHING_TO_DO##', $result[ 0 ] );
  }

  /**
   * test
   */
  public function testStartOnBootActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testAllowDenyIpsActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testAddingTrustedIpsWorks() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'trusted_ips' => '10.10.11.12' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testRemovingTrustedIpsWorks() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'trusted_ips' => '' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testAddedBlockedIpsWorks() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'blocked_ips' => '10.10.11.12' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testRemovingBlockedIpsWorks() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'blocked_ips' => '10.10.11.12' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateConfigActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateConfig', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'update', array(), $NW );
    $this->assertType( 'array', $result );
    $this->assertEquals( '##LG_NOTHING_TO_DO##', $result[ 0 ] );
  }
}
?>