<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: HttpTest.php,v 1.5 2009/09/22 20:50:59 jdaniel Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Http
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_HttpTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Http';

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_apache', $result );
    $this->assertArrayHasKey( 'iw_form', $result );
  }

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedForms() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'sob', $forms );
    $class_name = Form::NW_HTTP_START_ON_BOOT;
    $this->assertTrue( $forms['sob'] instanceof $class_name );

    $this->assertArrayHasKey( 'restart', $forms );
    $class_name = Form::NW_HTTP_AUTORESTART;
    $this->assertTrue( $forms['restart'] instanceof $class_name );

    $this->assertArrayHasKey( 'rrd', $forms );
    $class_name = Form::NW_HTTP_RRD_GRAPH;
    $this->assertTrue( $forms['rrd'] instanceof $class_name );

    $this->assertArrayHasKey( 'options', $forms );
    $class_name = Form::NW_HTTP_SERVER_OPTIONS;
    $this->assertTrue( $forms['options'] instanceof $class_name );
  }

  /**
   * tests toggling start on boot action
   */
  public function testStartOnBootActionToggleWorks() {
    $NW = self::getMasterNodeWorx();

    $sob_status = Driver::isServiceStartOnBoot( 'httpd' );
    $input      = array( 'startonboot' => ($sob_status) ? 0 : 1 );
    $result     = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', $input, $NW );
    $this->assertTrue( $result );
    $this->assertTrue( $sob_status != Driver::isServiceStartOnBoot( 'httpd' ) );

    Form::resetInstances();

    $sob_status = Driver::isServiceStartOnBoot( 'httpd' );
    $input      = array( 'startonboot' => ($sob_status) ? 0 : 1 );
    $result     = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', $input, $NW );
    $this->assertTrue( $result );
    $this->assertTrue( $sob_status != Driver::isServiceStartOnBoot( 'httpd' ) );
  }

  /**
   * tests toggling update rrd action
   */
  public function testUpdateRrdActionToggleWorks() {
    $NW = self::getMasterNodeWorx();

    $rrd    = Ini::get( 'rrd', 'http' );
    $input  = array( 'rrd' => ($rrd) ? 0 : 1 );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateRrd', $input, $NW );
    $this->assertTrue( $result );
    $this->assertTrue( $rrd != Ini::get( 'rrd', 'http' ) );

    $rrd    = Ini::get( 'rrd', 'http' );
    $input  = array( 'rrd' => ($rrd) ? 0 : 1 );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateRrd', $input, $NW );
    $this->assertTrue( $result );
    $this->assertTrue( $rrd != Ini::get( 'rrd', 'http' ) );
  }

  /**
   * tests toggling autorestart action
   */
  public function testAutorestartActionToggleWorks() {
    $NW = self::getMasterNodeWorx();

    $sim_info = SIM::getInfo();
    $auto     = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $input    = array( 'apache_autorestart' => ($auto) ? 0 : 1 );
    $result   = IW::FC()->routeFromPHP( self::CTRL, 'autorestart', $input, $NW );
    $this->assertTrue( $result );
    $sim_info = SIM::getInfo();
    $auto2    = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $this->assertTrue( $auto != $auto2 );

    $sim_info = SIM::getInfo();
    $auto     = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $input    = array( 'apache_autorestart' => ($auto) ? 0 : 1 );
    $result   = IW::FC()->routeFromPHP( self::CTRL, 'autorestart', $input, $NW );
    $this->assertTrue( $result );
    $sim_info = SIM::getInfo();
    $auto2    = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $this->assertTrue( $auto != $auto2 );
  }

  /**
   * tests apache module action with no input errors
   */
  public function testApacheModulesActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'apacheModules', array(), $NW );
    $this->assertType( 'array', $result );
    $this->assertTrue( $result[ 0 ] == '##LG_NO_MODULES_SELECTED##' );
  }

  /**
   * tests apache server options action with no input (still works)
   */
  public function testApacheUpdateActionWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * Tests that the keepalive setting turns on
   */
  public function testKeepaliveSettingTurnsOn() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'keepalive' => '1' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertTrue( $result );

    $http      = DriverManager::getDefaultDriver( 'http' )->getServerInfo();
    $keepalive = ($http['keepalive'] === 'off') ? '0' : '1';

    $this->assertEquals( 'on', $http['keepalive'] );
  }

  /**
   * Tests that the keepalive setting turns off
   */
  public function testKeepaliveSettingTurnsOff() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'keepalive' => '0' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertTrue( $result );

    $http      = DriverManager::getDefaultDriver( 'http' )->getServerInfo();
    $keepalive = ($http['keepalive'] === 'off') ? '0' : '1';

    $this->assertEquals( 'off', $http['keepalive'] );
  }

  /**
   * Tests that the force graceful restart turns on
   */
  public function testForceGracefulRestartTurnsOn() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'force_graceful' => '1' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertTrue( $result );

    $this->assertEquals( '1', Ini::get( Ini::APACHE, 'force_graceful' ) );
  }

  /**
   * Tests that force graceful restart turns off
   */
  public function testForceGracefulRestartTurnsOff() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'force_graceful' => '0' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertTrue( $result );

    $this->assertEquals( '0', Ini::get( Ini::APACHE, 'force_graceful' ) );
  }

  /*
  TODO: do something for these actions

  public function testSyntaxAction() {

  }

  public function testServerStatusAction() {

  }

  public function testServerInfoAction() {

  }

  public function testPhpinfoAction() {

  }
  */
}
?>