<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: MailMtaTest.php,v 1.6 2009/09/16 07:46:23 poehler Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_MailMta
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_MailMtaTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_MailMta';

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_mta', $result );
    $this->assertArrayHasKey( 'iw_form', $result );

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'alt', $forms );
    $class_name = Form::NW_MAILMTA_ALT_SMTP_SETTINGS;
    $this->assertTrue( $forms['alt'] instanceof $class_name );

    $this->assertArrayHasKey( 'rrd', $forms );
    $class_name = Form::NW_MAILMTA_RRD_GRAPH;
    $this->assertTrue( $forms['rrd'] instanceof $class_name );

    $this->assertArrayHasKey( 'opts', $forms );
    $class_name = Form::NW_MAILMTA_SERVER_OPTIONS;
    $this->assertTrue( $forms['opts'] instanceof $class_name );

    $this->assertArrayHasKey( 'bounce', $forms );
    $class_name = Form::NW_MAILMTA_BOUNCE_OPTIONS;
    $this->assertTrue( $forms['bounce'] instanceof $class_name );

    $this->assertArrayHasKey( 'in', $forms );
    $class_name = Form::NW_MAILMTA_INBOUND_SMTP_OPTIONS;
    $this->assertTrue( $forms['in'] instanceof $class_name );

    $this->assertArrayHasKey( 'out', $forms );
    $class_name = Form::NW_MAILMTA_OUTBOUND_SMTP_OPTIONS;
    $this->assertTrue( $forms['out'] instanceof $class_name );
  }

  /**
   * test
   */
  public function testUpdateBounceActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateBounce', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * Gets the double bounce input
   *
   * @return array
   */
  private function _getDoubleBounceInput() {
    $mta = DriverManager::getDefaultDriver( 'omail' )->getServerInfo();

    $bouncefrom    = $mta['controls']['bouncefrom'][ 0 ];
    $bouncehst     = $mta['controls']['bouncehost'][ 0 ];
    $doublebounce  = $mta['controls']['doublebounce'];
    $dblbounceto   = $mta['controls']['doublebounceto'][ 0 ];
    $dblbouncehost = $mta['controls']['doublebouncehost'][ 0 ];

    $input = array( 'bounce_from' => $bouncefrom,
                    'bounce_host' => $bouncehst,
                    'double_bounce_messages' => $doublebounce,
                    'double_bounce_to' => $dblbounceto,
                    'double_bounce_host' => $dblbouncehost );

    return $input;
  }

  /**
   * Tests the enabling of double bounce
   */
  public function testDoubleBounceEnable() {
    $NW                              = self::getMasterNodeWorx();
    $input                           = $this->_getDoubleBounceInput();
    $input['double_bounce_messages'] = '1';
    $input['double_bounce_to']       = 'postmaster';

    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateBounce', $input, $NW );
    $this->assertTrue( $result );

    $mta = DriverManager::getDefaultDriver( 'omail' )->getServerInfo( true );

    $this->assertEquals( '1', $mta['controls']['doublebounce'] );
    $this->assertEquals( 'postmaster', $mta['controls']['doublebounceto'][0] );
  }

  /**
   * Tests that if the user sets the double bounce to disabled but sets the
   * doublebounceto option to doublebounce that it will actually disable.
   */
  public function testDoubleBounceEnbableAlwaysEnables() {
    $NW                              = self::getMasterNodeWorx();
    $input                           = $this->_getDoubleBounceInput();
    $input['double_bounce_messages'] = '1';
    $input['double_bounce_to']       = 'doublebounce';

    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateBounce', $input, $NW );
    $this->assertTrue( $result );

    $mta = DriverManager::getDefaultDriver( 'omail' )->getServerInfo( true );

    $this->assertEquals( '1', $mta['controls']['doublebounce'] );
    $this->assertNotEquals( 'doublebounce', $mta['controls']['doublebounceto'][0] );
  }

  /**
   * Tests the disabling of double bounce
   */
  public function testDoubleBounceDisable() {
    $NW                              = self::getMasterNodeWorx();
    $input                           = $this->_getDoubleBounceInput();
    $input['double_bounce_messages'] = 0;

    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateBounce', $input, $NW );
    $this->assertTrue( $result );

    $mta = DriverManager::getDefaultDriver( 'omail' )->getServerInfo( true );

    $this->assertEquals( 0, $mta['controls']['doublebounce'] );
    $this->assertEquals( 'doublebounce', $mta['controls']['doublebounceto'][0] );
  }

  /**
   * test
   */
  public function testUpdateGeneralActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateGeneral', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateInboundActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateInbound', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateOutboundActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateOutbound', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateRrdActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'rrd' => 0 );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateRrd', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function updateSmtp2portActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateSmtp2port', array(), $NW );
    $this->assertTrue( $result );
  }
}
?>