<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: MailSpamTest.php,v 1.3 2009/04/01 15:07:28 tcollege Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_MailSpam
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_MailSpamTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_MailSpam';

  const PREF_1 = 'dcc_timeout';
  const PREF_2 = 'dcc_body_max';
  const PREF_3 = 'whatever';

  const VALUE_1 = '123';
  const VALUE_2 = 'asdf';
  const VALUE_3 = 'boogers';

  /**
   * Gets the index data about the preference
   *
   * @param string $prefname
   * @return array|false
   */
  private function _getPreferenceData( $prefname = self::PREF_1 ) {
    $Spm   = new SpamPrefManager();
    $prefs = $Spm->getFlatPrefsArray();

    $preference = false;

    foreach( $prefs as $pref ) {
      if( $preference === false ) {
        if( $pref['preference'] === $prefname ) {
          $preference = $pref;
        }
      }
    }

    return $preference;
  }

  /**
   * Deletes existing preferences if they'll conflict
   */
  public function testSetup() {
    $Spm   = new SpamPrefManager();
    $prefs = $Spm->getFlatPrefsArray();

    $ids_to_delete = array();

    foreach( $prefs as $pref ) {
      if( $pref['preference'] === self::PREF_1 ||
          $pref['preference'] === self::PREF_2 ||
          $pref['preference'] === self::PREF_3 ) {
        $ids_to_delete[] = $pref['prefid'];
      }
    }

    // This sucks a bit...calling the controller we're trying to test
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'pref_id' => $ids_to_delete );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'deleteAdvanced', $input, $NW );
  }

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_spam', $result );
    $this->assertArrayHasKey( 'iw_form', $result );

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'auto', $forms );
    $class_name = Form::NW_MAILSPAM_AUTORESTART;
    $this->assertTrue( $forms['auto'] instanceof $class_name );

    $this->assertArrayHasKey( 'sob', $forms );
    $class_name = Form::NW_MAILSPAM_START_ON_BOOT;
    $this->assertTrue( $forms['sob'] instanceof $class_name );

    $this->assertArrayHasKey( 'opts', $forms );
    $class_name = Form::NW_MAILSPAM_OPTIONS;
    $this->assertTrue( $forms['opts'] instanceof $class_name );

    $this->assertArrayHasKey( 'bayes', $forms );
    $class_name = Form::NW_MAILSPAM_BAYES;
    $this->assertTrue( $forms['bayes'] instanceof $class_name );

    $this->assertArrayHasKey( 'access', $forms );
    $class_name = Form::NW_MAILSPAM_ACCESS_LISTS;
    $this->assertTrue( $forms['access'] instanceof $class_name );
  }

  /**
   * test
   */
  public function testStartOnBootActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateOptionsActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateOptions', array(), $NW );
    $this->assertTrue( $result );

  }

  /**
   * test
   */
  public function testUpdateBayesActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateBayes', array(), $NW );
    $this->assertTrue( $result );

  }

  /**
   * test
   */
  public function testUpdateListsActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateLists', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testAutoRestartActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'autoRestart', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * Tests that addAdvancedAction fails without input
   */
  public function testAddAdvancedFailsWithoutInput() {
    $NW     = self::getMasterNodeWorx();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'addAdvanced', array(), $NW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that addAdvancedAction returns true
   */
  public function testAddAdvancedReturnsTrue() {
    $NW     = self::getMasterNodeWorx();

    $input = array( 'preference' => self::PREF_1,
                    'value'      => self::VALUE_1 );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'addAdvanced', $input, $NW );

    $this->assertTrue( $result );
  }

  /**
   * Checks for the added preference
   */
  public function testAddAdvancedWorked() {
    $pref = $this->_getPreferenceData();

    $this->assertEquals( self::VALUE_1, $pref['value'] );
  }

  /**
   * Tests that editAdvancedAction fails without input
   */
  public function testEditAdvancedFailsWithoutInput() {
    $NW     = self::getMasterNodeWorx();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'editAdvanced', array(), $NW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that editAdvancedAction returns true
   */
  public function testEditAdvancedReturnsTrue() {
    $pref = $this->_getPreferenceData();
    $id   = $pref['prefid'];
    $NW   = self::getMasterNodeWorx();

    $input = array( 'pref_id' => $id,
                    'value'   => self::VALUE_2 );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'editAdvanced', $input, $NW );

    $this->assertTrue( $result );
  }

  /**
   * Verifies that the preference was edited successfully
   */
  public function testEditAdvancedWorked() {
    $preference = $this->_getPreferenceData();

    $this->assertEquals( self::VALUE_2, $preference['value'] );
  }

  /**
   * Tests that deleteAdvancedAction fails without input
   */
  public function testDeleteAdvancedFailsWithoutInput() {
    $NW     = self::getMasterNodeWorx();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'deleteAdvanced', array(), $NW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAdvancedAction returns true
   */
  public function testDeleteAdvancedReturnsTrue() {
    $pref = $this->_getPreferenceData();
    $id   = $pref['prefid'];
    $NW   = self::getMasterNodeWorx();

    $input = array( 'pref_id' => $id );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'deleteAdvanced', $input, $NW );

    $this->assertTrue( $result );
  }

  /**
   * Checks that the preference was deleted
   */
  public function testDeleteAdvancedWorked() {
    $pref = $this->_getPreferenceData();

    $this->assertFalse( $pref );
  }

  /**
   * Tests deleting multiple prefs at once
   */
  public function testDeleteAdvancedMultiple() {
    $NW     = self::getMasterNodeWorx();

    $input = array( 'preference' => self::PREF_2,
                    'value'      => self::VALUE_2 );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'addAdvanced', $input, $NW );

    $input = array( 'preference' => self::PREF_3,
                    'value'      => self::VALUE_3 );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'addAdvanced', $input, $NW );

    $pref_2 = $this->_getPreferenceData( self::PREF_2 );
    $pref_3 = $this->_getPreferenceData( self::PREF_3 );

    $ids = array( $pref_2['prefid'], $pref_3['prefid'] );

    $input = array( 'pref_id' => $ids );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'deleteAdvanced', $input, $NW );

    $this->assertTrue( $result );
  }

  /**
   * Checks that the preference was deleted
   */
  public function testDeleteAdvancedMultipleWorked() {
    $pref = $this->_getPreferenceData( self::PREF_2 );

    $this->assertFalse( $pref );

    $pref = $this->_getPreferenceData( self::PREF_3 );

    $this->assertFalse( $pref );
  }
}

?>