<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 5, 2009
 * @id $Id: MysqlTest.php,v 1.5 2009/08/21 02:30:17 poehler Exp $
 */

/**
 * Controller test for Ctrl_Nodeworx_Mysql
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class Ctrl_Nodeworx_MysqlTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Mysql';

  /**
   * Makes sure index routes properly when multiple_servers is disabled
   */
  public function testIndexWithSingleServer() {
    $old_setting = Ini::get( 'mysql', 'multiple_servers' );

    $NodeWorx = self::getMasterNodeWorx();
    Ini::set( 'mysql', 'multiple_servers', '0' );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'index',
                                      array(),
                                      $NodeWorx );

    Ini::set( 'mysql', 'multiple_servers', $old_setting );

    $this->assertArrayHasKey( 'iw_server', $result );
    $this->assertArrayHasKey( 'iw_form', $result );
    $this->assertArrayHasKey( 'iw_mysql', $result );
    $this->assertArrayHasKey( 'iw_process_actions', $result );

    $forms = $result['iw_form'];
    $this->assertArrayHasKey( 'rrd', $forms );
    $this->assertArrayHasKey( 'sob', $forms );
    $this->assertArrayHasKey( 'autorestart', $forms );
    $this->assertArrayHasKey( 'options', $forms );
    $this->assertArrayHasKey( 'rootpw', $forms );

    $this->assertArrayNotHasKey( 'nickname', $forms );

    $server_id = $result['iw_server']['id'];
    $Server = new MySQLServer( $server_id );
    $this->assertTrue( $Server->exists() );
    $this->assertTrue( $Server->isLocal() );
  }

  /**
   * checks what is going on in the index function with multiple
   * servers enabled
   */
  public function testIndexWithMultiServer() {
    $old_setting = Ini::get( 'mysql', 'multiple_servers' );

    $NodeWorx = self::getMasterNodeWorx();
    Ini::set( 'mysql', 'multiple_servers', '1' );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'index',
                                      array(),
                                      $NodeWorx );

    Ini::set( 'mysql', 'multiple_servers', $old_setting );

    $this->assertArrayHasKey( 'iw_servers', $result );
  }

  /**
   * checks what is going on in the config function with multiple
   * servers enabled
   */
  public function testConfigWithMultiServer() {
    $old_setting = Ini::get( 'mysql', 'multiple_servers' );

    $NodeWorx = self::getMasterNodeWorx();
    Ini::set( 'mysql', 'multiple_servers', '1' );

    $Server = MySQLServer::getLocalServer();
    $input  = array( 'server' => $Server->getNickname() );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'config',
                                      $input,
                                      $NodeWorx );

    Ini::set( 'mysql', 'multiple_servers', $old_setting );

    $this->assertArrayHasKey( 'iw_server', $result );
    $this->assertArrayHasKey( 'iw_form', $result );
    $this->assertArrayHasKey( 'iw_mysql', $result );
    $this->assertArrayHasKey( 'iw_process_actions', $result );

    $forms = $result['iw_form'];
    $this->assertArrayHasKey( 'rrd', $forms );
    $this->assertArrayHasKey( 'sob', $forms );
    $this->assertArrayHasKey( 'autorestart', $forms );
    $this->assertArrayHasKey( 'options', $forms );
    $this->assertArrayHasKey( 'rootpw', $forms );
    $this->assertArrayHasKey( 'nickname', $forms );

    $server_id = $result['iw_server']['id'];
    $Server    = new MySQLServer( $server_id );
    $this->assertTrue( $Server->exists() );
    $this->assertTrue( $Server->isLocal() );
  }

  /**
   * Now we test changing the nickname to something
   */
  public function testChangingNicknameActionReturnsTrue() {
    $old_name = $this->_getLocalName();
    $new_name = 'Local Server';

    $NodeWorx = self::getMasterNodeWorx();
    $input    = array( 'server'   => $this->_getLocalName(),
                       'nickname' => $new_name );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'nickname',
                                      $input,
                                      $NodeWorx );

    $this->assertTrue( $result );
    $this->assertEquals( $new_name, $this->_getLocalName() );

    // now we change back
    $input  = array( 'server'   => $this->_getLocalName(),
                     'nickname' => $old_name );
    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'nickname',
                                      $input,
                                      $NodeWorx );
    $this->assertTrue( $result );
    $this->assertEquals( $old_name, $this->_getLocalName() );
  }

  /**
   * add action test
   */
  public function testAddAction() {
    $this->markTestSkipped( 'test not written' );
  }

  /**
   * delete action test
   */
  public function testDeleteAction() {
    $this->markTestSkipped( 'test not written' );
  }

  /**
   * test
   */
  public function testConfigActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'server' => $this->_getLocalName() );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'config', $input, $NW );

    $this->assertArrayHasKey( 'iw_form', $result );
    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'rrd', $forms );
    $class_name = Form::NW_MYSQL_RRD_GRAPH;
    $this->assertTrue( $forms['rrd'] instanceof $class_name );

    $this->assertArrayHasKey( 'sob', $forms );
    $class_name = Form::NW_MYSQL_START_ON_BOOT;
    $this->assertTrue( $forms['sob'] instanceof $class_name );

    $this->assertArrayHasKey( 'autorestart', $forms );
    $class_name = Form::NW_MYSQL_AUTO_RESTART;
    $this->assertTrue( $forms['autorestart'] instanceof $class_name );

    $this->assertArrayHasKey( 'options', $forms );
    $class_name = Form::NW_MYSQL_OPTIONS;
    $this->assertTrue( $forms['options'] instanceof $class_name );
  }

  /**
   * tests that the root password function returns properly
   */
  public function testRootpassActionWontAcceptBlank() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'server' => $this->_getLocalName() );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'rootpass',
                                      $input,
                                      $NW );

    // we should have gotten a negative result here
    $this->assertTrue( $result !== true );
  }

  /**
   * tests to make sure the rootpass action won't accept bad passwords
   */
  public function testRootpassActionDoesntAcceptBadPasswords() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'server'           => $this->_getLocalName(),
                    'password'         => 'abc123',
                    'confirm_password' => 'thisshouldfail' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'rootpass', $input, $NW );
    $this->assertTrue( $result !== true );
  }

  /**
   * test
   */
  public function testRrdGraphActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'rrdGraph', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testStartOnBootActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testAutoRestartActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'autoRestart', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testOptionsActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'options', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * Gets the localserver nickname
   *
   * @return string
   */
  private function _getLocalName() {
    $Server = MySQLServer::getLocalServer();
    return $Server->getNickname();
  }

}
?>