<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: ResellerTest.php,v 1.4 2009/05/02 02:45:59 poehler Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Packages
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_ResellerTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Reseller';

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_resellers', $result );
    $this->assertArrayHasKey( 'iw_master_reseller', $result );
  }

  /**
   * tests add action with no input
   */
  public function testAddActionReturnsErrorWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $NW );
    $this->assertIsFormErrorArray( $result );
  }

  /**
   * tests add action with good input
   */
  public function testAddActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = $this->_getResellerInput();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );
    $this->assertTrue( $result, 'reseller add did not return true' );
  }

  /**
   * tests add action again (expecting fail)
   */
  public function testAddActionAgain() {
    $NW     = self::getMasterNodeWorx();
    $input  = $this->_getResellerInput();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );
    $this->assertIsFormErrorArray( $result, 'reseller add did not return true' );
  }

  /**
   * tests edit action returns true with no extra input
   */
  public function testEditActionReturnsTrueWithNoChanges() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'reseller_id' => self::getResellerNodeWorx()->getId() );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertTrue( $result, 'reseller edit did not return true' );
  }

  /**
   * Tests changing the user password to prevent regression
   */
  public function testChangePassword() {
    $NW = self::getMasterNodeWorx();
    $input  = array( 'reseller_id' => self::getResellerNodeWorx()->getId(),
                     'password'    => 'new_password',
                     'confirm_password' => 'new_password' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertTrue( $result, 'reseller edit did not return true' );

    $Reseller = self::getResellerNodeWorx();

    $this->assertTrue( $Reseller->login( $Reseller->getMasterUser()->getEmail(),
                                         'new_password' ),
                       $Reseller->getLastError() );
  }

  /**
   * tests delete action return true
   */
  public function testDeleteActionReturnsTrue() {
    $RSL = self::getResellerNodeWorx();
    $NW  = self::getMasterNodeWorx();
    $input = array( 'reseller_id' => $RSL->getId() );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );
    $this->assertTrue( $result, 'delete did not return true' );
  }

  /**
   * tests delete actually deleted the reseller
   */
  public function testDeleteActionWorked() {
    $RSL = self::getResellerNodeWorx();
    $this->assertEquals( '-1', $RSL->getId(), 'reseller delete did not delete' );
  }
}
?>