<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: ShellTest.php,v 1.1 2009/02/25 07:35:36 poehler Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Siteworx
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_ShellTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Shell';

  const SHELL        = '/bin/bash';
  const BASH_HISTORY = 'sample text goes here';

  /**
   * tests that assigned vars exists for index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_shell', $result );
  }

  /**
   * tests shell user password changing
   */
  public function testPasswordActionReturnsTrue() {
    self::addSWAccountIfNecessary();
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'user' => self::SW_UNIQNAME,
                     'password' => self::SW_PASS,
                     'confirm_password' => self::SW_PASS );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'password', $input, $NW );
    $this->assertTrue( $result, 'shell password changing did not return true' );
  }

  /**
   * tests result of shell user password changing
   */
  public function testPasswordActionWorked() {
    $shadow  = $this->_getShadowPassword();
    $salt    = substr( $shadow, 0, 12 );
    $md5pass = crypt( self::SW_PASS, $salt );
    $this->assertEquals( $shadow, $md5pass, 'Shell password failed to be set' );
  }

  /**
   * tests enabling shell user
   */
  public function testShellSiteworxUsersActionEnable() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'input' => array( 'action' => 'enable',
                                       'users' => array( self::SW_UNIQNAME ) ) );
    $action = 'ShellSiteworxUsers';
    $result = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );
    $this->assertTrue( $result, 'Enabling shell did not return true' );
  }

  /**
   * tests changing shell of shell user
   */
  public function testShellSiteworxUsersActionChangeShell() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'input' => array( 'action' => 'shell',
                                       'users' => array( self::SW_UNIQNAME ),
                                       self::SW_UNIQNAME =>
                                       array( 'shell' => '/bin/bash' ) ) );
    $action = 'shellSiteworxUsers';
    $result = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );
    $this->assertTrue( $result, 'Changing shell did not return true' );
  }

  /**
   * tests retrieval of shell user bash_history
   */
  public function testHistoryActionReturnsTrue() {
    $this->_addSomeShellUserHistory();
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'user' => self::SW_UNIQNAME );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'history', $input, $NW );
    $this->assertArrayHasKey( 'iw_printout', $result );
    $this->assertEquals( self::BASH_HISTORY, $result['iw_printout'] );
  }

  /**
   * tests disabling shell user
   */
  public function testShellSiteworxUsersActionDisable() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'input' => array( 'action' => 'disable',
                                       'users' => array( self::SW_UNIQNAME ) ) );
    $action = 'shellSiteworxUsers';
    $result = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );
    $this->assertTrue( $result, 'Disabling shell did not return true' );
  }

  /**
   * tests setting default shell
   */
  public function testShellOptionsActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'input' => array( 'default' => self::SHELL ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'shellOptions', $input, $NW );
    $this->assertTrue( $result, 'Setting default shell did not return true' );
  }

  /**
   * cleans up
   */
  public function testCleanup() {
    $this->deleteSWAccountIfNecessary();
  }

  /**
   * write some contents to shell user bash_history file
   */
  private function _addSomeShellUserHistory() {
    file_put_contents( '/home/addedbyt/.bash_history', self::BASH_HISTORY );
  }

  /**
   * gets the crypted password from the shadow file
   */
  private function _getShadowPassword() {
    $shadowfile = file( '/etc/shadow' );
    foreach( $shadowfile as $line ) {
      $line = trim( $line );
      list( $username, $password, $rest ) = split( ':', $line, 3 );
      if( $username == self::SW_UNIQNAME ) {
        break;
      }
    }
    return $password;
  }

}
?>