<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: SiteworxTest.php,v 1.11 2009/09/16 07:46:23 poehler Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Siteworx
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_SiteworxTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Siteworx';

  const NEW_NICKNAME = 'New Nickname Here';

  /**
   * Ensures that the favorite is not checked at the start
   */
  public function testSetup() {
    $this->deleteSWAccountIfNecessary(); // make sure we get a clean account
    $this->getSWAccount();
    $NW       = self::getMasterNodeWorx();
    $filters  = $this->_getDefaultFilters();
    $accounts = $NW->getAccountsSearch( $filters );

    if( $accounts[ 1 ][ self::SW_UNIQNAME ]['is_favorite'] === '1' ) {
      // Toggle it to off for the starting position
      $this->testToggleFavoriteActionReturnsTrue();
    }
  }

  /**
   * test
   */
  public function testAddAction() {
    $this->deleteSWAccountIfNecessary();
    $NW    = self::getMasterNodeWorx();
    $input = self::_getSWAccountInput();

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'add',
                                      $input,
                                      $NW );

    $this->assertTrue( $result );
    $SW = $this->getSWAccount();

    $this->assertEquals( $input['domainname'], $SW->getWorkingDomainName() );
    $this->assertEquals( $input['ipaddress'], $SW->getIP() );
    $this->assertEquals( $input['uniqname'], $SW->getUniqname() );
    $this->assertEquals( $input['nickname'], $SW->getWorkingUser()->getNickName() );
    $this->assertEquals( $input['email'], $SW->getWorkingUser()->getEmail() );
  }

  /**
   * test
   */
  public function addAddActionNoUniqname() {
    $this->deleteSWAccountIfNecessary();
    $NW    = self::getMasterNodeWorx();
    $input = self::_getSWAccountInput();

    unset( $input['uniqname'] );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'add',
                                      $input,
                                      $NW );

    $this->assertTrue( $result );
    $SW = $this->getSWAccount();

    $this->assertEquals( self::SW_UNIQNAME, $SW->getUniqname() );
  }

  /**
   * tests assigned variables from the index action
   */
  public function testIndexActionAssignedVars() {
    self::addSWAccountIfNecessary();
    $NW = self::getMasterNodeWorx();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );

    $this->assertArrayHasKey( 'iw_siteworx', $result );
    $this->assertArrayHasKey( 'iw_all_stor_used', $result );
    $this->assertArrayHasKey( 'iw_all_bw_used', $result );
    $this->assertGreaterThanOrEqual( 1, count( $result['iw_siteworx'] ) );
  }

  /**
   * tests toggle on favorite action
   */
  public function testToggleFavoriteActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domain' => self::SW_DOMAIN );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'toggleFavorite', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests toggle on favorite works
   */
  public function testToggleFavoriteActionOnWorked() {
    $NW       = self::getMasterNodeWorx();
    $filters  = $this->_getDefaultFilters();
    $accounts = $NW->getAccountsSearch( $filters );
    $this->assertEquals( '1', $accounts[ 1 ][ self::SW_UNIQNAME ]['is_favorite'] );
  }

  /**
   * tests toggle off favorite
   */
  public function testToggleFavoriteActionAgain() {
    $this->testToggleFavoriteActionReturnsTrue();
  }

  /**
   * tests toggle off favorite works
   */
  public function testToggleFavoriteActionOffWorked() {
    $NW       = self::getMasterNodeWorx();
    $filters  = $this->_getDefaultFilters();
    $accounts = $NW->getAccountsSearch( $filters );
    $this->assertEquals( '0', $accounts[ 1 ][ self::SW_UNIQNAME ]['is_favorite'] );
  }

  /**
   * tests edit action (no changes)
   */
  public function testEditActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domain' => self::SW_DOMAIN );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests that edit action didn't change anything
   */
  public function testEditActionChangedNothing() {
    Form::resetInstances();
    IW::Env()->addInput( 'domain', self::SW_DOMAIN );

    $Form = Form::getInstance( Form::NW_SITEWORX_EDIT );
    $this->assertTrue( $Form->validateForm() );

    $ip = self::_getFirstSharedIP();

    $this->assertEquals( self::SW_DOMAIN, $Form->getValue( 'domainname' ) );
    $this->assertEquals( $ip, $Form->getValue( 'ipaddress' ) );
    $this->assertEquals( self::SW_UNIQNAME, $Form->getValue( 'uniqname' ) );
    $this->assertEquals( self::SW_NICK, $Form->getValue( 'nickname' ) );
    $this->assertEquals( self::SW_EMAIL, $Form->getValue( 'email' ) );

    $options = Option::getOptionList();

    $unlimited = Ini::get( Ini::CONSTANTS, 'unlimited' );
    foreach( $options as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $input_name = substr( $opt['name'], 3 );
      $this->assertEquals( $unlimited, $Form->getValue( $input_name ) );
    }

    foreach( $options as $opt ) {
      $input_name = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      if( $input_name != 'OPT_SUPHP' && $input_name != 'OPT_SSI' ) {
        $this->assertEquals( 1, $Form->getValue( $input_name ) );
      } else {
        $this->assertEquals( 0, $Form->getValue( $input_name ) );
      }
    }
  }

  /**
   * tests edit action, changing nickname
   */
  public function testEditActionChangeNicknameReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domain' => self::SW_DOMAIN,
                     'nickname' => self::NEW_NICKNAME );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests edit action, nickname change worked
   */
  public function testEditActionChangeNicknameWorked() {
    Form::resetInstances();
    IW::Env()->addInput( 'domain', self::SW_DOMAIN );

    $Form = Form::getInstance( Form::NW_SITEWORX_EDIT );
    $this->assertTrue( $Form->validateForm() );

    $this->assertEquals( self::NEW_NICKNAME, $Form->getValue( 'nickname' ) );
  }

  /**
   * tests bulk deactivate action
   */
  public function testBulkdeactivateActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domains' => array( self::SW_DOMAIN ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'bulkdeactivate', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests results of bulk deactivate action
   */
  public function testBulkdeactivateActionWorked() {
    $NW       = self::getMasterNodeWorx();
    $filters  = $this->_getDefaultFilters();
    $accounts = $NW->getAccountsSearch( $filters );
    $this->assertEquals( 'inactive', $accounts[ 1 ][self::SW_UNIQNAME]['status'] );
  }

  /**
   * tests bulk activate action
   */
  public function testBulkactivateActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domains' => array( self::SW_DOMAIN ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'bulkactivate', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests results of bulk activate action
   */
  public function testBulkactivateActionWorked() {
    $NW       = self::getMasterNodeWorx();
    $filters  = $this->_getDefaultFilters();
    $accounts = $NW->getAccountsSearch( $filters );
    $this->assertEquals( 'active', $accounts[ 1 ][self::SW_UNIQNAME]['status'] );
  }

  /**
   * tests bulk delete action
   */
  public function testBulkdeleteActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domains' => array( self::SW_DOMAIN ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'bulkdelete', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests result of bulk delete action
   */
  public function testBulkdeleteActionWorked() {
    $NW       = self::getMasterNodeWorx();
    $filters  = $this->_getDefaultFilters();
    $accounts = $NW->getAccountsSearch( $filters );
    if( isset( $accounts[ 1 ] ) ) {
      $this->assertArrayNotHasKey( self::SW_UNIQNAME, $accounts[ 1 ] );
    } else {
      // it's ok
    }
  }

  /**
   * tests delete action
   */
  public function testDeleteActionReturnsTrue() {
    self::addSWAccountIfNecessary();
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domain' => self::SW_DOMAIN );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );
    $this->assertTrue( $result );
  }

  /**
   * tests results of delete action
   */
  public function testDeleteActionWorked() {
    $NW       = self::getMasterNodeWorx();
    $filters  = $this->_getDefaultFilters();
    $accounts = $NW->getAccountsSearch( $filters );
    if( isset( $accounts[ 1 ] ) ) {
      $this->assertArrayNotHasKey( self::SW_UNIQNAME, $accounts[ 1 ] );
    } else {
      // it's ok
    }
  }

  /**
   * tests delete action
   */
  public function testDeleteActionRemovesDatabases() {
    self::addSWAccountIfNecessary();
    $input  = array( 'name' => 'sampledb' );
    $SW     = $this->getSWAccount();
    $result = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_MysqlDb',
                                      'add',
                                      $input,
                                      $SW );
    $this->assertTrue( $result );

    $NW     = self::getMasterNodeWorx();
    $input  = array( 'domain' => self::SW_DOMAIN );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );
    $this->assertTrue( $result );

    $this->assertFalse( is_dir( '/var/lib/mysql/addedbyt_sampledb' ) );
  }

  /**
   * test
   */
  public function testAddOnDedicatedIPMakesIPUnavailable() {
    $NW    = self::getMasterNodeWorx();
    $input = self::_getSWAccountInput();

    $input['ipaddress'] = self::_getFirstAvailableDedicatedIP();

    $result = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                                      'add',
                                      $input,
                                      $NW );

    $available_ips = $NW->getUnusedIPList();
    $this->assertFalse( isset( $available_ips[ $input['ipaddress'] ] ),
                        'Dedicated IP used but still shown as available!' );

  }

  /**
   * return
   */
  public function testAddOnDedicatedIPMakesIPUnavailableCleanup() {
    $this->testDeleteActionReturnsTrue();

    $NW            = self::getMasterNodeWorx();
    $available_ips = $NW->getUnusedIPList();
    $dedi_ip       = self::_getFirstAvailableDedicatedIP();
    $this->assertTrue( isset( $available_ips[ $dedi_ip ] ),
                       'Dedicated IP not freed after siteworx account deleted!' );

  }

  /**
   * helper function to get the SiteWorx::getAccountsSearch() filters array
   */
  private function _getDefaultFilters() {
    return
      array( 'sort'       => 'id',
             'order'      => 'desc',
             'page'       => 1,
             'perpage'    => Ini::get( 'nodeworx', 'siteworx_accounts_perpage' ),
             'reseller'   => self::getMasterNodeWorx()->getId(),
             'search'     => '',
             'search_all' => 0 );
  }
}
?>