<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 * @id $Id: SshdTest.php,v 1.4 2009/09/03 14:26:43 jdaniel Exp $
 */

/**
 * Test Suite for Ctrl_Nodeworx_Sshd
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_SshdTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Sshd';

  /**
   * test assigned vars of index action
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $this->assertArrayHasKey( 'iw_sshd', $result );
    $this->assertArrayHasKey( 'iw_shell', $result );
    $this->assertArrayHasKey( 'iw_form', $result );

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'autorestart', $forms );
    $class_name = Form::NW_SSHD_AUTO_RESTART;
    $this->assertTrue( $forms['autorestart'] instanceof $class_name );

    $this->assertArrayHasKey( 'rrd', $forms );
    $class_name = Form::NW_SSHD_RRD_GRAPH;
    $this->assertTrue( $forms['rrd'] instanceof $class_name );

    $this->assertArrayHasKey( 'startonboot', $forms );
    $class_name = Form::NW_SSHD_START_ON_BOOT;
    $this->assertTrue( $forms['startonboot'] instanceof $class_name );

    $this->assertArrayHasKey( 'options', $forms );
    $class_name = Form::NW_SSHD_OPTIONS;
    $this->assertTrue( $forms['options'] instanceof $class_name );
  }

  /**
   * test
   */
  public function testShellSessionsActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'shellSessions', array(), $NW );
    $this->assertType( 'array', $result );
    $this->assertEquals( '##LG_NO_USERS_SELECTED##', $result[ 0 ] );
  }

  /**
   * test
   */
  public function testStartOnBootActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateAutorestartActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'updateAutorestart',
                                      array(),
                                      $NW );
    $this->assertTrue( $result );
  }

  /**
   * Tests to make sure enabing autorestart works via the controller
   *
   * Test for bug #1268
   */
  public function testAutorestartEnablingWorks() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'sshd_autorestart' => '1' );

    IW::FC()->routeFromPHP( self::CTRL,
                            'updateAutorestart',
                            $input,
                            $NW );

    $sim_info = SIM::getInfo();
    $this->assertTrue( $sim_info['services']['ssh'] );
  }

  /**
   * Tests to make sure disabling autorestart works via the controller
   *
   * Test for bug #1268
   */
  public function testAutorestartDisablingWorks() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'sshd_autorestart' => '0' );

    IW::FC()->routeFromPHP( self::CTRL,
                            'updateAutorestart',
                            $input,
                            $NW );

    $sim_info = SIM::getInfo();
    $this->assertFalse( $sim_info['services']['ssh'] );
  }


  /**
   * test
   */
  public function testUpdateRrdActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateRrd', array(), $NW );
    $this->assertTrue( $result );
  }

  /**
   * test
   */
  public function testUpdateConfigActionReturnsTrueWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateConfig', array(), $NW );
    $this->assertTrue( $result );
  }


}
?>