<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jun 23, 2009
 * @id $Id: BackupTest.php,v 1.7 2009/09/16 07:46:21 poehler Exp $
 */

/**
 * Tests for Ctrl_Siteworx_Backup
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_BackupTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_Backup';

  /**
   * checks to make sure actions are correct
   */
  public function testIndexManageActions() {
    $data = $this->getSWIndexData( self::CTRL );

    $actions = $data['iw_manage_actions'];
    $this->assertArrayHasKey( 'delete', $actions );
  }

  /**
   * tests to make sure the domain is correct
   */
  public function testIndexBackupDomain() {
    $SiteWorx        = $this->getSWAccount();
    $expected_domain = $SiteWorx->getMasterDomainName();

    $data = $this->getSWIndexData( self::CTRL );

    $domain = $data['iw_backup_domain'];
    $this->assertEquals( $expected_domain, $domain );
  }

  /**
   * full backup to the default directory
   */
  public function testCreateFullBackupToDefaultDir() {
    $SiteWorx = $this->getSWAccount();

    $input = array( 'type'     => 'full',
                    'location' => 'siteworx',
                    'email'    => $SiteWorx->getWorkingUser()->getEmail()
                    );

    $result = $this->_createBackup( $input );

    $this->assertTrue( $result );
  }

  /**
   * partial backup to the default directory
   */
  public function testPartialBackupToDefaultDir() {
    $SiteWorx = $this->getSWAccount();

    $input = array( 'type'     => 'partial',
                    'options'  => array( 'web', 'mail', 'dbs' ),
                    'location' => 'siteworx',
                    'email'    => $SiteWorx->getWorkingUser()->getEmail()
                    );

    $result = $this->_createBackup( $input );

    $this->assertTrue( $result );
  }

  /**
   * structure backup to the default directory
   */
  public function testStructureBackupToDefaultDir() {
    $SiteWorx = $this->getSWAccount();

    $input = array( 'type'     => 'structure',
                    'location' => 'siteworx',
                    'email'    => $SiteWorx->getWorkingUser()->getEmail()
                    );

    $result = $this->_createBackup( $input );

    $this->assertTrue( $result );
  }

  /**
   * schedule backup to the default directory
   */
  public function testScheduleAction() {
    $SiteWorx = $this->getSWAccount();

    $input = array( 'frequency' => 'daily',
                    'hour'      => '1',
                    'type'     => 'structure',
                    'location' => 'siteworx',
                    'email'    => $SiteWorx->getWorkingUser()->getEmail() );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'schedule',
                                      $input,
                                      $SiteWorx );

    $this->assertTrue( $result );

    $data = $this->getSWIndexData( self::CTRL );
    $this->assertArrayHasKey( 'iw_backup_cronjob', $data );
    $this->assertTrue( is_array( $data['iw_backup_cronjob'] ) === true );
  }

  /**
   * tests deleting backups
   */
  public function testDeleteAction() {
    $SiteWorx = $this->getSWAccount();
    $domain   = $SiteWorx->getMasterDomainName();

    // create a backup, just to make sure there is atleast one
    $input = array( 'type'     => 'structure',
                    'location' => 'siteworx',
                    'email'    => $SiteWorx->getWorkingUser()->getEmail()
                    );
    $this->_createBackup( $input );

    // delete all backups
    $DS_Backups = new DataSource_Backup_Files( $domain );
    $input      = array( 'backups' => $DS_Backups->getValues() );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'delete',
                                      $input,
                                      $SiteWorx );

    $this->assertTrue( $result );
  }

  /**
   * tests deleting the cron backup
   */
  public function testDeleteCronAction() {
    $SiteWorx = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'deleteCron',
                                      array(),
                                      $SiteWorx );

    $this->assertTrue( $result );
  }

  /**
   * tests restoring the backup action.
   */
  public function testRestoreAction() {
    $SiteWorx = $this->getSWAccount();
    $domain   = $SiteWorx->getMasterDomainName();

    // delete all backups
    $DS_Backups    = new DataSource_Backup_Files( $domain );
    $backup_values = $DS_Backups->getValues();
    if( !empty( $backup_values ) ) {
      $input = array( 'backups' => $DS_Backups->getValues() );
      IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SiteWorx );
    }

    $input = array( 'type'     => 'partial',
                    'options'  => array( 'web', 'mail', 'dbs' ),
                    'location' => 'siteworx' );
    $this->_createBackup( $input );

    $DS_Backups    = new DataSource_Backup_PartialFiles( $domain );
    $backup_values = $DS_Backups->getValues();

    $input = array( 'file' => $backup_values[0] );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'restore',
                                      $input,
                                      $SiteWorx );

    $this->assertTrue( $result );
  }

  /**
   * creates a backup after waiting to make sure one isn't already running
   *
   * @param array $input
   * @return boolean|array
   */
  private function _createBackup( array $input ) {
    $SiteWorx = $this->getSWAccount();

    $this->_waitForBackupToFinish();

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'create',
                                      $input,
                                      $SiteWorx );

    $this->_waitForBackupToFinish();

    return $result;

  }

  /**
   * Waits for the backup to finish.
   */
  private function _waitForBackupToFinish() {
    $SiteWorx = $this->getSWAccount();
    $domain   = $SiteWorx->getMasterDomainName();

    do {
      sleep( 1 );
    } while( IWorxBR::isBackupRunning( $domain ) === true );
  }
}
?>