<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jun 25, 2009
 * @id $Id: CronTest.php,v 1.5 2009/08/21 02:29:39 poehler Exp $
 */

/**
 * Tests for Ctrl_Siteworx_Cron
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class Ctrl_Siteworx_CronTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_Cron';

  /**
   * tests index actions
   */
  public function testIndexAssignedIWJobAction() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertArrayHasKey( 'iw_job_actions', $data );
    $this->assertArrayHasKey( 'delete', $data['iw_job_actions'] );
  }

  /**
   * tests index actions
   */
  public function testIndexAssignedIWForm() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertArrayHasKey( 'iw_form', $data );
    $this->assertArrayHasKey( 'cron_options', $data['iw_form'] );
  }

  /**
   * tests index actions
   */
  public function testIndexAssignedFormsProperObjects() {
    $data        = $this->getSWIndexData( self::CTRL );
    $OptionsForm = $data['iw_form']['cron_options'];

    $this->assertTrue( $OptionsForm Instanceof Form_SW_Cron_Options );
  }

  /**
   * tests index actions
   */
  public function testIndexAssignedIWCron() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertArrayHasKey( 'cron_options', $data['iw_form'] );
    $this->assertArrayHasKey( 'user', $data['iw_cron'] );
    $this->assertArrayHasKey( 'crontab', $data['iw_cron'] );
  }

  /**
   * add action test
   */
  public function testSimpleAddAction() {
    $this->addSWAccountIfNecessary();

    $result = $this->_addCron( 'simple' );

    $this->assertTrue( $result );
  }

  /**
   * tests adding with the advanced editor
   */
  public function testAdvancedAddAction() {
    $this->addSWAccountIfNecessary();

    $result = $this->_addCron( 'advanced' );

    $this->assertTrue( $result );
  }

  /**
   * addCron function
   *
   * @param string $interface
   * @return boolean|array
   */
  private function _addCron( $interface = 'simple' ) {
    $SiteWorx = $this->getSWAccount();

    $input = array( 'minute'    => '*',
                    'hour'      => '*',
                    'day'       => '*',
                    'month'     => '*',
                    'dayofweek' => '*',
                    'script'    => 'echo hi' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SiteWorx );

    return $result;
  }

  /**
   * tests edit
   */
  public function testEditAction() {
    $SiteWorx    = $this->getSWAccount();
    $LineNumbers = new DataSource_Cron_LineNumbers( $SiteWorx->getUniqname() );

    $input = array( 'job'       => $LineNumbers->getFirstValue(),
                    'minute'    => '*',
                    'hour'      => '*',
                    'day'       => '*',
                    'month'     => '*',
                    'dayofweek' => '*',
                    'script'    => 'echo hi' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SiteWorx );

    $this->assertTrue( $result );
  }

  /**
   * tests delete
   */
  public function testDeleteAction() {
    $SiteWorx    = $this->getSWAccount();
    $LineNumbers = new DataSource_Cron_LineNumbers( $SiteWorx->getUniqname() );
    $input       = array( 'jobs' => $LineNumbers->getValues() );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SiteWorx );

    $this->assertTrue( $result );
  }

  /**
   * Tests options
   */
  public function testOptionsAction() {
    $SiteWorx = $this->getSWAccount();
    $user     = $SiteWorx->getUniqname();

    $crontab = IWorxCron::get_user_crontab( $user, true );

    if( empty( $crontab['vars']['shell'] ) ) {
      $shells  = array_keys( ShellAccount::getShells() );
      $crontab['vars']['shell'] = $shells[0];
    }

    $input = array( 'shell'  => (string)$crontab['vars']['shell'],
                    'path'   => (string)$crontab['vars']['path'],
                    'mailto' => (string)$crontab['vars']['mailto'] );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'options', $input, $SiteWorx );
    $this->assertTrue( $result );
  }
}
?>