<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 * @id $Id: EmailAutorespondTest.php,v 1.2 2009/03/18 08:17:31 poehler Exp $
 */

/**
 * Tests for Ctrl_Siteworx_EmailAutorespond
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailAutorespondTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_EmailAutorespond';

  const EMAIL   = 'otto';
  const COPYTO  = 'copy@to.com';
  const MESSAGE = 'Look, I am an autoresponder!';

  /**
   * Deletes teh account if needed
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_autoresponders'] ) !== 0 ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Tests the output of the index action
   */
  public function testIndexActionReturnsArray() {
    $output = $this->getSWIndexData( self::CTRL );

    $this->assertType( 'array', $output );
  }

  /**
   * Verifies initial count of autoresponders
   */
  public function testInitialAutoresponderCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_autoresponders'], 0,
                        'Autoresponders already exist!' );
  }

  /**
   * Tests that addAction fails without input
   */
  public function testAddFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that addAction returns true
   */
  public function testAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'emailaddress'       => self::EMAIL,
                    'copyto'             => self::COPYTO,
                    'autorespondmessage' => self::MESSAGE );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the add action seemed to work
   */
  public function testAddActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_autoresponders'], 1 );
  }

  /**
   * Tests that editAction fails without input
   */
  public function testEditFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that editAction returns true
   */
  public function testEditReturnsTrue() {
    Form::resetInstances();

    $SW = $this->getSWAccount();

    $input = array( 'username'   => self::EMAIL,
                    'copyto' => self::SW_EMAIL );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the edit action seemed to work
   */
  public function testEditActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    // Verify that the change happened
    $this->assertEquals( self::SW_EMAIL, $data['iw_autoresponders'][0]['copyto'] );
  }

  /**
   * Tests that deleteAction fails without input
   */
  public function testDeleteFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAction returns true
   */
  public function testDeleteReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'username' => self::EMAIL );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the delete action seemed to work
   */
  public function testDeleteActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_autoresponders'], 0 );
  }

}
?>