<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 26, 2009
 * @id $Id: EmailBoxTest.php,v 1.5 2009/09/16 08:15:51 poehler Exp $
 */

/**
 * Tests for Ctrl_Siteworx_EmailBox
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_EmailBoxTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_EmailBox';

  const EMAIL = 'bob';
  const PASS  = 'bobspassword';
  const QUOTA = '1234';

  /**
   * This test needs a clean SW account each time
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_mailboxes'] !== 1 ) ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Checks the initial count of boxes
   */
  public function testInitialMailboxCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_mailboxes'],
                        1,
                        'More email accounts than expected' );
  }

  /**
   * Checks that the form has been assigned
   */
  public function testForPresenceOfIwForm() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertArrayHasKey( 'iw_form', $data );
  }

  /**
   * Tests adding an email account
   */
  public function testAddActionReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'username'     => self::EMAIL,
                    'password'         => self::PASS,
                    'confirm_password' => self::PASS,
                    'diskspacequota'   => self::QUOTA );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertTrue( $result, 'Adding did not return true' );
  }

  /**
   * Checks that a second email account was actually added
   */
  public function testAddActionAdded() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_mailboxes'], 2, 'Account was not added' );
  }

  /**
   * Tests adding an email account duplicate fails
   */
  public function testAddActionDuplicateFails() {
    $SW = $this->getSWAccount();

    $input = array( 'username'     => self::EMAIL,
                    'password'         => self::PASS,
                    'confirm_password' => self::PASS,
                    'diskspacequota'   => self::QUOTA );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that editing returns true
   */
  public function testEditActionReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'username'       => self::EMAIL,
                    'diskspacequota' => '123' );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Makes sure that the value changed to the new one
   */
  public function testEditValueChanged() {
    $data = $this->getSWIndexData( self::CTRL );

    $user = $data['iw_mailboxes'][0];

    $this->assertEquals( '123MB', $user->quota, 'Quota was not changed!' );
  }

  /**
   * Tests delete action
   */
  public function testDeleteActionReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'username' => self::EMAIL );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Checks that the user was deleted
   */
  public function testDeleteActionDeleted() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_mailboxes'],
                        1,
                        'More email accounts than expected' );
  }
}
?>