<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 * @id $Id: EmailSpfTest.php,v 1.1 2009/03/05 21:27:47 tcollege Exp $
 */

/**
 * Tests for Ctrl_Siteworx_EmailSpf
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailSpfTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_EmailSpf';

  const SPF  = 'v=spf1';
  const SPF2 = 'v=spf1 a';

  /**
   * Deletes the account if something is in the way
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );
    $SW   = $this->getSWAccount();

    if( $data['iw_spf_data'][ $SW->getWorkingDomainName() ] !== '' ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Tests the output of the index action
   */
  public function testIndexActionReturnsArray() {
    $output = $this->getSWIndexData( self::CTRL );

    $this->assertType( 'array', $output );
  }

  /**
   * Tests that editAction fails without input
   */
  public function testEditAllFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'editAll', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that editAction returns true
   */
  public function testEditAllReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'spf_record_value' => self::SPF2 );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'editAll', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the edit action seemed to work
   */
  public function testEditAllActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );
    $SW   = $this->getSWAccount();
    $key  = $data['iw_spf_data'][ $SW->getWorkingDomainName() ];

    $this->assertEquals( self::SPF2, $key, 'Key was not set' );
  }

  /**
   * Tests that editAction fails without input
   */
  public function testEditFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that editAction returns true
   */
  public function testEditReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'domain'           => $SW->getWorkingDomainName(),
                    'spf_record_value' => self::SPF );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the edit action seemed to work
   */
  public function testEditActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );
    $SW   = $this->getSWAccount();
    $key  = $data['iw_spf_data'][ $SW->getWorkingDomainName() ];

    $this->assertEquals( self::SPF, $key, 'Key was not set' );
  }

  /**
   * Tests that deleteAction fails without input
   */
  public function testDeleteFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAction returns true
   */
  public function testDeleteReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'domain' => $SW->getWorkingDomainName() );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the delete action seemed to work
   */
  public function testDeleteActionWorked() {
    $SW   = $this->getSWAccount();
    $data = $this->getSWIndexData( self::CTRL );
    $key  = $data['iw_spf_data'][ $SW->getWorkingDomainName() ];

    $this->assertEquals( '', $key, 'Key was not deleted' );
  }

}
?>