<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    May 18, 2009
 * @id $Id: GemsTest.php,v 1.9 2009/09/21 22:05:57 poehler Exp $
 */

/**
 * Ctrl_Siteworx_GemsTest
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_GemsTest extends IWorx_Framework_TestCase {

  const CTRL        = 'Ctrl_Siteworx_Gems';
  const GEM_NAME    = 'fastercsv';
  const GEM_VERSION = '1.4.0';

  /**
   * tests the install action
   */
  public function testInstallAction() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }
    if( $this->_isGemInstalled( self::GEM_NAME ) === true ) {
      $this->_uninstallGem( self::GEM_NAME );
    }

    $result = $this->_installGem( self::GEM_NAME );

    $this->assertTrue( $result, 'Failed to install the test gem' );
  }

  /**
   * tests the reinstall action
   */
  public function testReinstallAction() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }
    if( $this->_isGemInstalled( self::GEM_NAME ) === false ) {
      $this->_installGem( self::GEM_NAME );
    }

    $SW     = $this->getSWAccount();
    $input  = array( 'gems' => array( self::GEM_NAME ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'reinstallCommit', $input, $SW );

    $this->assertTrue( $result, 'Failed to reinstall the test gem' );
  }

  /**
   * tests the update action
   */
  public function testUpdateAction() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }
    if( $this->_isGemInstalled( self::GEM_NAME ) === false ) {
      $this->_installGem( self::GEM_NAME );
    }

    $SW     = $this->getSWAccount();
    $input  = array( 'gems' => array( self::GEM_NAME ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'updateCommit', $input, $SW );

    $this->assertTrue( $result, 'Failed to update the test gem' );
  }

  /**
   * tests the cleanup action
   */
  public function testCleanupAction() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }
    if( $this->_isGemInstalled( self::GEM_NAME ) === false ) {
      $this->_installGem( self::GEM_NAME );
    }

    $SW     = $this->getSWAccount();
    $input  = array( 'gems' => array( self::GEM_NAME ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'cleanupCommit', $input, $SW );

    $this->assertTrue( $result, 'Failed to update the test gem' );
  }

  /**
   * tests the uninstall action
   */
  public function testUninstallAction() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }
    if( $this->_isGemInstalled( self::GEM_NAME ) === false ) {
      $this->_installGem( self::GEM_NAME );
    }

    $SW     = $this->getSWAccount();
    $input  = array( 'gems' => array( self::GEM_NAME ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'uninstallCommit', $input, $SW );

    $this->assertTrue( $result, 'Failed to uninstall the test gem' );
  }

  /**
   * installVersionActionTest
   */
  public function testInstallVersionAction() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }
    if( $this->_isGemInstalled( self::GEM_NAME ) === false ) {
      $this->_installGem( self::GEM_NAME );
    }

    $SW     = $this->getSWAccount();
    $input  = array( 'name'    => self::GEM_NAME,
                     'version' => self::GEM_VERSION );

    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'installversionCommit',
                                      $input,
                                      $SW );

    $this->assertTrue( $result, 'Failed to install gem version' );

    $gems = IWorxGems::home_gem_list( $SW->getWorkingDomainName(),
                                      self::GEM_NAME,
                                      $all_versions = true );

    // make sure the gem version is actually there
    $versions = $gems[0]['versions'];
    $this->assertTrue( in_array( self::GEM_VERSION, $versions ) === true );
  }

  /**
   * uninstallVersionActionTest
   */
  public function testUninstallVersionAction() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }
    $this->_installGem( self::GEM_NAME );

    $SW = $this->getSWAccount();
    $input = array( 'name'    => self::GEM_NAME,
                    'version' => self::GEM_VERSION );
    $result = IW::FC()->routeFromPHP( self::CTRL,
                                      'uninstallversionCommit',
                                      $input,
                                      $SW );

    $this->assertTrue( $result, 'Failed to uninstall gem version' );

    $gems = IWorxGems::home_gem_list( $SW->getWorkingDomainName(),
                                      self::GEM_NAME,
                                      $all_versions = true );

    // make sure the gem version is actually gone
    $versions = $gems[0]['versions'];
    $this->assertTrue( in_array( self::GEM_VERSION, $versions ) === false );
  }

  /**
   * is the specified gem installed
   *
   * @param string $gem_name
   * @return boolean
   */
  private function _isGemInstalled( $gem_name ) {
    $SW     = $this->getSWAccount();
    $domain = $SW->getMasterDomainName();
    $gems   = IWorxGems::local_gem_list( $domain, $gem_name );

    return !empty( $gems ) ? true : false;
  }

  /**
   * Installed the specified gem
   *
   * @param string $gem_name
   * @return boolean
   */
  private function _installGem( $gem_name ) {
    $SW = $this->getSWAccount();

    $input = array( 'name' => $gem_name,
                    'docs' => true );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'installCommit', $input, $SW );
    return $result;
  }

  /**
   * uninstall gem
   *
   * @param string $gem_name
   * @return boolean
   */
  private function _uninstallGem( $gem_name ) {
    $SW     = $this->getSWAccount();

    $input  = array( 'gems' => array( self::GEM_NAME ) );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'uninstallCommit', $input, $SW );

    return $result;
  }
}
?>