<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 * @id $Id: MysqlUserTest.php,v 1.1 2009/03/06 20:33:36 tcollege Exp $
 */

/**
 * Tests for Ctrl_Siteworx_MysqlUser
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_MysqlUserTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_MysqlUser';

  const NAME = 'dbuser';
  const PASS = 'password';

  /**
   * Clears the account if necessary
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_users'] ) !== 0 ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Tests the output of the index action
   */
  public function testIndexActionReturnsArray() {
    $output = $this->getSWIndexData( self::CTRL );

    $this->assertType( 'array', $output );
  }

  /**
   * Verifies the initial count of database users
   */
  public function testInitialUserCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_users'], 0, 'DB users already exist' );
  }

  /**
   * Tests that addAction fails without input
   */
  public function testAddFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that addAction returns true
   */
  public function testAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'name'             => self::NAME,
                    'password'         => self::PASS,
                    'confirm_password' => self::PASS );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the add action seemed to work
   */
  public function testAddActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_users'], 1 );
  }

  /**
   * Tests that editAction fails without input
   */
  public function testEditFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that editAction returns true
   */
  public function testEditReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'name'             => self::NAME,
                    'password'         => self::PASS . 1,
                    'confirm_password' => self::PASS . 1 );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Tests that deleteAction fails without input
   */
  public function testDeleteFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $SW );

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAction returns true
   */
  public function testDeleteReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'name' => self::NAME );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Test that the delete action seemed to work
   */
  public function testDeleteActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( $data['iw_users'], 0 );
  }

}
?>