<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    May 19, 2009
 * @id $Id: RailsTest.php,v 1.3 2009/08/21 02:29:39 poehler Exp $
 */

/**
 * Ctrl_SiteWorx_RailsTest
 *
 * @package    Tests
 * @subpackage InterWorx
 *
 */
class Ctrl_SiteWorx_RailsTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Siteworx_Rails';

  const APP_NAME = 'testrailsapp';
  const PROD     = 'production';
  const DEV      = 'development';

  /**
   * tests the create action
   */
  public function testCreate() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }

    $result = $this->_createApplication( self::APP_NAME,
                                         self::PROD );

    $this->assertTrue( $result, 'Failed to create test rails application' );
  }

  /**
   * tests the edit action
   */
  public function testEdit() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }

    $SW = $this->getSWAccount();
    if( $this->_applicationExists( self::APP_NAME ) === false ) {
      $this->_createApplication( self::APP_NAME, self::PROD );
    }

    $input = array( 'name'        => self::APP_NAME,
                    'environment' => self::PROD );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'editCommit', $input, $SW );
    $this->assertTrue( $result, 'Failed to edit test rails application' );
  }

  /**
   * Tests the restart action
   */
  public function testRestart() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }

    $SW = $this->getSWAccount();
    if( $this->_applicationExists( self::APP_NAME ) === false ) {
      $this->_createApplication( self::APP_NAME, self::PROD );
    }

    $input  = array( 'name' => self::APP_NAME );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'restartCommit', $input, $SW );

    $this->assertTrue( $result, 'Failed to restart test rails application' );
  }

  /**
   * Tests the delete action
   */
  public function testDelete() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }

    $SW = $this->getSWAccount();
    if( $this->_applicationExists( self::APP_NAME ) === false ) {
      $this->_createApplication( self::APP_NAME, self::PROD );
    }

    $input  = array( 'name' => self::APP_NAME );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'deleteCommit', $input, $SW );

    $this->assertTrue( $result, 'Failed to delete rails application' );
  }

  /**
   * tests the rake function
   */
  public function testRake() {
    if( IWorxGems::isRailsEnabled() === false ) {
      return;
    }

    $SW = $this->getSWAccount();
    if( $this->_applicationExists( self::APP_NAME ) === false ) {
      $this->_createApplication( self::APP_NAME, self::PROD );
    }

    // create the db
    $input  = array( 'name'    => self::APP_NAME,
                     'command' => 'db:create:all' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'rakeCommit', $input, $SW );

    // migrate it
    $input  = array( 'name'    => self::APP_NAME,
                     'command' => 'db:migrate' );
    $result = IW::FC()->routeFromPHP( self::CTRL, 'rakeCommit', $input, $SW );

    $this->assertTrue( $result, 'Failed to properly execute rake commands' );
  }

  /**
   * Checks to see if a given application exists.
   *
   * @param string $name
   * @return boolean
   */
  private function _applicationExists( $name ) {
    $ds     = new DataSource_SW_Rails_Applications();
    $values = $ds->getDisplayValues();

    return in_array( $name, $values );
  }

  /**
   * Create a rails application given a name and environment
   *
   * @param string $name
   * @param string $environment
   * @return boolean
   */
  private function _createApplication( $name, $environment ) {
    $SW    = $this->getSWAccount();
    $input = array( 'name'        => $name,
                    'environment' => $environment );

    return IW::FC()->routeFromPHP( self::CTRL, 'createCommit', $input, $SW );
  }
}
?>