<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 24, 2009
 * @id $Id: UsersTest.php,v 1.4 2009/03/05 18:28:54 tcollege Exp $
 */

/**
 * Tests for siteworx users controller
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_UsersTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_Users';

  const SW_SECONDARY_NICK_CHANGE = 'Changed Nickname';

  /**
   * Deletes the SW account to make sure there's a clean one
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_user_list'] ) !== 0 ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Tests that the index information has the user list
   */
  public function testIndexInformation() {
    $data = $this->_getIndexInfo();

    $this->assertArrayHasKey( 'iw_user_list', $data, 'User list is missing' );
  }

  /**
   * Checks the initial count of master users - damn well better be 1
   */
  public function testInitialMasterUserExists() {
    $data = $this->_getIndexInfo();

    $this->assertArrayHasKey( 'iw_master_user', $data, 'Master user is missing' );
  }

  /**
   * Tests that there are no secondary users initially
   */
  public function testInitialSecondaryUserCount() {
    $data = $this->_getIndexInfo();

    $this->assertCount( $data['iw_user_list'],
                        0,
                        'A Secondary User is already installed' );
  }

  /**
   * Tests adding a secondary user
   */
  public function testAddActionReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'nickname' => self::SW_SECONDARY_NICK,
                    'email'    => self::SW_SECONDARY_EMAIL,
                    'password' => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS,
                    'perms'    => array( 'LOGIN' ) );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Tests adding a secondary user again
   */
  public function testAddActionFailsWithDuplicate() {
    $SW = $this->getSWAccount();

    $input = array( 'nickname' => self::SW_SECONDARY_NICK,
                    'email'    => self::SW_SECONDARY_EMAIL,
                    'password' => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertType( 'array', $result );
  }

  /**
   * Verifies the added user data
   */
  public function testAddedUserData() {
    $data = $this->_getIndexInfo();

    $user = $data['iw_user_list'][0];

    $this->assertEquals( self::SW_SECONDARY_EMAIL, $user['email'] );
    $this->assertEquals( self::SW_SECONDARY_NICK, $user['nickname'] );
    $this->assertFalse( $user['ismaster'] );
  }

  /**
   * Tests that the edit action returns true
   */
  public function testEditActionReturnsTrue() {
    $SW      = $this->getSWAccount();
    $user_id = self::getSWUserId( self::SW_DOMAIN, self::SW_SECONDARY_EMAIL );

    $input = array( 'id'       => $user_id,
                    'nickname' => self::SW_SECONDARY_NICK_CHANGE );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Verifies the edited user data
   */
  public function testEditedUserData() {
    $data = $this->_getIndexInfo();

    $user = $data['iw_user_list'][0];

    $this->assertEquals( self::SW_SECONDARY_NICK_CHANGE, $user['nickname'] );
  }

  /**
   * Tests deleting the user
   */
  public function testDeleteActionReturnsTrue() {
    $SW      = $this->getSWAccount();
    $user_id = self::getSWUserId( self::SW_DOMAIN, self::SW_SECONDARY_EMAIL );

    $input = array( 'id' => $user_id );

    $result = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertTrue( $result );
  }

  /**
   * Verifies that the user was actually deleted
   */
  public function testDeleteActionHappened() {
    $user_id = self::getSWUserId( self::SW_DOMAIN, self::SW_SECONDARY_EMAIL );
    $this->assertFalse( $user_id, 'User still exists!' );
  }

  /**
   * Gets the array of information assigned to the index action
   *
   * @return array
   */
  private function _getIndexInfo() {
    $SW = $this->getSWAccount();
    return IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $SW );
  }
}
?>