<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 18, 2008
 * @id $Id: CLITestCase.php,v 1.5 2009/09/16 08:54:23 poehler Exp $
 */

/**
 * Used as a base class for command line based scripts
 *
 * @package    InterWorx
 * @subpackage Tests
 */
abstract class CLITestCase extends PHPUnit_Extensions_PerformanceTestCase {

  const QUICK_TEST_TIME_LIMIT = 10;

  /**
   * gets the name of the command line script we're testing
   */
  abstract protected function _getScriptToTest();

  /**
   * gets the command line parameters for testing.  Is a dataProvider
   *
   * @return array
   */
  abstract public function getCommandlineTestInfo();

  /**
   * setup
   */
  public function setUp() {
    $this->_setMaxTimeLimit();
  }

  /**
   * sets the max time limit depending on the group selection
   */
  private function _setMaxTimeLimit() {
    $opts = array( 'exclude-group=s' => '' );
    $Opt  = new IWorx_Console_Getopt( $opts );
    if( $Opt->getOption( 'exclude-group' ) === 'slow-tests' ) {
      $this->setMaxRunningTime( self::QUICK_TEST_TIME_LIMIT );
    } else {
      $this->setMaxRunningTime( 0 );
    }
  }


  /**
   * test command line script
   *
   * @param array  $parameters
   * @param string $expected_output
   * @param string $expected_retval
   * @param string $callback_func
   *
   * @dataProvider getCommandlineTestInfo
   */
  public function testCommandlineScript( array $parameters,
                                         $expected_output,
                                         $expected_retval,
                                         $callback_func = null ) {

    $bin = $this->_getScriptToTest();
    $cmd = "$bin " . implode( ' ', $parameters );
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::STDERR_2_STDOUT );

    $this->assertEquals( $expected_output,
                         implode( "\n", $result ),
                         'output did not match expected output' );
    $this->assertEquals( $expected_retval,
                         $retval,
                         'return value did not match expected return value' );

    if( !is_null( $callback_func ) ) {
      call_user_func( array( $this, $callback_func ) );
    }
  }

}

?>