<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College  <tcollege@interworx.com>
 * @date       Aug 5, 2008
 * @id $Id: IWorxSeleniumTestCase.php,v 1.55 2009/11/04 16:10:05 tcollege Exp $
 */

/**
 * IWorx Selenium helper class
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxSeleniumTestCase extends BrowserConfigSeleniumTestCase {
  const NW_USER = 'selenium@interworx.com';
  const NW_PASS = 'rickytickytimbo554';

  const SECONDARY_NICK  = 'NW Secondary User';
  const SECONDARY_EMAIL = 'second@interworx.com';
  const SECONDARY_PASS  = 'secondary';

  const SW_DOMAIN   = 'addedbytest.com';
  const SW_UNIQNAME = 'addedbyt';
  const SW_EMAIL    = 'abt@addedbytest.com';
  const SW_PASS     = 'password';
  const SW_NICK     = 'Added By Test';

  const SW_SECONDARY_NICK  = 'SW Secondary User';
  const SW_SECONDARY_EMAIL = 'sw_second@interworx.com';
  const SW_SECONDARY_PASS  = 'sw_secondary';

  const SW_PACKAGE = 'Package By Test';

  const TIMEOUT = 60; //seconds

  /**
   * Logs the Selenium browser into NodeWorx
   *
   * @param string $user     User email address to log in as
   * @param string $password User password
   */
  public function loginToNWAsUser( $user     = self::NW_USER,
                                   $password = self::NW_PASS ) {
    $this->type( 'email', $user );
    $this->type( 'password', $password );
    $this->clickAndWait( 'submit' );
  }

  /**
   * Logs the Selenium browser into SiteWorx
   *
   * @param string $user   User email address to log in as
   * @param string $pass   Password
   * @param string $domain Domain
   */
  public function loginToSWAsUser( $user   = self::SW_EMAIL,
                                   $pass   = self::SW_PASS,
                                   $domain = self::SW_DOMAIN ) {
    $this->type( 'email', $user );
    $this->type( 'password', $pass );
    $this->type( 'domain', $domain );
    $this->clickAndWait( 'submit' );
  }

  /**
   * Logs in to NodeWorx and verifies that redirect worked as expected
   *
   * @param string $page
   * @param string $user
   * @param string $pass
   */
  public function loginToNWAndGoTo( $page,
                                    $user = self::NW_USER,
                                    $pass = self::NW_PASS ) {

    $this->open( $page );
    $this->waitForPageToLoad( '60000' );

    // if we've been redirected to the login page...login.
    if( strstr( $this->getLocation(), '/nodeworx/index' ) ) {
      $this->waitForElement( 'email' );
      $this->loginToNWAsUser( $user, $pass );
    }
    $this->waitForElement( 'iw-menu' );
    $this->assertContains( $page, $this->getLocation() );
    $this->assertPageRenderedWithoutErrors();
  }

  /**
   * Logs you in to SiteWorx and redirects
   *
   * @param string $page
   * @param string $user
   * @param string $pass
   * @param string $domain
   */
  public function loginToSWAndGoTo( $page,
                                    $user = self::SW_EMAIL,
                                    $pass = self::SW_PASS,
                                    $domain = self::SW_DOMAIN ) {

    $this->open( $page );
    $this->waitForPageToLoad( '60000' );

    // if we've been redirected to the login page...login.
    if( strstr( $this->getLocation(), '/siteworx/index' ) !== false ) {
      $this->waitForElement( 'email' );
      $this->loginToSWAsUser( $user, $pass, $domain );
    }
    /* This is because SiteWorx always logs a user into the main page
     * instead of where the (then unauthenticated) user was trying to go.
     */
    if( $page != $this->getLocation() ) {
      $this->open( $page );
    }
    $this->waitForElement( 'iw-menu' );
    $this->assertContains( $page, $this->getLocation() );
    $this->assertPageRenderedWithoutErrors();
  }

  /**
   * Gets the siteworx id of the account that was created
   *
   * @param string $domain
   *
   * @return integer
   */
  public function getSWAccountId( $domain = self::SW_DOMAIN ) {
    $SiteWorx = new SiteWorx( $domain );
    return intval( $SiteWorx->getId() );
  }

  /**
   * Gets the user id of an email in nodeworx
   *
   * @param string $email
   * @return integer
   */
  public function getNWUserId( $email ) {
    if( IWorxValidate::falsey( $email ) === true ) {
      return false;
    }

    $NodeWorx = new NodeWorx( $email );

    $user = $NodeWorx->getWorkingUser();

    if( IWorxValidate::falsey( $user ) === true ) {
      return false;
    }

    $user_ID = $NodeWorx->getWorkingUser()->getId();

    $this->assertTrue( IWorxValidate::integer( $user_ID, 2 ),
                       'Unable to deterimine NodeWorx ID' );

    return intval( $user_ID );
  }


  /**
   * Creates a default SiteWorx account for tests
   */
  public function addSWAccount() {
    $this->loginToNWAndGoTo( '/nodeworx/siteworx' );

    $this->assertEquals( -1, $this->getSWAccountId(),
                         'Default SW Account already exists, cannot add!' );

    $this->click( 'link=' . IWorxTranslate::tr( '##LG_ADDSITEWORXACCOUNT##' ) );
    $this->waitForPageToLoad();

    $this->fillInAddSWAccountForm();

    $this->click( 'submit' );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SITEWORX_ADDSUCCESS##' );

    $this->waitForPageToLoad();
  }

  /**
   * Checks for default SiteWorx account and creates if necessary
   */
  public function addSWAccountIfNecessary() {
    IWorx_Framework_TestCase::addSWAccountIfNecessary();
  }

  /**
   * Clicks the save button of a FormBuilder form
   */
  public function clickSave() {
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_SAVE##' ) . "']" );
  }

  /**
   * Deletes the default SiteWorx account
   */
  public function deleteSWAccount() {
    $this->loginToNWAndGoTo( '/nodeworx/siteworx' );

    // Click the delete button
    $this->click( "//a[contains(@href, '" .
                  self::SW_DOMAIN .
                  "')][contains(@href, 'action=del')]" );
    $this->waitForPageToLoad();
    $deleteConfirm = '##LG_CONFIRM_SITEWORX_ACCOUNT_DELETION##';
    $this->assertTextPresent( IWorxTranslate::tr( $deleteConfirm ) );
    $this->assertTextPresent( self::SW_DOMAIN );

    // Gotta click the confirm button
    $this->click( 'confirm_action_0' );

    $this->click( "//input[@value='" .
                  IWorxTranslate::tr( '##LG_DELETE##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SITEWORX_DELETESUCCESS##' );

    $this->waitForPageToLoad();
  }

  /**
   * Checks if default SiteWorx Account exists and deletes if it does
   */
  public function deleteSWAccountIfNecessary() {
    if( IWorx_Framework_TestCase::getSWAccountExists() === true ) {
      $this->deleteSWAccount();
    }
  }

  /**
   * Fills in default values for creating a siteworx account in NodeWorx
   */
  public function fillInAddSWAccountForm() {
    $this->assertContains( 'nodeworx/siteworx', $this->getLocation() );
    $this->assertContains( 'action=add', $this->getLocation() );
    $this->type( 'domainname', self::SW_DOMAIN );
    $this->type( 'uniqname', self::SW_UNIQNAME );
    $this->select( 'billing_day', 'label=12' );
    $this->type( 'nickname', self::SW_NICK );
    $this->type( 'email', self::SW_EMAIL );
    $this->type( 'password', self::SW_PASS );
    $this->type( 'confirm_password', self::SW_PASS );
    $this->select( 'theme', 'value=interworx' );

    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $radio_button_name = substr( $opt['name'], 3 ) . '_check_0';
      $this->click( $radio_button_name );
    }

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      if( $opt['name'] != 'OPT_SUPHP' && $opt['name'] != 'OPT_SSI' ) {
        $this->select( $opt['name'],
                       'label=' . IWorxTranslate::tr( '##LG_YES##' ) );
      }
    }
  }

  /**
   * Verifies that the page did not whitescreen
   *
   * @param string $body
   */
  public function assertIsNotAWhitescreen( $body ) {
    $this->assertNotEquals( '', $body, $this->makeFailMessage( 'whitescreened' ) );
  }

  /**
   * Searches through the string given (bodyText)
   *
   * Verifies that Smarty didn't hock a wobbly
   *
   * @param string $body
   */
  public function assertNotContainsSmartyBarf( $body ) {
    $msg = 'contains Smarty barf or maybe just lang entry with no translation';
    $this->assertNotContains( '##LG_',
                              $body,
                              $this->makeFailMessage( $msg ) );
  }

  /**
   * Searches through the string given (bodyText)
   *
   * Verifies that Smarty didn't hock a wobbly
   *
   * @param string $body
   */
  public function assertNotContainsUntranslatedString( $body ) {
    $msg = 'contains untranslated string';
    $this->assertNotContains( 'LG_',
                              $body,
                              $this->makeFailMessage( $msg ) );
  }

  /**
   * Verifies that the page contains no stacktraces
   *
   * @param string $body
   */
  public function assertNotContainsStacktrace( $body ) {
    // Make sure there's no errors on the page
    $this->assertNotContains( 'InterWorx Error Report',
                              $body,
                              $this->makeFailMessage( 'contains a stacktrace' ) );
  }

  /**
   * Searches the string given to verify no PHP errors are present
   *
   * @param string $body
   */
  public function assertNotContainsPHPErrors( $body ) {
    if( strstr( $body, 'Parse error' ) &&
        strstr( $body, 'on line' ) ) {
      $this->fail( $this->makeFailMessage( 'contains PHP Parse Errors' ) );
    }
  }

  /**
   * Shell function to call several other assertions
   *
   * @param boolean $check_translation
   */
  public function assertPageRenderedWithoutErrors( $check_translation = false ) {
    $body = $this->getBodyText();

    $this->assertIsNotAWhitescreen( $body );
    $this->assertNotContainsStacktrace( $body );
    $this->assertNotContainsPHPErrors( $body );
    $this->assertNotContainsSmartyBarf( $body );
    if( $check_translation === true ) {
      $this->assertNotContainsUntranslatedString( $body );
    }
  }

  /**
   * Verifies that a given regex is present in the bodyText
   *
   * @param string $regex
   * @param string $message
   */
  public function assertRegExpPresent( $regex, $message = '' ) {
    $body = $this->getBodyText();
    $this->assertRegExp( $regex, $body, $message );
  }

  /**
   * Verifies that a regex is NOT in the bodyText
   *
   * @param string $regex
   * @param string $message
   */
  public function assertRegExpNotPresent( $regex, $message = '' ) {
    $body = $this->getBodyText();
    $this->assertNotRegExp( $regex, $body, $message );
  }

  /**
   * Asserts that a given language entry is present in translated form
   *
   * @param string $lang_key
   * @param string $message
   */
  public function assertTranslatedTextPresent( $lang_key, $message = '' ) {
    $this->assertTextPresent( IWorxTranslate::tr( $lang_key ), $message );
  }

  /**
   * Checks if the translated value of a key is present
   *
   * @param string $lang_key
   * @return boolean
   */
  public function isTranslatedTextPresent( $lang_key ) {
    return $this->isTextPresent( IWorxTranslate::tr( $lang_key ) );
  }

  /**
   * Asserts that a given language entry is not present in translated form
   *
   * @param string $lang_key
   * @param string $message
   */
  public function assertTranslatedTextNotPresent( $lang_key, $message = '' ) {
    $this->assertTextNotPresent( IWorxTranslate::tr( $lang_key ), $message );
  }

  /**
   * Deletes the default SiteWorx package
   */
  public function deleteSWPackage() {
    $this->loginToNWAndGoTo( '/nodeworx/packages' );

    // Click the delete button
    $this->clickAndWait( 'delete-' . self::SW_PACKAGE );

    $deleteConfirm = '##LG_CONFIRMDELETIONOF##';
    $this->assertTextPresent( IWorxTranslate::tr( $deleteConfirm ) );
    $this->assertTextPresent( self::SW_PACKAGE );
    $this->click( "//input[@value='" .
                  IWorxTranslate::tr( '##LG_DELETE##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_PACKAGE_DELETESUCCESS##' );

    $this->waitForPageToLoad();
  }

  /**
   * Checks if default SiteWorx package exists and deletes if it does
   */
  public function deleteSWPackageIfNecessary() {
    if( $this->packageTemplateExists() ) {
      $this->deleteSWPackage();
    }
  }

  /**
   * Creates a default SiteWorx package for tests
   */
  public function addSWPackage() {
    $this->loginToNWAndGoTo( '/nodeworx/packages' );

    $this->assertFalse( $this->packageTemplateExists(),
                        'Default SW Package already exists, cannot add!' );

    $this->click( 'link=' . IWorxTranslate::tr( '##LG_ADD##' ) );
    $this->waitForPageToLoad();

    $this->fillInAddSWPackageForm();

    $this->click( "//input[@value='Add']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_PACKAGE_ADDSUCCESS##' );
  }

  /**
   * Checks for default SiteWorx package and creates if necessary
   */
  public function addSWPackageIfNecessary() {
    $exists = $this->packageTemplateExists();
    if( !$exists ) {
      $this->addSWPackage();
    }
  }

  /**
   * Checks to see if a package exists
   *
   * @param string $package_name
   * @return boolean
   */
  public function packageTemplateExists( $package_name =  self::SW_PACKAGE ) {
    $NodeWorx = NodeWorx::getDefaultNodeWorx();
    return $NodeWorx->packageTemplateExists( $package_name );
  }

  /**
   * Fills in default values for creating a siteworx package in NodeWorx
   */
  public function fillInAddSWPackageForm() {
    $this->assertContains( 'nodeworx/packages', $this->getLocation() );

    $this->type( 'package_name', self::SW_PACKAGE );

    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $radio_button_label = $opt['name'] . '_check_0';
      $this->click( $radio_button_label );
    }

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      $this->select( $opt['name'], 'label=' . IWorxTranslate::tr( '##LG_YES##' ) );
    }
  }

  /**
   * Waits for an element and ensures it has the correct text.
   *
   * @param string  $locator
   * @param string  $text
   * @param integer $timeout
   */
  public function waitForTextInElement( $locator, $text, $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementPresent', $locator, $timeout );
    $this->assertElementContainsText( $locator, $text );
  }

  /**
   * Waits for an element to appear before continuing
   *
   * @param string  $locator
   * @param integer $timeout seconds
   */
  public function waitForElement( $locator, $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementPresent', $locator, $timeout, $sleep = 1 );
  }

  /**
   * Waits for the message banner
   *
   * @param integer $timeout How long before the function times out (in seconds)
   */
  public function waitForMessageBanner( $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementVisable', 'iw-mb', $timeout );
  }

  /**
   * Returns true if the element is visable, and false otherwise.  Also if the
   * visibility is set to 'inherit' (as it would is some browsers) then we'll
   * continue look at each parent until we can figure out the visibility of the
   * element.
   *
   * @param string $id
   * @return boolean
   */
  public function isElementVisable( $id ) {
    $element = "window.document.getElementById( '{$id}' )";
    $style   = '.style.visibility;';

    $js         = $element . $style;
    $visibility = $this->getEval( $js );

    $count     = 0;
    $max_loops = 100;  // to prevent a shitty DOM Tree from putting us in a
                       // never ending loop.
    while( $visibility == 'inherit' ) {
      $element    = $element . '.parentNode';
      $js         = $element . $style;
      $visibility = $this->getEval( $js );

      $count++;
      if( $count >= $max_loops ) {
        break;
      }
    }
    return $visibility == 'visible';
  }

  /**
   * Gets the Message Banner inner-HTML
   *
   * @param string  $id
   * @param integer $timeout
   * @return string
   */
  public function getMessageBannerHTML( $id = 'iw-mb-content',
                                        $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementVisable', 'iw-mb', $timeout );

    return $this->getIWMessageHTML( $id );
  }

  /**
   * Gets the innerHTML of any iw_message, by id
   *
   * @param string $id
   * @return string
   */
  public function getIWMessageHTML( $id ) {
    $js_script  = "window.document.getElementById( '$id' ).";
    $js_script .= "getElementsByTagName( 'div' )[2].innerHTML";

    return $this->getEval( $js_script );
  }

  /**
   * Gets an element attribute.
   *
   * @param string $id
   * @param string $attribute
   * @return string
   */
  public function getElementAttributeById( $id, $attribute ) {
    $js = "window.document.getElementById( '{$id}' )
          .getAttribute( '{$attribute}' );";

    return $this->getEval( $js );
  }

  /**
   * Closes the iw-mg banner
   */
  public function closeMessageBanner() {
    $this->click( 'link=Close' );
  }

  /**
   * Waits until the callback function returns a success
   *
   * @param string       $callback
   * @param string|array $args
   * @param float        $timeout
   * @param float        $sleep_time
   */
  private function _waitWithTimeout( $callback,
                                     $args,
                                     $timeout,
                                     $sleep_time=0.10 ) {

    $start_time = microtime( true );
    $sleep_time = $sleep_time * 1000000;

    $success = $this->{$callback}( $args );
    while( $success !== true ) {
      usleep( $sleep_time );
      $elapsed_time = microtime( true ) - $start_time;

      if( $elapsed_time >= $timeout ) {
        $this->fail( "$callback timed out" );
        return;
      }
      $success = $this->{$callback}( $args );
    }
    return;
  }
}
?>
