<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 23, 2009
 * @id $Id: IWorx_Framework_TestCase.php,v 1.32 2009/09/16 18:32:25 poehler Exp $
 */

/**
 * Helper functions as a base class for unit tests
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Framework_TestCase extends PHPUnit_Extensions_PerformanceTestCase {
  const NW_USER = 'selenium@interworx.com';
  const NW_PASS = 'rickytickytimbo554';

  const SECONDARY_NICK  = 'NW Secondary User';
  const SECONDARY_EMAIL = 'second@interworx.com';
  const SECONDARY_PASS  = 'secondary';

  const RSL_NICKNAME = 'Reseller Nickname';
  const RSL_EMAIL    = 'reseller@interworx.com';
  const RSL_PASSWORD = 'rslpassword';
  const RSL_STATUS   = User::ACTIVE;


  const SW_DOMAIN   = 'addedbytest.com';
  const SW_UNIQNAME = 'addedbyt';
  const SW_EMAIL    = 'abt@addedbytest.com';
  const SW_PASS     = 'password';
  const SW_NICK     = 'Added By Test';

  const SW_SECONDARY_DOMAIN = 'secondary.com';

  const SW_SECONDARY_NICK  = 'SW Secondary User';
  const SW_SECONDARY_EMAIL = 'sw_second@interworx.com';
  const SW_SECONDARY_PASS  = 'sw_secondary';

  const SW_POINTER_DOMAIN = 'pointer.com';

  const SW_PACKAGE = 'Package By Test';

  const FORM_ERROR_HEADER =
  '##LG_INPUT_FORM_DID_NOT_VALIDATE##. ##LG_INPUT_FORM_SEE_DETAILS_BELOW##.';

  const QUICK_TEST_TIME_LIMIT = 10;

  /**
   * NodeWorx object for the master user
   *
   * @var NodeWorx
   */
  static private $_NodeWorx;

  /**
   * SiteWorx object for the self::SW_DOMAIN domain
   *
   * @var SiteWorx
   */
  private $_SiteWorx;

  /**
   * Gets the master NodeWorx object
   *
   * @return NodeWorx
   */
  static public function getMasterNodeWorx() {
    if( is_null( self::$_NodeWorx ) === true ) {
      self::$_NodeWorx = new NodeWorx( self::NW_USER );
    }

    return self::$_NodeWorx;
  }

  /**
   * Gets the Reseller NodeWorx object
   *
   * DO NOT CACHE
   *
   * @return NodeWorx
   */
  static public function getResellerNodeWorx() {
    return new NodeWorx( self::RSL_EMAIL );
  }

  /**
   * Gets the account id of a SW domain
   *
   * @param string $domain
   * @return integer
   */
  static public function getSWAccountId( $domain = self::SW_DOMAIN ) {
    $SiteWorx = new SiteWorx( $domain );
    return intval( $SiteWorx->getId() );
  }

  /**
   * Gets the SW User Id for the info provided
   *
   * @param string $domain
   * @param string $email
   * @return integer|false
   */
  static public function getSWUserId( $domain = self::SW_DOMAIN,
                                      $email = self::SW_EMAIL ) {
    if( !$email ) {
      return false;
    }

    $SiteWorx = new SiteWorx( $domain );
    $user     = $SiteWorx->getUser( $email );

    if( empty( $user ) ) {
      return false;
    } else {
      $user_ID = $user->getId();
      return intval( $user_ID );
    }
  }

  /**
   * Gets the NW User Id for the info provided
   *
   * @param string $email
   * @return integer|false
   */
  static public function getNWUserId( $email = self::SW_SECONDARY_EMAIL ) {
    if( !$email ) {
      return false;
    }

    $NodeWorx = self::getMasterNodeWorx();
    $where    = 'email = ' . q( self::SW_SECONDARY_EMAIL );
    $user     = $NodeWorx->getUserInfo( $where );

    if( empty( $user ) ) {
      return false;
    } else {
      $user_ID = $user->getId();
      return intval( $user_ID );
    }
  }

  /**
   * Checks if a domain exists
   *
   * @param string $domain
   * @return boolean
   */
  static public function getSWAccountExists( $domain = self::SW_DOMAIN ) {
    return self::getSWAccountId( $domain ) !== -1;
  }

  /**
   * Adds the default SW account if it doesn't exist
   *
   * @return boolean
   */
  static public function addSWAccountIfNecessary() {
    if( self::getSWAccountExists() === false ) {
      $result = self::addSWAccount();

      if( $result !== true ) {
        echo "\nUnable to add SiteWorx Account for test\n";
        return false;
      } else {
        return true;
      }
    }

    return true;
  }

  /**
   * Gets the array of input needed for adding the account
   *
   * @return array
   */
  static protected function _getSWAccountInput() {
    $input = array();
    $input['domainname']       = self::SW_DOMAIN;
    $input['ipaddress']        = self::_getFirstSharedIP();
    $input['uniqname']         = self::SW_UNIQNAME;
    $input['nickname']         = self::SW_NICK;
    $input['email']            = self::SW_EMAIL;
    $input['password']         = self::SW_PASS;
    $input['confirm_password'] = $input['password'];

    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $input_name = substr( $opt['name'], 3 );
      $input[ $input_name ] = Ini::get( Ini::CONSTANTS, 'unlimited' );
    }

    foreach( $options as $opt ) {
      $input_name = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      if( $input_name != 'OPT_SUPHP' && $input_name != 'OPT_SSI' ) {
        $input[ $input_name ] = 1;
      }
    }

    $input = array_merge( $input, self::_getPackageOptionsInput() );

    return $input;
  }

  /**
   * gets an array of siteworx package options w/ default values
   *
   * @return array
   */
  static protected function _getPackageOptionsInput() {
    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $input_name = substr( $opt['name'], 3 );
      $input[ $input_name ] = Ini::get( Ini::CONSTANTS, 'unlimited' );
    }

    foreach( $options as $opt ) {
      $input_name = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] === '1' ) {
        continue;
      }
      if( $input_name != 'OPT_SUPHP' && $input_name != 'OPT_SSI' ) {
        $input[ $input_name ] = 1;
      } else {
        $input[ $input_name ] = 0;
      }
    }

    return $input;
  }

  /**
   * gets valid reseller input data
   */
  protected function _getResellerInput() {
    $input = array( 'nickname'         => self::RSL_NICKNAME,
                    'email'            => self::RSL_EMAIL,
                    'password'         => self::RSL_PASSWORD,
                    'confirm_password' => self::RSL_PASSWORD,
                    'status'           => self::RSL_STATUS,
                    'database_servers' => array( 'localhost' ),
                    'ips'              =>
                    array( $this->_getFirstSharedResellerIP() ) );
    $input = array_merge( $input, $this->_getResellerPackageOptionsInput() );
    return $input;
  }

  /**
   * gets reseller package option input
   *
   * @return array
   */
  static protected function _getResellerPackageOptionsInput() {
    $options = Reseller_Option::getOptionList();

    foreach( $options as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $input_name = substr( $opt['name'], 3 );
      $input[ $input_name ] = Ini::get( Ini::CONSTANTS, 'unlimited' );
    }

    foreach( $options as $opt ) {
      $input_name = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] === '1' ) {
        continue;
      }
      $input[ $input_name ] = 1;
    }

    return $input;
  }

  /**
   * gets the first shared ip
   *
   * @return string
   */
  static protected function _getFirstSharedIP() {
    $ips = IPAddress::getList( IPAddress::AVAILABLE, IPAddress::SHAREDIP );
    $ip  = array_shift( $ips );
    return $ip['ipaddr'];
  }

  /**
   * gets the first available dedicated ip
   *
   * @return string
   */
  static protected function _getFirstAvailableDedicatedIP() {
    $ips = IPAddress::getList( IPAddress::AVAILABLE, IPAddress::STATICIP );

    if( 0 === count( $ips ) ) {
      print( 'Booooo.  No free dedicated IPs, I need one.' );
    }
    $ip  = array_shift( $ips );
    return $ip['ipaddr'];
  }

  /**
   * gets the first shared reseller ip
   */
  protected function _getFirstSharedResellerIP() {
    $ips = IPAddress::getList( IPAddress::AVAILABLE,
                               IPAddress::SHAREDIP,
                               IPAddress::RESELLER_YES );

    $this->assertGreaterThan( 0, count( $ips ),
                              'adding reseller, no shared reseller ips' );
    $ip = array_shift( $ips );
    return $ip['ipaddr'];
  }

  /**
   * Adds a SiteWorx account for testing
   *
   * @return boolean
   */
  static public function addSWAccount() {
    $NW    = self::getMasterNodeWorx();
    $input = self::_getSWAccountInput();

    $result = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                                      'add',
                                      $input,
                                      $NW );

    return $result;
  }

  /**
   * Deletes the testing SW account
   */
  public function deleteSWAccount() {
    $SW = $this->getSWAccount();

    $NW = self::getMasterNodeWorx();

    $NW->deleteAccount( $SW );
    $this->_SiteWorx = null;
  }

  /**
   * Deletes the test SW Account if it exists
   */
  public function deleteSWAccountIfNecessary() {
    if( self::getSWAccountExists() === true ) {
      $this->setMaxRunningTime( 0 );
      $this->deleteSWAccount();
    }
  }

  /**
   * Gets a SiteWorx object for the self::SW_DOMAIN domain
   *
   * @return SiteWorx
   */
  public function getSWAccount() {
    if( is_null( $this->_SiteWorx ) ) {
      $this->setMaxRunningTime( 0 );
      self::addSWAccountIfNecessary();
      $this->_SiteWorx = new SiteWorx( self::SW_DOMAIN );
      ApacheVhostReader::getInstance( $reset = true );
    }
    return $this->_SiteWorx;
  }

  /**
   * Gets the index data from a controller
   *
   * @param string $controller
   * @return array
   */
  public function getSWIndexData( $controller ) {
    $SW = $this->getSWAccount();

    return IW::FC()->routeFromPHP( $controller, 'index', array(), $SW );
  }

  /**
   * Asserts the count of an array
   *
   * @param mixed   $array
   * @param integer $count
   * @param string  $message
   */
  public function assertCount( array $array, $count, $message = null ) {
    $this->assertEquals( $count, count( $array ), $message );
  }

  /**
   * Asserts that the passed in var is an array, and is a "Form Error" array
   *
   * @param mixed $array not typehinted to an array on purpose
   */
  public function assertIsFormErrorArray( $array ) {
    $this->assertTrue( is_array( $array ),
                       'Looking for a Form Error Array, but got a ' .
                       gettype( $array ) .
                       " ({$array})" );

    $this->assertEquals( self::FORM_ERROR_HEADER, $array[ 0 ],
                         'Looking for a Form Error array, but got a ' .
                         gettype( $array[ 0 ] ) .
                         " ({$array[ 0 ]})" );
  }

  /**
   * Asserts a Form Error Array contains "needle"
   *
   * @param string $needle
   * @param mixed  $haytack not typehinted to an array on purpose
   */
  public function assertFormErrorArrayContains( $needle, $haytack ) {
    $this->assertIsFormErrorArray( $haytack );
    $this->assertContains( $needle, $haystack,
                           'Form Error Array did not contain ' . $needle );
  }

  /**
   * Asserts a Form Error Array DOES NOT contain "needle"
   *
   * @param string $needle
   * @param mixed  $haytack not typehinted to an array on purpose
   */
  public function assertFormErrorArrayNotContains( $needle, $haytack ) {
    $this->assertIsFormErrorArray( $haytack );
    $this->assertNotContains( $needle, $haystack,
                              'Form Error Array contains ' . $needle .
                              'but I did not want it to' );
  }

  /**
   * Asserts that an number is between two other numbers
   *
   * @param mixed  $min
   * @param mixed  $max
   * @param mixed  $actual
   * @param string $message
   */
  public function assertBetween( $min, $max, $actual, $message = null ) {
    if( is_null( $message ) ) {
      $message = "Failed asserting that $actual is between $min and $max.";
    }

    $this->assertGreaterThanOrEqual( $min, $actual, $message );
    $this->assertLessThanOrEqual( $max, $actual, $message );
  }

  /**
   * Resets the forms before every test
   */
  public function setUp() {
    Form::resetInstances();
    IW::Env()->resetSession();

    // Clear the input values
    $input = IW::Env()->getAllInput();

    foreach( $input as $name => $value ) {
      IW::Env()->addInput( $name, null );
    }

    IWorxDbUtil::killCachedDbInstance();
    $this->_setMaxTimeLimit();
  }

  /**
   * sets the max time limit depending on the group selection
   */
  private function _setMaxTimeLimit() {
    $opts = array( 'exclude-group=s' => '' );
    $Opt  = new IWorx_Console_Getopt( $opts );
    if( $Opt->getOption( 'exclude-group' ) === 'slow-tests' ) {
      $this->setMaxRunningTime( self::QUICK_TEST_TIME_LIMIT );
    } else {
      $this->setMaxRunningTime( 0 );
    }
  }

  /**
   * I can't believe this all is now one function. :P
   * Does a backup and import on an account.
   *
   * @return boolean
   */
  public function doBackupAndImport() {
    self::addSWAccountIfNecessary();
    $NW = self::getMasterNodeWorx();

    $tgz_path = $this->backupAccount( self::SW_DOMAIN );

    $this->deleteSWAccount();

    // Import the backup
    $ip       = self::_getFirstSharedIP();
    $dbserver = MySQLServer::getLocalServer()->getNickname();

    $input = array( 'remote_file' => $tgz_path,
                    'cp'          => 'siteworx',
                    'dbserver'    => $dbserver,
                    'reseller'    => $NW->getId(),
                    'ipaddr'      => $ip );

    $result = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Import',
                                      'import',
                                      $input,
                                      $NW );

    //Cleanup
    IWorxFileSys::rm( $tgz_path );

    if( $result !== true ) {
      $this->markTestIncomplete( 'Import failed, test incomplete' );
    }

    return $result;
  }

  /**
   * Backs up the account and returns the path of the backup.
   *
   * @param string $domain
   * @return string $filepath
   */
  public function backupAccount( $domain ) {
    $tmp      = Ini::get( 'dir', 'tmp' );
    $path     = "{$tmp}/";
    $filename = 'backupforunittest';
    $tgz_path = "{$path}{$filename}.tgz";

    $backup = Ini::get( 'iworx.bin', 'backup' );

    $cmd = "$backup
      --domain {$domain}
      --output-dir={$path}
      --filename-format={$filename}";

    IWorxExec::exec( $cmd, $result, $retval );
    $this->assertTrue( $retval == 0 );

    return $tgz_path;
  }

  /**
   * Change the database on the dsn to the one given.
   *
   * @param string $database
   * @param string $dsn
   * @return string
   */
  public function changeDatabaseOnDsn( $database, $dsn ) {
    $new_dsn = preg_replace( '/\/mysql$/', "/{$database}", $dsn );
    return $new_dsn;
  }

  /**
   * check to see if the given routine exists
   *
   * @param string $database
   * @param string $routine_name
   * @return boolean
   */
  public function checkIfRoutineExists( $database, $routine_name ) {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();
    $db       = IWorxDbUtil::db_connect( $Driver->getDsn(), $die = false );

    $q_routine_name = q( $routine_name );
    $q_database     = q( $Driver->getFQDN( $database ) );

    $sql = "SELECT COUNT(*)
            FROM   `proc`
            WHERE  `name` = {$q_routine_name} AND
                   `db`   = {$q_database};";

    $routine_count = IWorxDbUtil::db_getOne( $db, $sql );
    return $routine_count > 0;
  }

  /**
   * Deletes the routine as the "SUPER" iworx user.
   *
   * @param string $database
   * @param string $routine_name
   * @return boolean
   */
  public function deleteRoutineAsSuper( $database, $routine_name ) {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $dsn = $this->changeDatabaseOnDsn( $database, $Driver->getDsn() );
    $db  = IWorxDbUtil::db_connect( $dsn, $die = false );

    $sql = "DROP PROCEDURE IF EXISTS `{$routine_name}`";
    $result = IWorxDbUtil::db_query( $db, $sql );

    return Db::isError( $result ) === false;
  }
}
?>