<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jul 1, 2009
 * @id $Id: PhpmyadminTest.php,v 1.3 2009/08/21 02:30:40 poehler Exp $
 */

/**
 * PhpMyAdmin selenium test.  Check for security holes.
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class PhpmyadminTest extends IWorxSeleniumTestCase {

  const NO_LOGIN_FOR_YOU = 'Please login via your control panel';

  /**
   * tests PMA security when linking directly to pma.
   */
  public function testPMASecurtyFromDirectLinkNoArgs() {
    $this->open( '/mysql/index.php' );
    $this->waitForPageToLoad();

    sleep( 1 );

    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );
  }

  /**
   * tests from direct link while trying to pass in a bad session arg
   */
  public function testPMASecuirtyFromDirectLinkWithBadStuff() {
    $md5 = md5( rand() );

    $this->open( '/mysql/index.php?phpMyAdmin=' . $md5 );
    $this->waitForPageToLoad();

    sleep( 1 );

    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );
  }

  /**
   * tests from direct link while trying to access a server in the config array
   */
  public function testPMASecurityFromDirectLinkWithBadArgs() {
    $this->open( '/mysql/index.php?server=1' );
    $this->waitForPageToLoad();

    sleep( 1 );

    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );
  }

  /**
   * tests diret link with bad session arg and config array
   */
  public function testPMASecurityFromDirectLinkDoubleTeam() {
    $md5 = md5( rand() );

    $this->open( '/mysql/index.php?server=1&phpMyAdmin=' . $md5 );
    $this->waitForPageToLoad();

    sleep( 1 );

    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );

  }

  /**
   * tries to connect to the iworx-login page with no args
   */
  public function testPMASecurityFromIWorxLoginNoArgs() {
    $this->open( '/mysql/iworx-login.php' );
    $this->waitForPageToLoad();

    sleep( 1 );

    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );
  }

  /**
   * tries to connect with login arg
   */
  public function testPMASecurityFromIWorxLoginWithArgs() {
    $this->addSWAccountIfNecessary();

    $this->open( '/mysql/iworx-login.php?action=login' );
    $this->waitForPageToLoad();
    sleep( 1 );
    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );

    for( $i = 0; $i < 10; $i++ ) {
      $this->open( '/mysql/iworx-login.php?action=login&server=' . $i );
      $this->waitForPageToLoad();
      sleep( 1 );
      $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );
    }
  }

  /**
   * tries to access the domain directly.
   */
  public function testPMASecurityFromIWorxTryToAccessDomain() {
    $domain   = self::SW_DOMAIN;
    $SiteWorx = new SiteWorx( $domain );
    $Driver   = $SiteWorx->getMySQLDriver();

    if( $Driver->databaseExists( 'dummy' ) === false ) {
      $Driver->addDatabase( 'dummy' );
    }

    $this->open( '/mysql/iworx-login.php?action=login&domain=' . $domain );
    $this->waitForPageToLoad();
    sleep( 1 );
    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );
  }

  /**
   * tries to do a command injection attack
   */
  public function testPMASercurityDomainCMDScrub() {
    $extra = '> /tmp/loginstuff.txt';

    $domain = self::SW_DOMAIN;
    $this->open( "/mysql/iworx-login.php?action=login&domain={$domain}{$extra}" );

    $this->waitForPageToLoad();
    sleep( 2 );
    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );

    if( file_exists( '/tmp/loginstuff.txt' ) === true ) {
      IWorxFileSys::rm( '/tmp/loginstuff.txt' );
      $this->assertTrue( false );
    }//end if
  }

  /**
   * tries to do a command injection attack
   */
  public function testPMASercurityServerCMDScrub() {
    $extra = '> /tmp/loginstuff.txt';

    $domain = self::SW_DOMAIN;
    $this->open( "/mysql/iworx-login.php?action=login&server=1{$extra}" );

    $this->waitForPageToLoad();
    sleep( 2 );
    $this->assertTextPresent( self::NO_LOGIN_FOR_YOU );

    if( file_exists( '/tmp/loginstuff.txt' ) === true ) {
      IWorxFileSys::rm( '/tmp/loginstuff.txt' );
      $this->assertTrue( false );
    }//end if
  }

}
?>