<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    May 21, 2009
 * @id $Id: PageNWImportTest.php,v 1.3 2009/10/27 05:10:20 poehler Exp $
 */

/**
 * Test to make sure the proper elements are being displayed on the import
 * single account pages
 *
 * @group NodeWorx
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class PageNWImportTest extends IWorxSeleniumTestCase {

  /**
   * testImportInitialFormRender
   */
  public function testImportInitialFormRender() {
    $this->loginToNWAndGoTo( '/nodeworx/import?action=importSingle',
                             self::NW_USER,
                             self::NW_PASS );

    $this->assertPageRenderedWithoutErrors();

    $this->assertTranslatedTextPresent( '##LG_UPLOAD_BACKUP_FILE##' );
    $this->assertTranslatedTextPresent( '##LG_FILE_ALREADY_ON_SERVER##' );
  }

  /**
   * testImportUploadFormRender
   */
  public function testImportUploadFormRender() {
    $this->loginToNWAndGoTo( '/nodeworx/import?action=importSingle',
                             self::NW_USER,
                             self::NW_PASS );

    $this->assertPageRenderedWithoutErrors();

    $this->open( '/nodeworx/import?action=importSingle' );
    $this->click( 'submit' );
    $this->waitForPageToLoad();

    $this->assertTranslatedTextPresent( '##LG_LOCALBACKUPFILE##' );

    $this->assertElementPresent( 'local_file' );
    $this->assertElementPresent( 'cp' );
    $this->assertElementPresent( 'reseller' );
    $this->assertElementPresent( 'ipaddr' );
    $this->assertElementPresent( 'dbserver' );
  }

  /**
   * testImportLocalFileFormRender
   */
  public function testImportLocalFileFormRender() {
    $this->loginToNWAndGoTo( '/nodeworx/import?action=importSingle',
                             self::NW_USER,
                             self::NW_PASS );

    $this->assertPageRenderedWithoutErrors();

    $this->open( '/nodeworx/import?action=importSingle' );
    $this->select( 'filetype', 'value=remote' );
    $this->click( 'submit' );
    $this->waitForPageToLoad();

    $this->assertTranslatedTextPresent( '##LG_REMOTEBACKUPFILE##' );

    $this->assertElementPresent( 'remote_file' );
    $this->assertElementPresent( 'cp' );
    $this->assertElementPresent( 'reseller' );
    $this->assertElementPresent( 'ipaddr' );
    $this->assertElementPresent( 'dbserver' );
  }

  /**
   * testActualImport
   */
  public function testActualImport() {
    $this->addSWAccountIfNecessary();

    $this->loginToNWAndGoTo( '/nodeworx/import?action=importSingle',
                             self::NW_USER,
                             self::NW_PASS );

    $this->open( '/nodeworx/import?action=importSingle' );
    $this->select( 'filetype', 'value=remote' );
    $this->click( 'submit' );
    $this->waitForPageToLoad();

    $backup_path = $this->_backupAccount();

    $this->type( 'remote_file', $backup_path );
    $this->click( 'submit' );

    $this->waitForPageToLoad();

    $this->click( 'submit' );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_RESTORE_SUCCESS##' );

    $homedir   = '/home/' . self::SW_UNIQNAME;
    $homeperms = substr( sprintf( '%o', fileperms( $homedir ) ), -4 );
    $this->assertNotEquals( '0700', $homeperms, "{$homedir} perms are bad" );
  }

  /**
   * Backs up the test siteworx account
   *
   * @return string
   */
  private function _backupAccount() {
    $domain = self::SW_DOMAIN;

    $tmp      = Ini::get( 'dir', 'tmp' );
    $path     = "{$tmp}/";
    $filename = 'backupforunittest';
    $tgz_path = "{$path}{$filename}.tgz";

    $backup = Ini::get( 'iworx.bin', 'backup' );

    $cmd = "$backup
      --domain {$domain}
      --output-dir={$path}
      --filename-format={$filename}";

    IWorxExec::exec( $cmd, $result, $retval );
    $this->assertTrue( $retval == 0 );

    return $tgz_path;
  }
}
?>