<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright ( c ) 2000-2008 InterWorx L.L.C., All Rights Reserved.     |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | ( INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @id $Id: PageNWResellersTest.php,v 1.3 2009/11/04 16:10:29 tcollege Exp $
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 2, 2008
 */

/**
 * Tests resellers.php in NodeWorx
 *
 * @group NodeWorx
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PageNWResellersTest extends IWorxSeleniumTestCase {

  const NW_RESELLER_NICK  = 'Reseller';
  const NW_RESELLER_EMAIL = 'reseller@interworx.co';
  const NW_RESELLER_PASS  = 'reseller';

  /**
   * testAddingResellerNWUser
   */
  public function testAddingResellerNWUser() {
    $email = 'reseller@interworx.co';

    $NodeWorx = new NodeWorx( $email );
    if( $NodeWorx->isReseller() ) {
      $this->testDeletingResellerNWUser();
    }

    $this->loginToNWAndGoTo( '/nodeworx/reseller' );
    $this->click( 'link=' . IWorxTranslate::tr( '##LG_ADDRESELLER##' ) );
    $this->waitForPageToLoad();

    $this->type( 'nickname', self::NW_RESELLER_NICK );
    $this->type( 'email', self::NW_RESELLER_EMAIL );
    $this->type( 'password', self::NW_RESELLER_PASS );
    $this->type( 'confirm_password', self::NW_RESELLER_PASS );
    $this->type( 'RSL_OPT_STORAGE', '15' );
    $this->type( 'RSL_OPT_BANDWIDTH', '15' );
    $this->type( 'RSL_OPT_EMAIL_ALIASES', '15' );
    $this->type( 'RSL_OPT_EMAIL_AUTORESPONDERS', '15' );
    $this->type( 'RSL_OPT_EMAIL_BOXES', '15' );
    $this->type( 'RSL_OPT_EMAIL_GROUPS', '15' );
    $this->type( 'RSL_OPT_FTP_ACCOUNTS', '15' );
    $this->type( 'RSL_OPT_MYSQL_DBS', '15' );
    $this->type( 'RSL_OPT_MYSQL_DB_USERS', '15' );
    $this->type( 'RSL_OPT_POINTER_DOMAINS', '15' );
    $this->type( 'RSL_OPT_SITEWORX_ACCOUNTS', '15' );
    $this->type( 'RSL_OPT_SLAVE_DOMAINS', '15' );
    $this->type( 'RSL_OPT_SUBDOMAINS', '15' );
    $this->select( 'RSL_OPT_OVERSELL_STORAGE', 'label=' .
                   IWorxTranslate::tr( '##LG_YES##' ) );

    $this->click( 'ips_0' );
    $this->click( 'database_servers_0' );

    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_RESELLERADDED##' );
  }

  /**
   * Logs in as the newly created user, just to be sure.  Also does initial
   * reseller setup, to ensure it works.
   */
  public function testResellerNWUserLoginWorks() {
    $this->loginToNWAndGoTo( '/nodeworx/dns?setup=true',
                             self::NW_RESELLER_EMAIL,
                             self::NW_RESELLER_PASS );

    $this->type( 'ns1', 'ns1.interworx.com' );
    $this->type( 'ns2', 'ns2.interworx.com' );
    $this->click( 'submit' );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SETTINGSUPDATEDSUCCESSFULLY##' );

    $this->waitForPageToLoad();

    $this->click( 'iw-menu-home' );
    $this->waitForPageToLoad();
    $this->assertTextPresent( self::NW_RESELLER_NICK );
  }

  /**
   * Tests deleting the reseller account
   */
  public function testDeletingResellerNWUser() {
    $this->loginToNWAndGoTo( '/nodeworx/reseller' );

    $this->click( 'link=Delete' );
    $this->waitForPageToLoad();
    $this->assertTextPresent( 'Please confirm the following action' );
    $this->assertTextPresent( 'reseller@interworx.co' );

    $this->click( "//input[@value='" .
                  IWorxTranslate::tr( '##LG_DELETE##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_RESELLERDELETED##',
                                        'Reseller delete sucess message missing' );
  }

}
?>