<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2009 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Nov 3, 2009
 * @id $Id: PageNWSimpleScriptsTest.php,v 1.1 2009/11/04 16:33:18 tcollege Exp $
 */

/**
 * Tests for SimpleScripts plugin
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PageNWSimpleScriptsTest extends IWorxSeleniumTestCase {

  /**
   * Performs a few pre-test actions
   */
  public function setUp() {
    parent::setUp();
    Plugin_Manager::clearInstance();
    Form::resetInstances();
  }

  /**
   * Tests that the plugin is initially disabled
   */
  public function testInitallyEnabled() {
    $this->loginToNWAndGoTo( 'nodeworx/plugins' );

    $status = $this->getIWMessageHTML( 'plug-stat-simplescripts' );

    $this->assertEquals( 'Enabled', trim( $status ),
                         'SimpleScripts already enabled!' );
  }

  /**
   * Verifies that elements are present in NodeWorx
   */
  public function testNWElementsPresent() {
    $this->loginToNWAndGoTo( 'nodeworx/siteworx?action=add&reseller=1' );

    $this->assertElementPresent( "//a[@id='menu-simplescripts']",
                                 'Menu item was not present while enabled' );

    $this->assertElementPresent( "//*[@id='simplescripts']",
                                 'SiteWorx Option not present while enabled' );

    $this->addSWAccountIfNecessary();
    $blocked = IW::PluginManager()->getPlugin( 'simplescripts' )
      ->isAccountBlocked( 'addedbyt' );

    $this->assertFalse( $blocked, 'ABT account was blocked' );

    $this->loginToNWAndGoTo( 'nodeworx/simplescripts' );

    $this->assertTextPresent( 'Host Key', 'The form appears to be missing' );
  }

  /**
   * Verifies that elements are present in SiteWorx
   */
  public function testSWElementsPresent() {
    $Form   = Form::getInstance( 'Form_SW_Users_Add' );
    $values = $Form->getInput( 'perms' )->getDataSource()->getValues();

    $this->assertContains( 'simplescripts', $values, 'Unable to find permission' );

    $this->loginToSWAndGoTo( 'siteworx/overview' );

    $this->assertElementPresent( "//a[@id='menu-simplescripts']",
                                 'Menu item was not present while enabled' );
  }

  /**
   * Attempts to disable SimpleScripts
   */
  public function testDisableSimpleScripts() {
    $this->loginToNWAndGoTo( 'nodeworx/plugins' );
    $this->clickAndWait( 'plug-simplescripts' );

    $this->select( 'status', 'value=0' );
    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTrue( $this->isTextPresent( 'Plugin Saved - ' ) );

    $this->waitForPageToLoad();

    $status = $this->getIWMessageHTML( 'plug-stat-simplescripts' );

    $this->assertEquals( 'Disabled', trim( $status ),
                         'SimpleScripts was not enabled!' );
  }

  /**
   * Verifies that elements are not present in NodeWorx
   */
  public function testNWElementsNotPresent() {
    $this->loginToNWAndGoTo( 'nodeworx/siteworx?action=add&reseller=1' );

    // Verify that the menu item is not there
    $this->assertElementNotPresent( "//a[@id='menu-simplescripts']",
                                    'Menu item was present while disabled' );

    // Verify that the SiteWorx Option is not there
    $this->assertElementNotPresent( "//*[@id='simplescripts']",
                                    'SiteWorx Option present while disabled' );

    $this->loginToNWAndGoTo( 'nodeworx/simplescripts?action=launch' );

    // Verify that accessing the controller fails
    $this->assertTextPresent( "Unknown Controller: 'Ctrl_Nodeworx_Simplescripts'",
                              'No error msg when trying to access SimpleScripts' );
  }

  /**
   * Verifies that SiteWorx elements are unavailable
   */
  public function testSWElementsNotPresent() {
    Ini::reload();
    $Form   = Form::getInstance( 'Form_SW_Users_Add' );
    $values = $Form->getInput( 'perms' )->getDataSource()->getValues();

    $this->assertNotContains( 'simplescripts',
                              $values,
                              'SimpleScripts perm was present' );

    $this->loginToSWAndGoTo( 'siteworx/overview' );

    $this->assertElementNotPresent( "//a[@id='menu-simplescripts']",
                                    'Menu item was not present while enabled' );
  }

  /**
   * Attempts to enable SimpleScripts
   */
  public function testEnableSimpleScripts() {
    $this->loginToNWAndGoTo( 'nodeworx/plugins' );
    $this->clickAndWait( 'plug-simplescripts' );

    $this->select( 'status', 'value=1' );
    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTrue( $this->isTextPresent( 'Plugin Saved - ' ) );

    $this->waitForPageToLoad();

    $status = $this->getIWMessageHTML( 'plug-stat-simplescripts' );

    $this->assertEquals( 'Enabled', trim( $status ),
                         'SimpleScripts was not enabled!' );
  }
}