<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @id $Id: PageNWUpdatesTest.php,v 1.2 2009/11/03 21:22:58 tcollege Exp $
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 */

/**
 * Tests for the Updates page
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PageNWUpdatesTest extends IWorxSeleniumTestCase {
  private $_rpms = array( 'cos4x' => 'http://updates.interworx.com/iworx/RPMS/cos4x/legacy/i386/spamassassin-3.2.4-100.rhe4x.iworx.i386.rpm',
                          'cos5x' => 'http://updates.interworx.com/iworx/RPMS/rhe5x/legacy/i386/spamassassin-3.2.4-100.rhe5x.iworx.i386.rpm' );


  /**
   * Force installs an old package
   */
  private function _forceInstallOldPackage() {
    $sysinfo = new IWorxSysInfo();

    $distro = $sysinfo->distro();

    $short_dist = false;

    if( strstr( $distro, 'CentOS release 5' ) ) {
      $short_dist = 'cos5x';
    } else if( strstr( $distro, 'CentOS release 4' ) ) {
      $short_dist = 'cos4x';
    }

    $this->assertNotEquals( false, $short_dist, 'Unable to determine distro' );

    $cmd = 'rpm -Uvh --force ' . $this->_rpms[ $short_dist ];

    IWorxExec::exec( $cmd );
  }

  /**
   * Checks if updates are available
   *
   * @return boolean
   */
  private function _isUpdatesAvailable() {
    return !( $this->isTranslatedTextPresent( '##LG_NOUPDATESPENDING##' ) );
  }

  /**
   * Checks if YUM is still running
   *
   * @return boolean
   */
  private function _yumIsStillRunning() {
    $js     = 'this.browserbot.getCurrentWindow().yumLogHandler.isStillRunning';
    $result =  $this->getEval( $js );

    return $result === 'true';
  }

  /**
   * Sets up the old package, if necessary
   */
  public function testUpdatesAvailable() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    if( $this->_isUpdatesAvailable() === false ) {
      $this->_forceInstallOldPackage();
      $NW = IWorx_Framework_TestCase::getMasterNodeWorx();

      $result = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Updates',
                                        'refresh',
                                        array(),
                                        $NW );

      $this->assertTrue( $result, 'Unable to refresh updates' );
    }
  }

  /**
   * Doublechecks that the force install worked.
   */
  public function testUpdatesAvailableFollowup() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    $this->assertTranslatedTextNotPresent( '##LG_NOUPDATESPENDING##',
                                           'Force install of old RPM failed' );
  }

  /**
   * Verifies that the install button is enabled
   */
  public function testInstallButtonEnabled() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    $enabled = $this->getElementAttributeById( 'install-updates-btn', 'disabled' );

    $this->assertEquals( 'null', $enabled, 'Install button is not enabled' );
  }

  /**
   * Installs the updates
   */
  public function testInstallingUpdates() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    if( $this->_isUpdatesAvailable() === true ) {
      $this->click( 'install-updates-btn' );
      sleep( 1 );
      $this->assertVisible( 'iw-file-output_h', 'Yum output not visible' );

      while( $this->_yumIsStillRunning() ) {
        // Sleep a bit to wait for the install to complete
        sleep( 5 );
      }
    }
  }

  /**
   * Checks that updates applied
   */
  public function testInstallWorked() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    $this->assertFalse( $this->_isUpdatesAvailable() );
  }
}