<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date Aug 6, 2008
 * @id $Id: PageNWUsersTest.php,v 1.3 2009/11/04 16:10:29 tcollege Exp $
 */

/**
 * Test everything on the nodeworx/users.php page
 *
 * @group NodeWorx
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PageNWUsersTest extends IWorxSeleniumTestCase {

  /**
   * Gets all the permissions for NodeWorx
   *
   * @return array
   */
  public static function getAllNWPerms() {
    $cnt = count( NodeWorx::getPermsList() );
    $ids = array();

    for( $i = 0; $i < $cnt - 1; $i++ ) {
      $ids[] = array( 'short_name' => "perms_{$i}" );
    }
    return $ids;
  }

  /**
   * Gets the NodeWorx user ID for the email
   *
   * @param string $email
   * @return integer
   */
  public function getNWUserId( $email ) {
    if( !$email ) {
      return false;
    }

    $NodeWorx = new NodeWorx( $email );

    $user = $NodeWorx->getWorkingUser();

    if( !$user ) {
      return false;
    }

    $user_ID = $NodeWorx->getWorkingUser()->getId();

    $this->assertTrue( IWorxValidate::integer( $user_ID, 2 ),
                       'Unable to deterimine NodeWorx ID' );

    return intval( $user_ID );
  }

  /**
   * Fills in the form to add a NW user
   */
  public function fillInAddSecondaryNWUserForm() {
    $this->type( 'nickname', self::SECONDARY_NICK );
    $this->type( 'email', self::SECONDARY_EMAIL );
    $this->type( 'password', self::SECONDARY_PASS );
    $this->type( 'confirm_password', self::SECONDARY_PASS );
    $this->select( 'theme', 'value=interworx' );
    $this->click( 'link=' . IWorxTranslate::tr( '##LG_CHECKALL##' ) );
  }

  /**
   * Clears the form to add a NW User
   */
  public function clearSecondaryNWUserForm() {
    $this->type( 'nickname', '' );
    $this->type( 'email', '' );
    $this->type( 'password', '' );
    $this->type( 'confirm_password', '' );
    $this->click( 'link=' . IWorxTranslate::tr( '##LG_CHECKALL##' ) );
  }

  /**
   * Deletes the test users if they already exist
   *
   * The setup function is called before EVERY test, so I had to put this here.
   *
   * @group NW_User_Secondary
   */
  public function testDeleteSecondaryTestUser() {
    // Clear out the Secondary NodeWorx Test User if exists
    $user_ID = $this->getNWUserId( self::SECONDARY_EMAIL );

    if( $user_ID ) {
      $this->assertTrue( NodeWorx::deleteNodeWorxUser( $user_ID ),
                         'Unable to delete pre-existing user' );
    }
  }

  /**
   * Verifies default is unchecked, tests check all and check none
   *
   * @group js
   * @group NW_User_Secondary
   */
  public function testAssingSecondaryNWUserCheckAllNone() {
    $this->loginToNWAndGoTo( '/nodeworx/users?action=add' );

    $perms = self::getAllNWPerms();

    foreach( $perms as $perm ) {
      $this->assertFalse( $this->isChecked( $perm['short_name'] ),
                          'Permission ' . $perm['short_name'] . ' was checked' );
    }

    $this->click( 'link=check all' );

    foreach( $perms as $perm ) {
      $this->assertTrue( $this->isChecked( $perm['short_name'] ),
                         'Permission ' .
                         $perm['short_name'] .
                         ' was not checked after "check all"' );
    }

    $this->click( 'link=check none' );

    foreach( $perms as $perm ) {
      $this->assertFalse( $this->isChecked( $perm['short_name'] ),
                          'Permission ' .
                          $perm['short_name'] .
                          ' was still checked after "check none"' );
    }
  }

  /**
   * Test adding secondary users required data
   *
   * @group NW_User_Secondary
   */
  public function testAddingSecondaryNWUserRequiredData() {
    $this->loginToNWAndGoTo( '/nodeworx/users?action=add' );

    // Blank user name
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'nickname', '' );
    $this->clickSave();
    $this->clearSecondaryNWUserForm();

    // Bad user name
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'nickname', 'Bad#$!@Chars!' );
    $this->clickSave();
    $this->clearSecondaryNWUserForm();

    // blank email
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'email', '' );
    $this->clickSave();
    $this->clearSecondaryNWUserForm();

    // bad email
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'email', 'tcollege@interworx' );
    $this->clickSave();
    $this->clearSecondaryNWUserForm();

    // blank passwords
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'password', '' );
    $this->type( 'confirm_password', '' );
    $this->clickSave();
    $this->clearSecondaryNWUserForm();

    // blank password 1
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'password', '' );
    $this->clickSave();
    $this->waitForMessageBanner();
    $this->clearSecondaryNWUserForm();

    // blank password 2
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'confirm_password', '' );
    $this->clickSave();
    $this->clearSecondaryNWUserForm();

    // mismatched passwords
    $this->fillInAddSecondaryNWUserForm();
    $this->type( 'password', self::SECONDARY_EMAIL . 'mismatch' );
    $this->clickSave();
  }

  /**
   * Adds a NodeWorx User
   *
   * @group NW_User_Secondary
   */
  public function testAddingSecondaryNWUser() {
    $this->loginToNWAndGoTo( '/nodeworx/users?action=add' );

    if( $this->getNWUserId( self::SECONDARY_EMAIL ) ) {
      $this->fail( 'User already exists, cannot add!' );
    }

    $this->fillInAddSecondaryNWUserForm();

    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTrue( $this->isTextPresent( 'NodeWorx user successfully added' ) );

    $user_ID = $this->getNWUserId( self::SECONDARY_EMAIL );
    $this->assertGreaterThan( 2, $user_ID, 'User was not added properly' );
  }

  /**
   * Verifies setup from testAddingSecondaryNWUser()
   *
   * @group NW_User_Secondary
   */
  public function testSecondaryNWUserInformationIsCorrect() {
    $this->loginToNWAndGoTo( '/nodeworx/users' );

    $user_ID = $this->getNWUserId( self::SECONDARY_EMAIL );

    $path = "//a[contains(@href, 'action=edit')][contains(@href, 'id=$user_ID')]";
    $this->click( $path );
    $this->waitForPageToLoad();
    $this->assertPageRenderedWithoutErrors();
    $this->assertEquals( self::SECONDARY_NICK, $this->getValue( 'nickname' ) );
    $this->assertEquals( self::SECONDARY_EMAIL, $this->getValue( 'email' ) );
    $this->assertEquals( 'interworx', $this->getValue( 'theme' ) );

    foreach( self::getAllNWPerms() as $perm ) {
      $this->assertTrue( $this->isChecked( $perm['short_name'] ) );
    }
  }

  /**
   * Logs in as the newly created user, just to be sure.
   *
   * @group NW_User_Secondary
   */
  public function testSecondaryNWUserLoginWorks() {
    $this->loginToNWAndGoTo( '/nodeworx/overview',
                             self::SECONDARY_EMAIL,
                             self::SECONDARY_PASS );
    $this->assertTextPresent( self::SECONDARY_NICK );
  }

  /**
   * deletes the NodeWorx User
   *
   * @group NW_User_Secondary
   */
  public function testDeletingSecondaryNWUser() {
    $this->loginToNWAndGoTo( '/nodeworx/users' );

    $user_ID = $this->getNWUserId( self::SECONDARY_EMAIL );

    $path = "//a[contains(@href, 'action=delete')][contains(@href, 'id=$user_ID')]";
    $this->click( $path );
    $this->waitForPageToLoad();
    $this->assertTextPresent( 'Confirm deletion' );
    $this->assertTextPresent( self::SECONDARY_EMAIL );

    $path = "//input[@value='" . IWorxTranslate::tr( '##LG_DELETE##' ) . "']";
    $this->click( $path );

    $this->waitForMessageBanner();

    $this->assertTranslatedTextPresent( '##LG_NODEWORX_USER_DELETE_SUCCESS##' );
  }
}
?>