<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Aug 28, 2009
 * @id $Id: PageSWHtaccessDirindexTest.php,v 1.1 2009/08/28 20:08:12 jdaniel Exp $
 */

/**
 * Htaccess Directory Index Test
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PageSWHtaccessDirindexTest extends IWorxSeleniumTestCase {

  /**
   * Enables global htaccess
   */
  public function testEnableGlobalHtaccess() {
    $this->addSWAccountIfNecessary();
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );

    $this->loginToSWAndGoTo( '/siteworx/htaccess/' );

    // add it
    $this->click( 'add_global_subbutton' );
    $this->waitForPageToLoad();

    // enable it
    $this->click( 'add_global_subbutton' );
    $this->waitForPageToLoad();

    $this->assertPageRenderedWithoutErrors();
  }

  /**
   * Tests the dir index page
   */
  public function testDirindexPage() {
    $this->addSWAccountIfNecessary();
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );

    $this->loginToSWAndGoTo( '/siteworx/htaccess/' );

    $expected_indexes      = 'index.html';
    $expected_options      = 'On';
    $expected_indexoptions = 'Simple Listing';

    $this->click( 'link=Index Management' );
    $this->waitForPageToLoad();

    $this->type( 'default_indexes', 'index.html' );
    $this->select( 'options', 'label=On' );
    $this->select( 'indexoptions', 'label=Simple Listing' );

    $this->click( "//input[@value='Update']" );

    $this->waitForPageToLoad();
    $this->assertPageRenderedWithoutErrors();

    $htaccess = new HtaccessFile( $SiteWorx );
    $htaccess->setHtaccessDir( 'global' );

    $dirindex = $htaccess->getHtaccessInfo( 'dirindex' );

    $this->assertEquals( $dirindex['default_indexes']['index.html'],
                         $expected_indexes );
    $this->assertEquals( $dirindex['options']['indexes'], 'on' );
    $this->assertEquals( $dirindex['indexoptions']['style'], 'simple' );
  }

  /**
   * deletes the global htaccess
   */
  public function testDeleteGlobalHtaccess() {
    $this->addSWAccountIfNecessary();
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );

    $this->loginToSWAndGoTo( '/siteworx/htaccess/' );

    $this->click( "//input[@value='Delete']" );
    $this->waitForPageToLoad();
    $this->assertPageRenderedWithoutErrors();

    $this->click( 'submit' );
    $this->waitForPageToLoad();

    $this->assertPageRenderedWithoutErrors();
  }
}
?>