<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Oct 16, 2008
 * @id $Id: PageSWMySqlTest.php,v 1.2 2009/08/21 02:30:05 poehler Exp $
 */

/**
 * Tests for the mysql pages of SiteWorx
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PageSWMySqlTest extends IWorxSeleniumTestCase {

  const MYSQL_PREFIX   = 'addedbyt_';
  const MYSQL_USERNAME = 'db_test';
  const MYSQL_PASSWORD = 'drizzle';
  const MYSQL_DBNAME   = 'testdb';
  /**
   * fillInMySqlUserForm function
   */
  public function fillInMySqlUserForm() {
    $this->type( 'name',  self::MYSQL_USERNAME );
    $this->type( 'password', self::MYSQL_PASSWORD );
    $this->type( 'confirm_password', self::MYSQL_PASSWORD );
  }
  /**
   * fillInMySqlDBForm function
   */
  public function fillInMySqlDBForm() {
    $this->type( 'name', self::MYSQL_DBNAME );
  }
  /**
   * clearMySQLUserForm function
   */
  public function clearMySQLUserForm() {
    $this->type( 'name',  '' );
    $this->type( 'password', '' );
    $this->type( 'confirm_password', '' );
  }

  /**
   * testUserDeletion function
   * This "Test" function must be executed before any other normal test
   * functions because to ensure that no old test data exists.
   */
  public function testUserDeletion() {
    $siteworx = new SiteWorx( self::SW_DOMAIN );
    $driver   = $siteworx->getMySQLDriver();

    if( $driver->userExists( self::MYSQL_USERNAME ) ) {
      $driver->deleteUser( self::MYSQL_USERNAME );
    }
    $this->assertTrue( !$driver->userExists( self::MYSQL_USERNAME ) );
  }

  /**
   * testDatabaseDeletion function
   * This "Test" function must be executed before any other normal test
   * functions because to ensure that no old test data exists.
   */
  public function testDatabaseDeletion() {
    $siteworx = new SiteWorx( self::SW_DOMAIN );
    $driver   = $siteworx->getMySQLDriver();

    if( $driver->databaseExists( self::MYSQL_DBNAME ) ) {
      $driver->deleteDatabase( self::MYSQL_DBNAME );
    }
    $this->assertTrue( !$driver->databaseExists( self::MYSQL_DBNAME ) );
  }

  /**
   * testAddingDBRequiredData function
   */
  public function testAddingDBRequiredData() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/db' );
    $this->type( 'name', '' );

    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    sleep( 1 );

    $title = $this->getElementAttributeById( 'name_status_span', 'title' );
    $this->assertEquals(
                        IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                        $title );
  }

  /**
   * testAddedMySQLDatabase function
   */
  public function testAddingMySQLDatabase() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/db' );

    $this->fillInMySqlDBForm();
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTextPresent( 'Database created successfully',
                              'Missing success notice on ' .
                              'database creation.' );
  }

  /**
   * testAddingMySqlUserRequiredData function
   */
  public function testAddingMySqlUserReqiredData() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/user' );

    // blank username
    $this->fillInMySqlUserForm();
    $this->type( 'name', '' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $title = $this->getElementAttributeById( 'name_status_span', 'title' );
    $this->assertEquals(
                        IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                        $title );

    $this->clearMySQLUserForm();

    // blank password 1
    $this->fillInMySqlUserForm();
    $this->type( 'password', '' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $title = $this->getElementAttributeById(
                                            'password_status_span',
                                            'title' );
    $this->assertEquals(
                        IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                        $title );

    $this->clearMySQLUserForm();

    // blank password 2
    $this->fillInMySqlUserForm();
    $this->type( 'confirm_password', '' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $title = $this->getElementAttributeById( 'confirm_password_status_span',
                                             'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                         $title );

    $this->clearMySQLUserForm();

    // mismatched passwords
    $this->fillInMySqlUserForm();
    $this->type( 'confirm_password', 'darnbobbytables' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    sleep( 3 );

    $title = $this->getElementAttributeById( 'confirm_password_status_span',
                                             'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_PASSWORDSDONOTMATCH##' ),
                         $title );

    $this->clearMySQLUserForm();
  }

  /**
   * testAddingMySqlUser function
   */
  public function testAddingMySqlUser() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/user' );

    $this->fillInMySqlUserForm();
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTextPresent( 'User added successfully',
                              'Error message missing ' .
                              'for successfully adding a user' );

    $this->assertTranslatedTextPresent( '##LG_DATABASE_USER_ADDEDSUCCESSFULLY##',
                                        'Error message missing ' .
                                        'for successfully adding a user' );
  }
}
?>