<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Oct 16, 2008
 * @id $Id: PageSWUsersTest.php,v 1.2 2009/11/04 16:10:03 tcollege Exp $
 */

/**
 * Tests for the users page of SiteWorx
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PageSWUsersTest extends IWorxSeleniumTestCase {

  /**
   * Gets all the permissions for SiteWorx
   *
   * @return array
   */
  public static function getAllSWPerms() {
    $cnt = count( SiteWorx::getPermsList() );
    $ids = array();

    for( $i = 0; $i < $cnt - 1; $i++ ) {
      $ids[] = array( 'short_name' => "perms_{$i}" );
    }
    return $ids;
  }

  /**
   * Gets the SiteWorx user ID for the email
   *
   * @param string $domain
   * @param string $email
   * @return integer
   */
  public function getSWUserId( $domain, $email ) {
    return IWorx_Framework_TestCase::getSWUserId( $domain, $email );
  }

  /**
   * Fills in the form to add a SW user
   */
  public function fillInAddSecondarySWUserForm() {
    $this->type( 'nickname',  self::SW_SECONDARY_NICK );
    $this->type( 'email',    self::SW_SECONDARY_EMAIL );
    $this->type( 'password', self::SW_SECONDARY_PASS );
    $this->type( 'confirm_password', self::SW_SECONDARY_PASS );
    $this->click( 'link=' . IWorxTranslate::tr( '##LG_CHECKALL##' ) );
  }

  /**
   * Clears the form to add a SW User
   */
  public function clearSecondarySWUserForm() {
    $this->type( 'nickname', '' );
    $this->type( 'email', '' );
    $this->type( 'password', '' );
    $this->type( 'confirm_password', '' );
    $this->click( 'link=' . IWorxTranslate::tr( '##LG_CHECKALL##' ) );
  }

  /**
   * Deletes the test users if they already exist
   *
   * The setup function is called before EVERY test, so I had to put this here.
   */
  public function testDeleteSecondaryTestUser() {
    // Clear out the Secondary SiteWorx Test User if exists
    $user_ID = $this->getSWUserId( self::SW_DOMAIN, self::SW_SECONDARY_EMAIL );

    if( !empty( $user_ID ) ) {
      $siteworx = new SiteWorx( self::SW_DOMAIN );
      $this->assertTrue( $siteworx->deleteSiteWorxUser( $user_ID ),
                         'Unable to delete pre-existing user' );
    }
  }

  /**
   * Verifies default is unchecked, tests check all and check none
   */
  public function testAssingSecondarySWUserCheckAllNone() {
    $this->loginToSWAndGoTo( '/siteworx/users?action=add' );

    $perms = self::getAllSWPerms();

    foreach( $perms as $perm ) {
      $this->assertFalse( $this->isChecked( $perm['short_name'] ),
                          'Permission ' . $perm['short_name'] . ' was checked' );
    }

    $this->click( 'link=check all' );

    foreach( $perms as $perm ) {
      $this->assertTrue( $this->isChecked( $perm['short_name'] ),
                         'Permission ' .
                         $perm['short_name'] .
                         ' was not checked after "check all"' );
    }

    $this->click( 'link=check none' );

    foreach( $perms as $perm ) {
      $this->assertFalse( $this->isChecked( $perm['short_name'] ),
                          'Permission ' .
                          $perm['short_name'] .
                          ' was still checked after "check none"' );
    }
  }

  /**
   * Tests required data for secondary users
   */
  public function testAddingSecondarySWUserRequiredData() {
    $this->loginToSWAndGoTo( '/siteworx/users?action=add' );

    // blank email
    $this->fillInAddSecondarySWUserForm();
    $this->type( 'email', '' );
    $this->clickSave();

    $title = $this->getElementAttributeById( 'email_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );

    $this->clearSecondarySWUserForm();

    // bad email
    $this->fillInAddSecondarySWUserForm();
    $this->type( 'email', 'tcollege@interworx' );
    /*
    $title = $this->getElementAttributeById( 'email_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_EMAIL_INVALID##' ),
      $title );
    */
    $this->clearSecondarySWUserForm();

    // blank passwords
    $this->fillInAddSecondarySWUserForm();
    $this->type( 'password', '' );
    $this->type( 'confirm_password', '' );
    $this->clickSave();

    $title = $this->getElementAttributeById( 'password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );
    $this->clearSecondarySWUserForm();

    // blank password 1
    $this->fillInAddSecondarySWUserForm();
    $this->type( 'password', '' );
    $this->clickSave();

    $title = $this->getElementAttributeById( 'password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );
    $this->clearSecondarySWUserForm();

    // blank password 2
    $this->fillInAddSecondarySWUserForm();
    $this->type( 'confirm_password', '' );
    $this->clickSave();
    $title =
      $this->getElementAttributeById( 'confirm_password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );
    $this->clearSecondarySWUserForm();

    // mismatched passwords
    $this->fillInAddSecondarySWUserForm();
    $this->type( 'password', self::SW_SECONDARY_EMAIL . 'mismatch' );
    $this->clickSave();

    $this->waitForMessageBanner();

    $title =
      $this->getElementAttributeById( 'confirm_password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_PASSWORDSDONOTMATCH##' ),
                         $title );
  }

  /**
   * Adds a SiteWorx User
   */
  public function testAddingSecondarySWUser() {
    $this->loginToSWAndGoTo( '/siteworx/users?action=add' );

    if( $this->getSWUserId(  self::SW_DOMAIN, self::SW_SECONDARY_EMAIL  ) ) {
      $this->fail( 'User already exists, cannot add!' );
    }

    $this->fillInAddSecondarySWUserForm();

    $this->clickSave();

    $this->waitForMessageBanner();
    $this->assertTextPresent( 'SiteWorx user successfully added' );

    $user_ID = $this->getSWUserId(  self::SW_DOMAIN, self::SW_SECONDARY_EMAIL  );
    $this->assertGreaterThan( 2, $user_ID, 'User was not added properly' );
  }

  /**
   * Verifies setup from testAddingSecondarySWUser()
   */
  public function testSecondarySWUserInformationIsCorrect() {
    $this->loginToSWAndGoTo( '/siteworx/users' );

    $user_ID = $this->getSWUserId(  self::SW_DOMAIN, self::SW_SECONDARY_EMAIL  );

    $path = "//a[contains(@href, 'action=edit')][contains(@href, 'id=$user_ID')]";
    $this->click( $path );
    $this->waitForPageToLoad();
    $this->assertPageRenderedWithoutErrors();
    $this->assertEquals( self::SW_SECONDARY_NICK, $this->getValue( 'nickname' ) );
    $this->assertEquals( self::SW_SECONDARY_EMAIL, $this->getValue( 'email' ) );

    foreach( self::getAllSWPerms() as $perm ) {
      $this->assertTrue( $this->isChecked( $perm['short_name'] ) );
    }
  }

  /**
   * Logs in as the newly created user, just to be sure.
   */
  public function testSecondarySWUserLoginWorks() {
    $this->loginToSWAndGoTo( '/siteworx/overview',
                             self::SW_SECONDARY_EMAIL,
                             self::SW_SECONDARY_PASS,
                             self::SW_DOMAIN );
    $this->assertTextPresent( self::SW_SECONDARY_EMAIL );
  }

  /**
   * deletes the SiteWorx User
   */
  public function testDeletingSecondarySWUser() {
    $this->loginToSWAndGoTo( '/siteworx/users' );

    $user_ID = $this->getSWUserId(  self::SW_DOMAIN, self::SW_SECONDARY_EMAIL  );

    $path = "//a[contains(@href, 'action=del')][contains(@href, 'id=$user_ID')]";
    $this->click( $path );
    $this->waitForPageToLoad();
    $this->assertTextPresent( 'Confirm' );
    $this->assertTextPresent( self::SW_SECONDARY_EMAIL );

    $path = "//input[@value='" . IWorxTranslate::tr( '##LG_DELETE##' ) . "']";
    $this->click( $path );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SITEWORX_USER_DELETE_SUCCESS##' );
  }
}
?>