<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @id $Id: SpiderAllLinksTest.php,v 1.24 2009/07/28 15:13:01 poehler Exp $
 */

/**
 * Test that spiders the entire NodeWorx menu looking for errors
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class SpiderAllLinksTest extends IWorxSeleniumTestCase {

  const MAX_PATTERN = 3;
  const MAX_HYPERLINK_NUM = 300;

  /**
   * Array of strings that we don't want to be in links that we follow
   *
   * @var array
   */
  private $_bad_link_strings =
  array( 'action=logout',
         'action=login',
         'nodeworx/download',
         'siteworx/download',
         'action=download',
         '.gif',
         'action=server_info',
         'action=server_status',
         'phpinfo',
         'support.php',
         '2080/mysql',
         '2080/webmail',
         'cgi-bin/fileman',
         'webalizer',
         'analog?',
         'awstats?',
         'siteworx/mysql/phpmyadmin',
         '/nodeworx/siteworx?action=siteworxLoginCommit&domain' );

  /**
   * Queue of links to follow
   *
   * @var array
   */
  private $_link_queue = array();

  /**
   * Array of link IDs that we've seen before - primarly for bookkeepping
   *
   * @var array
   */
  private $_seen_ids = array();

  /**
   * Array of filtered query string links.
   *
   * key is the filtered url, value is the number of times the pattern has been
   * spidered
   *
   * @var array
   */
  private $_seen_querystring_patterns = array();

  /**
   * Array of links we've processed already
   *
   * @var array
   */
  private $_visited_links = array();

  /**
   * Array of errors we've errors we've encountered on our journey
   *
   * @var array
   */
  private $_errors = array();

  /**
   * url to the phpunit coverage start script
   *
   * @var string
   */
  //  protected $coverageScriptUrl = 'http://dev.interworx.com:2080/nodeworx/phpunit_coverage.php';

  /**
   * setup
   */
  public function setUp() {
    parent::setUp();
    $this->_link_queue    = array();
    $this->_seen_ids      = array();
    $this->_visited_links = array();
    $this->_errors        = array();
  }

  /**
   * Spiders all the links in nodeworx, looking for errors
   */
  public function testSpiderAllNodeworxLinks() {
    $this->open( '/nodeworx/' );
    $this->loginToNWAsUser();
    $this->_spider();
    $this->_displayReports();
    $this->_failOnError();
  }

  /**
   * Spiders all the links in siteworx, looking for errors
   */
  public function testSpiderAllSiteworxLinks() {
    $this->addSWAccountIfNecessary();
    $this->open( '/siteworx/' );
    $this->loginToSWAsUser();
    $this->_spider();
    $this->_displayReports();
    $this->_failOnError();
  }

  /**
   * Display Reports
   */
  private function _displayReports() {
    $this->_displayErrors();
    $this->_displayStats();
  }

  /**
   * Display Errors
   */
  private function _displayErrors() {
    if( count( $this->_errors ) > 0 ) {
      print( "\n" );
      foreach( $this->_errors as $error ) {
        print( "$error\n\n" );
      }
      print( "\n" );
    }
  }

  /**
   * Fails of there were errors detected
   */
  private function _failOnError() {
    if( count( $this->_errors ) > 0 ) {
      $this->fail( implode( "\n", $this->_errors ) );
    }
  }

  /**
   * display spidering stats
   */
  private function _displayStats() {
    print( "\n" . 'total links spidered: ' . count( $this->_visited_links ) . "\n" );
  }

  /**
   * Spider
   */
  private function _spider() {
    // Make sure the menu exists
    $this->waitForElement( 'iw-menu' );
    $this->_link_queue[] = $this->getEval( 'window.document.location' );
    $this->_runLinkQueue();
  }

  /**
   * Run the link queue, checking for errors on each page
   */
  private function _runLinkQueue() {
    $queue_count = count( $this->_link_queue );

    $href = null;

    for( $i = 0; $i < $queue_count; $i++ ) {
      $this->_addIdsToLinksOnPage();
      $this->_addLinksToQueue( $href );

      $href = $this->_link_queue[ $i ];

      $this->_doErrorChecks( $href );

      if( isset( $this->_link_queue[ $i + 1 ] ) ) {
        $href = $this->_link_queue[ $i + 1 ];

        while( $this->_hrefQueryStringPatternLimitedReached( $href ) ) {
          $i++;
          $href = $this->_link_queue[ $i + 1 ];
        }

        if( $href != '' ) {
          $this->_visited_links[] = $href;
          $this->open( $href );
        }
      }

      $queue_count = count( $this->_link_queue );
    }//end for
  }

  /**
   * Perform the actual error checking on the page content
   *
   * @param string $href
   */
  private function _doErrorChecks( $href ) {
    $check_translation = true;

    try {
      $this->assertPageRenderedWithoutErrors( $check_translation );
      $this->_assertLoginPageNotPresent( $href );
      $this->_assertMenuPresent( $href );
      $this->_assertNoFormInputErrors( $href );
      echo '.';
    } catch( PHPUnit_Framework_ExpectationFailedException $e ) {
      $this->_errors[] = $e->getMessage();
      echo 'f';
    }
  }

  /**
   * assert login page not present
   *
   * @param string $href
   */
  private function _assertLoginPageNotPresent( $href ) {
    $msg = $this->_getSpiderMessage( $href, 'Login Screen Returned' );
    $this->assertElementNotPresent( "@id='iw-login'", $msg );
  }

  /**
   * assert menu present
   *
   * @param string $href
   */
  private function _assertMenuPresent( $href ) {
    $msg = $this->_getSpiderMessage( $href, 'Menu Not Present' );
    $this->assertElementPresent( "//ul[@id='iw-menu']", $msg );
  }

  /**
   * assert no form input errors
   *
   * @param string $href
   */
  private function _assertNoFormInputErrors( $href ) {
    $htmlcontent = $this->getHtmlSource();
    $pos = strpos( $htmlcontent, 'iw-form-error' );

    $msg = $this->_getSpiderMessage( $href, 'Found Form Input Errors' );
    $this->assertFalse( $pos, $msg );
  }

  /**
   * Builds a message for output
   *
   * @param string $id
   * @param string $message
   * @return string
   */
  private function _getSpiderMessage( $id, $message ) {
    return $message . ': for ID ' . $id;
  }

  /**
   * For the links on the page that do not have ids, add them so getAllLinks works
   */
  private function _addIdsToLinksOnPage() {
    $add_link_id_js = "
var i = 0,
    k = 0;
for( i = 0; i < window.document.getElementsByTagName('a').length; i++ ) {
  if( window.document.getElementsByTagName('a')[i].id == '' ) {
    if( window.document.getElementsByTagName('a')[i].className != 'quickhelp' ) {
      window.document.getElementsByTagName('a')[i].id = 'hl_' + i;
      k++;
    }
  }
}
k;";
    $this->getEval( $add_link_id_js );
  }

  /**
   * Add new links that we find to the link queue
   *
   * @param string $src_href used for logging purposes
   */
  private function _addLinksToQueue( $src_href ) {
    $all_links = $this->getAllLinks();

    foreach( $all_links as $id ) {

      if( trim( $id ) == '' ) {
        continue;
      }

      if( $this->_idIsValid( $id ) ) {
        $this->_seen_ids[ $id ] = $id;

        try {
          $href = $this->_getHrefFromId( $id );
        } catch( Exception $e ) {
          echo $e->getMessage(), "id: $id", "\n";
          $this->_errors[] = $e->getMessage() . " id: $id";
          $href = '';
        }

        if( $this->_hrefIsValid( $href ) ) {
          //print( "adding $href to spider queue\n" );
          IWorxLog::log( "From: {$src_href} -- Adding: {$href}" );
          $this->_link_queue[] = $href;
        }//end if href is valid

      }//end if id is valid

    }//end foreach link
  }

  /**
   * Determine if this link id is one we want to process further
   *
   * @param string $id
   * @return boolean
   */
  private function _idIsValid( $id ) {
    return $this->_linkIdIsAnExistingMenuLink( $id ) === false;
  }

  /**
   * Test if the link id is a menu link we've already seen (if so we can skip)
   *
   * This speeds things up since $this->getAttribute() is slow
   *
   * @param string $id
   * @return boolean
   */
  private function _linkIdIsAnExistingMenuLink( $id ) {
    if( in_array( $id, $this->_seen_ids ) === true &&
        substr( $id, 0, 8 ) === 'iw-menu-' ) {

      return true;
    }
    return false;
  }

  /**
   * Gets the actual link href from the browser, given the id
   *
   * @param string $id
   * @return string
   */
  private function _getHrefFromId( $id ) {
    $href = '';
    $hl_count = substr( $href, 3 );
    if( $hl_count < self::MAX_HYPERLINK_NUM ) {
      $jscode = "window.document.getElementById( \"$id\" ).href";
      $href = trim( $this->getEval( $jscode ), '#' );
    }
    return $href;
  }

  /**
   * Determine if this href is one we want to spider
   *
   * @param string $href
   * @return boolean
   */
  private function _hrefIsValid( $href ) {
    return
      $this->_hrefContainsInitialUrl( $href )   === true &&
      $this->_hrefIsBlocked( $href )            === false &&
      $this->_hrefIsAlreadyInLinkQueue( $href ) === false;
  }

  /**
   * Determine if the href contains the initial URL
   *
   * For example, if the initial url was http://1.1.1.1:2080/
   * then http://1.1.1.1:2080/nodeworx "contains" the initial url
   *
   * @param string $href
   * @return boolean
   */
  private function _hrefContainsInitialUrl( $href ) {
    if( strstr( $href, $this->getBrowserUrl() ) !== false ) {
      return true;
    }
    return false;
  }

  /**
   * Determine if the href contains a blocked string
   *
   * @param string $href
   * @return boolean
   */
  private function _hrefIsBlocked( $href ) {
    foreach( $this->_bad_link_strings as $bad_link ) {
      if( strstr( $href, $bad_link ) !== false ) {
        return true;
      }
    }
    return false;
  }

  /**
   * Determine if href is already in the link queue or not
   *
   * @param string $href
   * @return boolean
   */
  private function _hrefIsAlreadyInLinkQueue( $href ) {
    if( in_array( $href, $this->_link_queue ) === true ) {
      return true;
    }
    return false;
  }

  /**
   * Checks whether a given href with similar query string vars has been spidered
   * the limit number of times
   *
   * @param string $href
   * @return boolean
   */
  private function _hrefQueryStringPatternLimitedReached( $href ) {
    $filtered = $this->_convertToQueryStringPattern( $href );
    if( $filtered !== false &&
        $this->_seen_querystring_patterns[ $filtered ] > self::MAX_PATTERN ) {

      return true;
    }

    if( $filtered !== false ) {
      $this->_seen_querystring_patterns[ $filtered ]++;
    }

    return false;
  }

  /**
   * turns a url like
   *
   * http://dev.interworx.com:2080/siteworx/htaccess/hotlink?dir=/home/addedbyt/addedbytest.com/html/ocportal/themes/default/templates
   *
   * into
   *
   * http://dev.interworx.com:2080/siteworx/htaccess/hotlink?dir=*
   *
   * @param string $href
   * @return string
   */
  private function _convertToQueryStringPattern( $href ) {

    if( false !== strpos( $href, '?' ) ) {
      list( $loc, $query ) = split( '\?', $href );

      $new_query = '';
      $i = 0;
      foreach( split( '&', $query ) as $var ) {
        $prev = '';
        foreach( split( '=', $var ) as $querypart ) {
          ++$i;
          if( $i % 2 ) {
            $new_query .= $querypart;
          } else {
            if( $prev === 'action' ) {
              $new_query .= "={$querypart}&";
            } else {
              $new_query .= '=*&';
            }
          }
          $prev = $querypart;
        }
      }

      $result = rtrim( "{$loc}?{$new_query}", '&' );
    } else {
      $result = false;
    }//end if
    return $result;
  }

}
?>