<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jan 15, 2009
 * @id $Id: Bug_1085_FixTest.php,v 1.2 2009/08/21 02:29:51 poehler Exp $
 */

/**
 * Tests to ensure Bug #1085 was corrected
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Bug_1085_FixTest extends IWorxSeleniumTestCase {

  /**
   * AddIpToNodeWorx
   *
   * @param string $ip_range_start
   * @param string $ip_range_end
   * @param array  $args
   */
  private function _AddIpToNodeWorx( $ip_range_start,
                                     $ip_range_end,
                                     array $args = array() ) {

    $this->loginToNWAndGoTo( '/nodeworx/ip' );

    $this->type( 'ip_start', $ip_range_start );
    $this->type( 'ip_end', $ip_range_end );

    if( !empty( $args ) ) {
      foreach( $args as $key => $label ) {
        $this->select( $key, "label=$label" );
      }
    }
    $this->click( 'submit' );

    $result = $this->getMessageBannerHTML();

    $this->assertTrue( !empty( $result ) );
  }

  /**
   * testAddIPWithResellerAccess
   */
  public function testAddIPWithResellerAccess() {
    $ip_start = '10.10.10.10';
    $ip_end   = '10.10.10.10';
    $args     = array( 'reseller' => 'No' );

    $this->_AddIpToNodeWorx( '10.10.10.10', '10.10.10.10', $args );
  }

  /**
   * testCheckIfIPAvailableToResellersAndRemove
   */
  public function testCheckIfIPAvailableToResellersAndRemove() {
    $this->loginToNWAndGoTo( '/nodeworx/ip' );
    $this->assertElementPresent( 'input[10.10.10.10][reseller]' );
    $this->assertElementValueEquals( 'input[10.10.10.10][reseller]', 'no' );

    $this->select( 'input[10.10.10.10][sys_aware]', 'label=No' );
    $this->select( "//select[@name='action']", 'label=Commit Changes' );

    $this->waitForPageToLoad();
    $this->assertPageRenderedWithoutErrors();

    $this->assertTextPresent( 'IP deleted from system successfully : 10.10.10.10' );
  }
}
?>