<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jan 16, 2009
 * @id $Id: Bug_949_FixTest.php,v 1.3 2009/08/21 02:29:51 poehler Exp $
 */

/**
 * Tests to ensure Bug# 949 was corrected.
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class Bug_949_FixTest extends IWorxSeleniumTestCase {

  /**
   * testHotlinkProtection
   */
  public function testHotlinkProtection() {
    $this->addSWAccountIfNecessary();
    $this->loginToSWAndGoTo( '/siteworx/overview' );
    $this->assertPageRenderedWithoutErrors();

    $this->click( 'iw-menu-features-htaccess' );
    $this->waitForPageToLoad();

    /*
     * Add the global htaccess if it is no present.
     */
    $global_htaccess_created = false;
    if( $this->isElementPresent( 'add_global_subbutton' ) ) {
      $global_htaccess_created = true;
      $this->click( 'add_global_subbutton' );
      $this->waitForPageToLoad();
    }

    $this->click( 'link=Hotlink Protection' );
    $this->waitForPageToLoad();

    /*
     * Turn hotlink protection off and remove all extensions.
     */
    $this->select( 'status', 'label=Off' );
    $this->type( 'extensions', '' );
    $this->click( "//input[@value='Update']" );
    $this->waitForPageToLoad();

    $message = IWorxTranslate::tr( '##LG_HTACCESS_HOTLINK_UPDATE_SUCCEEDED##' );
    $message .= ': global';

    $this->assertTextPresent( $message );
    $this->assertElementValueEquals( 'status', 'off' );
    $this->assertElementNotContainsText( 'extensions', '.jpeg' );

    /*
     * Turn hotlink protection on and add extensions
     */
    $this->select( 'status', 'label=On' );
    $this->type( 'extensions', '.jpeg' );
    $this->click( "//input[@value='Update']" );
    $this->waitForPageToLoad();

    $this->assertTextPresent( $message );
    $this->assertElementValueEquals( 'status', 'on' );
    $this->assertElementContainsText( 'extensions', '.jpeg' );

    /*
     * Leave hotlink protection on and remove all extensions.
     */
    $this->type( 'extensions', '' );
    $this->click( "//input[@value='Update']" );
    $this->waitForPageToLoad();

    $this->assertTextPresent( $message );
    $this->assertElementValueEquals( 'status', 'on' );
    $this->assertElementNotContainsText( 'extensions', '.jpeg' );

    /*
     * Get me out of here.
     */
    $this->click( 'iw-menu-features-htaccess' );
    $this->waitForPageToLoad();

    /*
     * Remove the global htaccess file
     */
    $this->click( "//input[@value='Delete']" );
    $this->waitForPageToLoad();
    $this->click( 'submit' );
  }

}
?>