<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Sept 30, 2008
 * @id $Id: BasicIWorxCronJobsTest.php,v 1.7 2009/11/02 18:32:47 jdaniel Exp $
 */

/**
 * Unit tests IWorx cron scripts
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class BasicIWorxCronJobsTest extends PHPUnit_Framework_TestCase {

  /**
   * Tear down function, reloads the ini to default state.
   */
  public function tearDown() {
    Ini::reload();
  }

  /**
   * cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxBandwidth() {
    $chunksize = Ini::get( 'iworx', 'chunksize' );
    $ids       = SiteWorx::getIdList();

    $chunks = array_chunk( $ids, $chunksize );

    $i       = 1;
    $success = true;
    foreach( $chunks as $chunk ) {
      IWorxExec::exec( Ini::get( 'iworx.cron', 'bandwidth' ) .
                       ' --overage ' .
                       " --chunksize=$chunksize " .
                       " --chunk=$i ",
                       $result,
                       $retval,
                       IWorxExec::NO_OUTPUT );
      $i++;
      if( $retval !== 0 ) {
        $success = false;
      }
    }
    return $success;
  }

  /**
   * cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxLocalhostCheck() {
    $hosts_file  = Ini::get( 'file', 'hosts' );
    $contents    = file_get_contents( $hosts_file );
    $check_regex = '^127\.0\.0\.1(.+)localhost(\s*)\n^';

    if( preg_match( $check_regex, $contents ) == 0 ) {
      $localhost_line = "\n127.0.0.1 localhost\n";
      file_put_contents( $hosts_file, $localhost_line, FILE_APPEND );
      IWorxLog::log( "localhost was missing from {$hosts_file}," .
                     ' it has been added' );
    }
    return true;
  }

  /**
   * cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxStorage() {
    // we just need to run this on regular servers
    // and/or cluster managers
    if( Node::isInCluster() && !Node::isManager() ) {
      IWorxLog::log( 'skipping storage on this clustered servers',
                     IWorxLog::INFO );
      return 1;
    }

    $chunksize = Ini::get( 'iworx', 'chunksize' );
    $ids       = SiteWorx::getIdList();

    $chunks = array_chunk( $ids, $chunksize );

    $i       = 1;
    $success = true;
    foreach( $chunks as $chunk ) {
      IWorxExec::exec( Ini::get( 'iworx.cron', 'storage' ) .
                       " --chunksize=$chunksize " .
                       " --chunk=$i ",
                       $result,
                       $retval,
                       IWorxExec::NO_OUTPUT );
      $i++;
      if( $retval !== 0 ) {
        $success = false;
      }
    }
    return $success;
  }

  /**
   * cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxStorageSyncQuota() {
    $chunksize = Ini::get( 'iworx', 'chunksize' );
    $ids       = SiteWorx::getIdList();

    $chunks = array_chunk( $ids, $chunksize );

    $i       = 1;
    $success = true;
    foreach( $chunks as $chunk ) {
      IWorxExec::exec( Ini::get( 'iworx.cron', 'storage' ) .
                       ' --syncquota ' .
                       " --chunksize=$chunksize " .
                       " --chunk=$i ",
                       $result,
                       $retval,
                       IWorxExec::NO_OUTPUT );
      $i++;
      if( $retval !== 0 ) {
        $success = false;
      }
    }
    return $success;
  }

  /**
   * Just run the bandwidth cron and see if it executes successfully or not
   */
  public function testBandwidthCronExecution() {
    $this->assertTrue( $this->iworxBandwidth() );
  }

  /**
   * Just run the storage cron and see if it executes successfully or not
   */
  public function testStorageCronExecution() {
    $this->assertTrue( $this->iworxStorage() );
  }

  /**
   * test the storage cron syncquota mechanism (just looking for bin failures)
   */
  public function testStorageCronSyncQuota() {
    $this->assertTrue( $this->iworxStorageSyncQuota() );
  }

  /**
   * Tests the iworx_localhostcheck function under present conditions
   */
  public function testLocalhostCheck() {
    $this->assertTrue( $this->iworxLocalhostCheck() );
  }

  /**
   * Tests the iworx_localhostcheck function when we are sure the 127.0.0.1
   * entry is *not* missing.
   *
   * For the test we are going to use a temp file, just so we aren't messing
   * with the real /etc/hosts file.
   */
  public function testLocalhostCheckWhenNotMissing() {
    $tmp        = Ini::get( 'dir', 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n127.0.0.1 localhost\n";

    file_put_contents( $test_hosts, $contents );
    $this->iworxLocalhostCheck();

    $expected_contents = $contents;
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Tests the iworx_localhostcheck function when we are sure the 127.0.0.1
   * entry is *not* missing.
   *
   * For the test we are going to use a temp file, just so we aren't messing
   * with the real /etc/hosts file.
   */
  public function testLocalhostCheckWhenMissing() {
    $tmp        = Ini::get( 'dir', 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n";

    file_put_contents( $test_hosts, $contents );
    Ini::setTemp( 'file', 'hosts', $test_hosts );

    $this->iworxLocalhostCheck();

    $expected_contents = "{$contents}\n127.0.0.1 localhost\n";
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Tests that the check will not add another line if additional entries were
   * added to the 127.0.0.1 line.
   */
  public function testLocalhostCheckWhenLocahostIsPresentIn127() {
    $tmp        = Ini::get( 'dir', 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n" .
      "127.0.0.1 xdev.interworx.com xdev localhost.localdomain localhost\n";

    file_put_contents( $test_hosts, $contents );
    Ini::setTemp( 'file', 'hosts', $test_hosts );

    $this->iworxLocalhostCheck();

    $expected_contents = $contents;
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Tests that the check will add a new line if the 127.0.0.1 IP is present
   * but does not contain localhost.
   */
  public function testLocalhostCheckWhenLocahostIsNotPresentIn127() {
    $tmp        = Ini::get( 'dir', 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n" .
      "127.0.0.1 xdev.interworx.com xdev localhost.localdomain\n";

    file_put_contents( $test_hosts, $contents );
    Ini::setTemp( 'file', 'hosts', $test_hosts );

    $this->iworxLocalhostCheck();

    $expected_contents = "{$contents}\n127.0.0.1 localhost\n";
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Test the rrd update cron
   */
  public function testRrdUpdateCronExecution() {
    IWorxExec::exec( Ini::get( 'iworx.cron', 'rrd' ) .
                     ' --update ' .
                     ' --services all ',
                     $result,
                     $retval,
                     IWorxExec::NO_OUTPUT );
    $this->assertEquals( 0, $retval );
  }

  /**
   * Test the dns cron
   */
  public function testDnsCronExecution() {
    IWorxExec::exec( Ini::get( 'iworx.cron', 'dns' ) .
                     ' --sync'    .
                     ' --export ' .
                     ' --cdb ',
                     $result,
                     $retval );
    $this->assertEquals( 0, $retval );
  }

  /**
   * Test the clear password reset cron
   */
  public function testClearPasswordResetCronExecution() {
    IWorxExec::exec( Ini::get( 'iworx.cron', 'clearpasswordreset' ),
                     $result,
                     $retval );
    $this->assertEquals( 0, $retval );
  }

  /**
   * Test to make sure that the temp errors generated by the error handler are
   * being properly deleted by the fively cron job.
   */
  public function testTempErrorDeletion() {
    $log_dir = IWorxLog::getIWorxLogDir();
    $touch   = Ini::get( 'bin', 'touch' );

    for( $i = 0; $i < 25; $i++ ) {
      $filename = 'temp-error.' . md5( $i );
      $target   = "$log_dir/$filename";
      IWorxExec::exec( "$touch $target", $result, $retval );

      $files[] = $filename;
    }

    IWorxExec::exec( Ini::get( 'iworx.cron', 'iworx' ) .
                     ' --fively ',
                     $result,
                     $retval );
    $this->assertEquals( 0, $retval );

    $all_good = true;
    if( $handle = opendir( $log_dir ) ) {
      while( false !== ($file = readdir( $handle ) ) ) {
        if( in_array( $file, $files ) ) {
          $all_good = false;
          break;
        }
      }
    }
    $this->assertTrue( $all_good === true );
  }

  /**
   * Test the license sync cron
   */
  public function testLicenseSyncCronExecution() {
    IWorxExec::exec( Ini::get( 'iworx.cron', 'license' ),
                     $result,
                     $retval );
    $this->assertEquals( 0, $retval );
  }
}
?>