<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2009 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Jun 11, 2009
 * @id $Id: EnvTest.php,v 1.2 2009/06/13 08:15:17 poehler Exp $
 */

/**
 * Generic test related to Environment classes
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class EnvTest extends IWorx_Framework_TestCase {
  /**
   * test
   */
  public function testDefaultEnvironmentIsPhpUnit() {
    $this->assertTrue( IW::Env() instanceof Env_PHPUnit );
  }

  /**
   * test
   */
  public function testSwitchEnvironmentTo() {
    Env_Factory::switchEnvironmentTo( new Env_PHP() );

    $this->assertTrue( IW::Env() instanceof Env_PHP );
  }

  /**
   * test
   */
  public function testSwitchEnvironmentToAgain() {
    Env_Factory::switchEnvironmentTo( new Env_HTML() );

    $this->assertTrue( IW::Env() instanceof Env_HTML );
  }

  /**
   * test
   */
  public function testSwitchEnvironmentBack() {
    Env_Factory::switchEnvironmentBack();

    $this->assertTrue( IW::Env() instanceof Env_PHP );
  }

  /**
   * test
   */
  public function testSwitchEnvironmentBackAgain() {
    Env_Factory::switchEnvironmentBack();

    $this->assertTrue( IW::Env() instanceof Env_PHPUnit );
  }

  /**
   * test
   */
  public function testEnvSwitchAroundRouteFromPHP() {
    $NW = new NodeWorx( 1 );

    IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                            'listMasterDomains',
                            array(),
                            $NW );

    $this->assertTrue( IW::Env() instanceof Env_PHPUnit );
  }

  /**
   * test
   */
  public function testPhpUnitGetSession() {
    $Session = IW::Env()->getNodeWorxSession();

    $this->assertTrue( $Session instanceof Session_NodeWorx );
  }

  /**
   * test
   */
  public function testEnvSwitchAroundRouteFromPHPToSiteWorx() {
    self::addSWAccountIfNecessary();
    $SW = new SiteWorx( self::SW_DOMAIN );

    IW::FC()->routeFromPHP( 'Ctrl_Siteworx_EmailBox', 'list', array(), $SW );

    $Session = IW::Env()->getNodeWorxSession();

    $this->assertTrue( $Session instanceof Session_NodeWorx );
  }
}
?>