<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @id $Id: IWorxExecTest.php,v 1.22 2009/09/16 07:46:25 poehler Exp $
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Aug 12, 2008
 */
require_once 'PHPUnit/Framework.php';

/**
 * Test class for IWorxExec.
 * Generated by PHPUnit on 2008-08-06 at 14:27:36.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxExecTest extends PHPUnit_Framework_TestCase {

  /**
   * Runs the test methods of this class.
   *
   * @access public
   * @static
   */
  public static function main() {
    require_once 'PHPUnit/TextUI/TestRunner.php';

    $suite  = new PHPUnit_Framework_TestSuite('IWorxExecTest');
    $result = PHPUnit_TextUI_TestRunner::run( $suite );
  }

  /**
   * Tears down the fixture, for example, closes a network connection.
   * This method is called after a test is executed.
   *
   * @access protected
   */
  public function tearDown() {
  }

  /**
   * A sample shell command used for testing
   *
   * @return string
   */
  private function _getValidCommand() {
    return '/bin/echo TEST';
  }

  /**
   * Sleep shell command used for testing
   *
   * @param integer $seconds
   * @return string
   */
  private function _getSleepCommand( $seconds ) {
    return "/bin/sleep $seconds";
  }

  /**
   * A sample shell command that does not exist
   *
   * @return string
   */
  private function _getInValidCommand() {
    return '/this/does/not/exist/bin/echo TEST';
  }

  /**
   * The valid output of the sample valid command
   *
   * @return string
   */
  private function _getValidCommandResult() {
    return array( 'TEST' );
  }

  /**
   * data provider that data and the appropriately escaped version of it
   *
   * @return array
   */
  public static function getEscapeTestData() {
    return( array(
                  array( '/bin/echo TEST',
                         '/bin/echo TEST' ),
                  array( '`',  // escape backtick
                         '\`' ),
                  array( '>', // escape bracket
                         '\>' ),
                  array( '>', // no escape bracket
                         '>',
                         '>' ),
                  array( '<', // escape bracket
                         '\<' ),
                  array( '<', // no escape bracket
                         '<',
                         '<' ),
                  array( '|', // escape pipe
                         '\|' ),
                  array( '|', // no escape pipe
                         '|',
                         '|' ),
                  array( '&', // escape ampersand
                         '\&' ),
                  array( '&', // no escape ampersand
                         '&',
                         '&' ),
                  array( '\\', // escape backslash
                         '\\\\' ),
                  array( '\\', // no escape backslash
                         '\\',
                         '\\' ),
                  array( '[', // escape left bracket
                         '\[' ),
                  array( '[', // no escape left bracket
                         '[',
                         '[' ),
                  array( '^', // escape caret
                         '\^' ),
                  array( '^', // no escape caret
                         '^',
                         '^' ),
                  array( ']', // escape right bracket
                         '\]' ),
                  array( ']', // no escape right bracket
                         ']',
                         ']' ),
                  array( '{', // escape left curly
                         '\{' ),
                  array( '}', // no escape left curly
                         '}',
                         '}' ),
                  array( '}', // escape right bracket
                         '\}' ),
                  array( '}', // no escape right bracket
                         '}',
                         '}' ),
                  array( '#', // escape pound
                         '\#' ),
                  array( '#', // no escape pound
                         '#',
                         '#' ),
                  array( '(', // escape left paren
                         '\(' ),
                  array( '(', // no escape left paren
                         '(',
                         '(' ),
                  array( ')', // escape right paren
                         '\)' ),
                  array( ')', // no escape right paren
                         ')',
                         ')' ),
                  array( '$', // escape dollar
                         '\$' ),
                  array( '$', // no escape dollar
                         '$',
                         '$' )
                  )
            );
  }

  /**
   * Tests Escapte shell command
   *
   * @param string $unescaped
   * @param string $escaped
   * @param mixed  $exceptions
   * @dataProvider getEscapeTestData()
   */
  public function testEscapeshellcmd( $unescaped,
                                      $escaped,
                                      $exceptions = array() ) {
    $result = IWorxExec::escapeshellcmd( $unescaped, $exceptions );
    $this->assertEquals( $escaped, $result );
  }

  /**
   * IWorxExec test method
   */
  public function testExecReturnsTrueForValidCommand() {
    $cmd    = $this->_getValidCommand();
    $result = array();
    $retval = '';
    $this->assertTrue( IWorxExec::exec( $cmd, $result, $retval ) );
  }

  /**
   * IWorxExec test method
   */
  public function testExecReturnsFalseForInvalidCommand() {
    $cmd    = $this->_getInvalidCommand();
    $result = array();
    $retval = '';
    $this->assertFalse( IWorxExec::exec( $cmd,
                                         $result,
                                         $retval,
                                         IWorxExec::KILL_STDERR ) );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAssignsResultOutput() {
    $cmd    = $this->_getValidCommand();
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $validOutput = $this->_getValidCommandResult();
    $this->assertTrue( isset( $result[0] ) );
    $this->assertTrue( $result[0] === $validOutput[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAssignsRetval() {
    $cmd    = $this->_getValidCommand();
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $this->assertTrue( $retval === 0 );
  }

  /**
   * IWorxExec test method
   */
  public function testExecDevNullRedirWorks() {
    $cmd    = $this->_getValidCommand();
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::REDIR_DEV_NULL );

    $this->assertFalse( isset( $result[0] ) );
  }

  /**
   * IWorxExec test method
   */
  public function testExecStderr2StdoutWorks() {
    $cmd    = $this->_getInvalidCommand();
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::KILL_STDERR );

    $this->assertFalse( isset( $result[0] ) );
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::STDERR_2_STDOUT );

    $this->assertTrue( isset( $result[0] ) );
  }

  /**
   * IWorxExec test method
   */
  public function testExecNoOutputWorks() {
    $cmd    = $this->_getInvalidCommand();
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::NO_OUTPUT );

    $this->assertFalse( isset( $result[0] ) );

    $cmd    = $this->_getValidCommand();
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::NO_OUTPUT );

    $this->assertFalse( isset( $result[0] ) );
  }

  /**
   * IWorxExec test method
   */
  public function testExecNoEscapeCmdWorks() {
    // executing
    //  echo TEST `echo TEST`
    // expected output 'TEST TEST' if not escaped
    // expected output 'TEST `echo TEST`' if escaped

    $cmd    = $this->_getValidCommand() . ' `' . $this->_getValidCommand() . '`';
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval,
                     IWorxExec::NO_ESCAPE_CMD |
                     IWorxExec::STDERR_2_STDOUT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} {$goodResult[0]}";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * test escape commands by default
   *
   * @group bug_961
   */
  public function testExecEscapesCommandsByDefault() {
    // executing
    //
    // expected output 'TEST TEST' if not escaped
    // expected output 'TEST `echo TEST`' if escaped
    $cmd    = $this->_getValidCommand() . ' `' . $this->_getValidCommand() . '`';
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} `{$this->_getValidCommand()}`";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAllowAsteriskWorks() {
    $cmd    = $this->_getValidCommand() . ' *';
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_ASTERISK );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} *";
    $this->assertNotEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAllowRedirInWorks() {
    $cmd    = $this->_getValidCommand() . ' < /dev/null';
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_REDIR_IN );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} < /dev/null";
    $this->assertNotEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAllowRedirOutWorks() {
    $cmd    = $this->_getValidCommand() . ' > /dev/null';
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_REDIR_OUT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} > /dev/null";
    $this->assertNotEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAllowPipeWorks() {
    $cmd    = $this->_getValidCommand() . ' | /bin/cat';
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_PIPE );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} | /bin/cat";
    $this->assertNotEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAllowAmpersandWorks() {
    $cmd    = $this->_getValidCommand() . " '&'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \\&";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_AMPERSAND );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} &";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAllowBackslashWorks() {
    $cmd    = $this->_getValidCommand() . " '\\'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \\\\";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_BACKSLASH );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \\";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecAllowCaretWorks() {
    $cmd    = $this->_getValidCommand() . " '^'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \^";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_CARET );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} ^";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecBracketLeftWorks() {
    $cmd    = $this->_getValidCommand() . " '['";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \[";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_BRACKETLEFT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} [";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecBracketRightWorks() {
    $cmd    = $this->_getValidCommand() . " ']'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \]";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_BRACKETRIGHT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} ]";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecBracketBothWorks() {
    $cmd    = $this->_getValidCommand() . " '[]'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \[\]";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_BRACKETBOTH );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} []";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecCurlyLeftWorks() {
    $cmd    = $this->_getValidCommand() . " '{'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \{";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_CURLYLEFT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} {";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecCurlyRightWorks() {
    $cmd    = $this->_getValidCommand() . " '}'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \}";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_CURLYRIGHT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} }";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecCurlyBothWorks() {
    $cmd    = $this->_getValidCommand() . " '{}'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \{\}";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_CURLYBOTH );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} {}";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecParenLeftWorks() {
    $cmd    = $this->_getValidCommand() . " '('";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \(";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_PARENLEFT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} (";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecParenRightWorks() {
    $cmd    = $this->_getValidCommand() . " ')'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \)";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_PARENRIGHT );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} )";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecParenBothWorks() {
    $cmd    = $this->_getValidCommand() . " '()'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \(\)";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_PARENBOTH );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} ()";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecPoundWorks() {
    $cmd    = $this->_getValidCommand() . " '#'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \#";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_POUND );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} #";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecDollarWorks() {
    $cmd    = $this->_getValidCommand() . " '$'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \\$";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_DOLLAR );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} $";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecMultiSpecialChars() {
    $cmd    = $this->_getValidCommand() . " '*<>|&\\{}[]#\$'";
    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} \*\<\>\|\&\\\\\{\}\[\]\#\\$";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_ASTERISK |
                     IWorxExec::ALLOW_AMPERSAND |
                     IWorxExec::ALLOW_BACKSLASH |
                     IWorxExec::ALLOW_BRACKETBOTH |
                     IWorxExec::ALLOW_CURLYBOTH |
                     IWorxExec::ALLOW_PIPE |
                     IWorxExec::ALLOW_REDIR_IN |
                     IWorxExec::ALLOW_REDIR_OUT |
                     IWorxExec::ALLOW_POUND |
                     IWorxExec::ALLOW_DOLLAR );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} *<>|&\\{}[]#\$";
    $this->assertEquals( $expectedOutput, $result[0] );

    $result = array();
    $retval = '';

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::NO_ESCAPE_CMD );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "{$goodResult[0]} *<>|&\\{}[]#\$";
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testExecInBackgroundWorks() {
    $cmd    = $this->_getSleepCommand( 7 );
    $result = array();
    $retval = '';
    $now    = time();
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::IN_BACKGROUND );
    $then = time();
    $this->assertEquals( 0, $then - $now, 'Exec in background took too long' );
  }

  /**
   * IWorxExec test method
   */
  public function testExecKillStderrWorks() {
    $cmd    = $this->_getInvalidCommand();
    $result = array();
    $retval = '';
    IWorxExec::exec( $cmd,
                     $result,
                     $retval,
                     IWorxExec::KILL_STDERR );

    $this->assertFalse( isset( $result[0] ) );
  }

  /**
   * IWorxExec test method
   */
  public function testExecKillInheritedFd() {
    $cmd    = $this->_getValidCommand();
    $result = array();
    $retval = '';
    IWorxExec::exec( $cmd,
                     $result,
                     $retval,
                     IWorxExec::KILL_INHERITED_FD );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = $goodResult[0];

    // this unfortunately doesn't really test the KILL_INHERITED_FD
    // Haven't found a way to really test it yet.
    $this->assertEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testShellExec() {
    $cmd = $this->_getValidCommand();

    $output = IWorxExec::shell_exec( $cmd );

    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = $goodResult[0];
    $this->assertEquals( $expectedOutput, trim( $output ) );
  }

  /**
   * IWorxExec test method
   */
  public function testPassthru() {
    $cmd  = $this->_getValidCommand();
    $ret1 = -1;
    ob_start();

    $ret2 = IWorxExec::passthru( $cmd, $ret1 );

    $output = ob_get_contents();
    ob_end_clean();
    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = $goodResult[0];
    $this->assertEquals( $ret1, $ret2 );
    $this->assertEquals( $expectedOutput, trim( $output ) );
  }

  /**
   * IWorxExec test method
   */
  public function testPassthruEscapesCommandsByDefault() {
    $cmd  = $this->_getValidCommand() . ' | /bin/cat';
    $ret1 = -1;
    ob_start();

    $ret2 = IWorxExec::passthru( $cmd, $ret1 );

    $output = ob_get_contents();
    ob_end_clean();
    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = "$goodResult[0] | /bin/cat";
    $this->assertEquals( $ret1, $ret2 );
    $this->assertEquals( $expectedOutput, trim( $output ) );
  }


  /**
   * IWorxExec test method
   */
  public function testPassthruNOSCRUB() {
    $cmd    = $this->_getValidCommand() . ' | /bin/cat';
    $result = array();
    $ret1   = '';
    ob_start();

    $ret2 = IWorxExec::passthru_NOSCRUB( $cmd, $ret1 );

    $output = ob_get_contents();
    ob_end_clean();
    $goodResult     = $this->_getValidCommandResult();
    $expectedOutput = $goodResult[0];
    $this->assertNotEquals( $expectedOutput, $result[0] );
  }

  /**
   * IWorxExec test method
   */
  public function testQuickExec() {
    $cmd = $this->_getValidCommand();

    $return = IWorxExec::quick_exec( $cmd );

    $this->assertTrue( $return );
  }

  /**
   * IWorxExec test method
   */
  public function testQuickExecNOSCRUB() {
    $cmd = $this->_getValidCommand();

    $return = IWorxExec::quick_exec_NOSCRUB( $cmd );

    $this->assertTrue( $return );
  }
}
?>