<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Aug 12, 2008
 * @id $Id: IWorxFileSysTest.php,v 1.31 2009/10/28 04:43:17 poehler Exp $
 */
require_once 'PHPUnit/Framework.php';

/**
 * Test class for IWorxFileSys.
 * Generated by PHPUnit on 2008-08-07 at 22:21:07.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxFileSysTest extends PHPUnit_Framework_TestCase {

  /**
   * Sample chmod codes for testing
   *
   * @return array
   */
  public static function getChmodCodes() {
    return( array(
                  array( 0700 ),
                  array( 0600 ),
                  array( 0400 ),
                  array( 0777 ),
                  array( 0755 ),
                  array( 0644 ),
                  array( 04755 ),
                  array( 0000 )
                  ));
  }

  /**
   * returns a tmp directory for tesitng
   *
   * @return string
   */
  public function getTempDir() {
    return Ini::get( 'dir', 'tmp' );
  }

  /**
   * Returns a temporary file
   *
   * @param string $indir
   * @return string
   */
  public function getTempFile( $indir = '' ) {
    if( $indir === '' ) {
      return tempnam( $this->getTempDir(), 'filesys_test_' );
    } else {
      return tempnam( $indir, 'filesys_test_' );
    }
  }

  /**
   * test func
   */
  public function testChdir() {
    $cwd = getcwd();
    $tmp = $this->getTempDir();
    $this->assertNotEquals( $tmp, $cwd, 'Chdir src and dst must differ' );

    IWorxFileSys::chdir( $tmp );

    $newcwd = getcwd();
    $this->assertEquals( realpath( $tmp ), realpath( $newcwd ) );
  }

  /**
   * test func
   */
  public function testChownOneFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chown( 'vpopmail', $tmpfile );

    $fstat = stat( $tmpfile );
    unlink( $tmpfile );
    $this->assertEquals( 108, $fstat['uid'], 'Chown vpopmail user wrong uid' );
  }

  /**
   * test func
   */
  public function testChownAndChgrpOneFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chown( 'vpopmail', $tmpfile, 'vchkpw' );

    $fstat = stat( $tmpfile );

    unlink( $tmpfile );

    $this->assertEquals( 108,
                         $fstat['uid'],
                         'Chown vpopmail wrong uid' );
    $this->assertEquals( 104,
                         $fstat['gid'],
                         'Chown vchkpw wrong gid' );
  }

  /**
   * test func
   */
  public function testChownRecursive() {
    $tmp       = $this->getTempDir();
    $newTmpDir = "$tmp/tmpdir";
    mkdir( $newTmpDir );
    $tmpfile = $this->getTempFile( $newTmpDir );

    IWorxFileSys::chown( 'vpopmail', $newTmpDir, 'vchkpw', $recurse = true );

    $dstat = stat( $newTmpDir );
    $fstat = stat( $tmpfile );

    unlink( $tmpfile );
    rmdir( $newTmpDir );

    $this->assertEquals( 108, $dstat['uid'], 'Chown dir vpopmail wrong uid' );
    $this->assertEquals( 104, $dstat['gid'], 'Chown dir vchkpw wrong gid' );
    $this->assertEquals( 108, $fstat['uid'], 'Chown file vpopmail wrong uid' );
    $this->assertEquals( 104, $fstat['gid'], 'Chown file vchkpw wrong gid' );
  }

  /**
   * test func
   */
  public function testChgrpOneFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chgrp( 'vchkpw', $tmpfile );

    $fstat = stat( $tmpfile );
    unlink( $tmpfile );
    $this->assertEquals( 104, $fstat['gid'], 'Chgrp vchkpw wrong gid' );
  }

  /**
   * test func
   */
  public function testChgrpRecursive() {
    $tmp       = $this->getTempDir();
    $newTmpDir = "$tmp/tmpdir";
    mkdir( $newTmpDir );
    $tmpfile = $this->getTempFile( $newTmpDir );

    IWorxFileSys::chgrp( 'vchkpw', $newTmpDir, $recurse = true );

    $dstat = stat( $newTmpDir );
    $fstat = stat( $tmpfile );

    unlink( $tmpfile );
    rmdir( $newTmpDir );

    $this->assertEquals( 0,   $dstat['uid'], 'chgrp dir uid should be root' );
    $this->assertEquals( 104, $dstat['gid'], 'chgrp dir vchkpw wrong gid' );
    $this->assertEquals( 0,   $fstat['uid'], 'chgrp file, uid should be root' );
    $this->assertEquals( 104, $fstat['gid'], 'chgrp file vchkpw wrong gid' );
  }

  /**
   * test func
   *
   * @param integer $mode
   * @dataProvider getChmodCodes()
   */
  public function testChmodOneFile( $mode ) {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chmod( $mode, $tmpfile );

    $fperms = substr( sprintf( '%o', fileperms( $tmpfile ) ), -4 );
    unlink( $tmpfile );

    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $fperms );
  }

  /**
   * test func
   *
   * @param integer $mode
   * @dataProvider getChmodCodes()
   */
  public function testChmodRecursive( $mode ) {
    $tmp       = $this->getTempDir();
    $newTmpDir = "$tmp/tmpdir";
    mkdir( $newTmpDir );
    $tmpfile = $this->getTempFile( $newTmpDir );

    IWorxFileSys::chmod( $mode, $newTmpDir, $recurse = true );

    $fperms = substr( sprintf( '%o', fileperms( $tmpfile ) ), -4 );
    $dperms = substr( sprintf( '%o', fileperms( $newTmpDir ) ), -4 );

    unlink( $tmpfile );
    rmdir( $newTmpDir );

    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $dperms );
    $this->assertEquals( $smode, $fperms );
  }

  /**
   * test func
   */
  public function testChownmod() {
    $tmpfile = $this->getTempFile();
    $mode    = 0777;

    IWorxFileSys::chownmod( 'vpopmail', 'vchkpw', $mode, $tmpfile );

    $fperms = substr( sprintf( '%o', fileperms( $tmpfile ) ), -4 );
    $fstat  = stat( $tmpfile );
    unlink( $tmpfile );

    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $fperms );
    $this->assertEquals( 108, $fstat['uid'], 'chown file vpopmail wrong uid' );
    $this->assertEquals( 104, $fstat['gid'], 'chgrp file vchkpw wrong gid' );
  }

  /**
   * test func
   */
  public function testMkdir() {
    $tmp = $this->getTempDir();

    IWorxFileSys::mkdir( "$tmp/mynewdir" );

    $direxists = is_dir( "$tmp/mynewdir" );
    rmdir( "$tmp/mynewdir" );
    $this->assertTrue( $direxists );
  }

  /**
   * test func
   */
  public function testMkdirWithMode() {
    $tmp  = $this->getTempDir();
    $mode = 0777;

    IWorxFileSys::mkdir( "$tmp/mynewdir", $mode );

    $direxists = is_dir( "$tmp/mynewdir" );
    $dperms    = substr( sprintf( '%o', fileperms( "$tmp/mynewdir" ) ), -4 );

    rmdir( "$tmp/mynewdir" );
    $this->assertTrue( $direxists );
    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $dperms );
  }

  /**
   * test func
   */
  public function testMkdirWithModeRecursive() {
    $tmp  = $this->getTempDir();
    $mode = 0777;

    IWorxFileSys::mkdir( "$tmp/mynewdir/a/b/c/d/e", $mode, $recurse = true );

    $direxists = is_dir( "$tmp/mynewdir/a/b/c/d/e" );
    $dperms    = substr( sprintf( '%o',
                                  fileperms( "$tmp/mynewdir/a/b/c/d/e" ) ),
                         -4 );

    rmdir( "$tmp/mynewdir/a/b/c/d/e" );
    rmdir( "$tmp/mynewdir/a/b/c/d" );
    rmdir( "$tmp/mynewdir/a/b/c" );
    rmdir( "$tmp/mynewdir/a/b" );
    rmdir( "$tmp/mynewdir/a" );
    rmdir( "$tmp/mynewdir" );
    $this->assertTrue( $direxists );
    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $dperms );
  }

  /**
   * Tests that the owner and group options work correctly.
   */
  public function testMkdirWithUser() {
    $tmp = $this->getTempDir();

    if( is_dir( "$tmp/mynewdir" ) === true ) {
      rmdir( "$tmp/mynewdir" );
    }

    IWorxFileSys::mkdir( "$tmp/mynewdir",
                         false,
                         $recurse = true );

    $stat    = IWorxFileSys::stat( "$tmp/mynewdir" );
    $pwnam   = posix_getpwuid( $stat['uid'] );
    $pwgrp   = posix_getgrgid( $stat['gid'] );

    $this->assertNotEquals( 'iworx', $pwnam['name'] );
    $this->assertNotEquals( 'iworx', $pwgrp['name'] );

    if( is_dir( "$tmp/mynewdir" ) === true ) {
      rmdir( "$tmp/mynewdir" );
    }

    IWorxFileSys::mkdir( "$tmp/mynewdir",
                         false,
                         $recurse = true,
                         'iworx',
                         'iworx' );

    $stat    = IWorxFileSys::stat( "$tmp/mynewdir" );
    $pwnam   = posix_getpwuid( $stat['uid'] );
    $pwgrp   = posix_getgrgid( $stat['gid'] );

    rmdir( "$tmp/mynewdir" );

    $this->assertEquals( 'iworx', $pwnam['name'] );
    $this->assertEquals( 'iworx', $pwgrp['name'] );
  }
  /**
   * test func
   */
  public function testLn() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();

    IWorxFileSys::ln( $tmpfile, "$tmpdir/mylink" );
    $islink = is_link( "$tmpdir/mylink" );

    unlink( "$tmpdir/mylink" );
    unlink( $tmpfile );

    $this->assertTrue( $islink );
  }

  /**
   * test func
   */
  public function testMv() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();
    $ostat   = stat( $tmpfile );

    IWorxFileSys::mv( $tmpfile, "$tmpdir/movedfile" );

    $fstat = stat( "$tmpdir/movedfile" );
    unlink( "$tmpdir/movedfile" );

    $this->assertFalse( is_file( $tmpfile ) );
    $this->assertEquals( $ostat['ino'], $fstat['ino'] );
  }

  /**
   * test func
   */
  public function testCp() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();
    $ostat   = stat( $tmpfile );

    IWorxFileSys::cp( $tmpfile, "$tmpdir/copiedfile" );

    $fstat = stat( "$tmpdir/copiedfile" );
    unlink( $tmpfile );
    unlink( "$tmpdir/copiedfile" );

    $this->assertNotEquals( $ostat['ino'], $fstat['ino'] );
  }

  /**
   * test func
   */
  public function testRm() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();

    IWorxFileSys::rm( $tmpfile );

    $this->assertFalse( is_file( $tmpfile ) );
  }

  /**
   * test func
   */
  public function testTmpdir() {
    $tmpdir = $this->getTempDir();
    $mode   = 0777;

    $newDir = IWorxFileSys::tmpdir( $tmpdir, 'tmp_dir_test_', $mode );
    $dperms = substr( sprintf( '%o', fileperms( $newDir ) ), -4 );
    $is_dir = is_dir( $newDir );
    rmdir( $newDir );
    $smode = sprintf( '%o', $mode );
    $this->assertTrue( $is_dir );
    $this->assertEquals( $smode, $dperms );
  }

  /**
   * test func
   */
  public function testTempnam() {
    $tmpdir = $this->getTempDir();

    $tmpfile = IWorxFileSys::tempnam( realpath( $tmpdir ), 'tempnam_test_' );

    $is_file = is_file( $tmpfile );
    if( $is_file ) {
      unlink( $tmpfile );
    }
    $this->assertTrue( $is_file );
  }

  /**
   * test func
   */
  public function testBackupFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::backup_file( $tmpfile );

    $is_file   = is_file( $tmpfile );
    $dname     = dirname( $tmpfile );
    $fname     = basename( $tmpfile );
    $backup    = "$dname/.$fname";
    $is_file_b = is_file( $backup );
    unlink( $tmpfile );
    unlink( $backup );
    $this->assertTrue( $is_file );
    $this->assertTrue( $is_file_b );
  }

  /**
   * test func
   */
  public function testRestoreFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::backup_file( $tmpfile );

    unlink( $tmpfile );

    $ret = IWorxFileSys::restore_file( $tmpfile );

    $is_file = is_file( $tmpfile );
    $dname   = dirname( $tmpfile );
    $fname   = basename( $tmpfile );
    $backup  = "$dname/.$fname";
    unlink( $tmpfile );
    unlink( $backup );

    $this->assertTrue( $is_file );
  }

  /**
   * test func
   */
  public function testSaveOrig() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::save_orig( $tmpfile );

    $is_file   = is_file( $tmpfile );
    $dname     = dirname( $tmpfile );
    $fname     = basename( $tmpfile );
    $backup    = "$dname/.$fname.orig";
    $is_file_b = is_file( $backup );
    unlink( $tmpfile );
    unlink( $backup );
    $this->assertTrue( $is_file );
    $this->assertTrue( $is_file_b );
  }

  /**
   * test func
   */
  public function testLockBarf() {
    $tmpfile = $this->getTempFile();

    $r = IWorxFileSys::lock_barf( $tmpfile, 'new content',
                                  $b = false,
                                  $s = false );
    $this->assertTrue( $r );
    $contents = file_get_contents( $tmpfile );
    $this->assertEquals( 'new content', $contents, 'file contents written' );
    unlink( $tmpfile );
  }

  /**
   * test func
   */
  public function testSmartBarf() {
    $tmpfile = $this->getTempFile();

    $r = IWorxFileSys::smart_barf( $tmpfile,
                                   'new content',
                                   false,
                                   false,
                                   0600,
                                   false,
                                   false );

    $this->assertTrue( $r );
    $contents = file_get_contents( $tmpfile );
    $this->assertEquals( 'new content', $contents, 'file contents written' );

    unlink( $tmpfile );

    $r = IWorxFileSys::smart_barf( $tmpfile,
                                   'new content',
                                   false,
                                   false,
                                   0600,
                                   false,
                                   false );
    $this->assertTrue( $r );
    $contents = file_get_contents( $tmpfile );
    $this->assertEquals( 'new content', $contents, 'file contents written' );

    unlink( $tmpfile );
  }

  /**
   * test func
   */
  public function testFilesize() {
    $tmpfile = $this->getTempFile();
    file_put_contents( $tmpfile, 'new content' );
    $size = IWorxFileSys::filesize( $tmpfile );
    $this->assertEquals( 11, $size, 'correct file size' );
    unlink( $tmpfile );
  }

  /**
   * test func
   */
  public function testStat() {
    $tmpfile = $this->getTempFile();
    file_put_contents( $tmpfile, 'new content' );
    $stat = IWorxFileSys::stat( $tmpfile );
    $this->assertArrayHasKey( 'dev',   $stat, 'dev exists' );
    $this->assertArrayHasKey( 'ino',   $stat, 'ino exists' );
    $this->assertArrayHasKey( 'mode',  $stat, 'mode exists' );
    $this->assertArrayHasKey( 'nlink', $stat, 'nlink exists' );
    $this->assertArrayHasKey( 'uid',   $stat, 'uid exists' );
    $this->assertArrayHasKey( 'gid',   $stat, 'gid exists' );
    $this->assertArrayHasKey( 'rdev',  $stat, 'rdev exists' );
    $this->assertArrayHasKey( 'size',  $stat, 'size exists' );
    $this->assertArrayHasKey( 'atime', $stat, 'atime exists' );
    $this->assertArrayHasKey( 'mtime', $stat, 'mtime exists' );
    $this->assertArrayHasKey( 'ctime', $stat, 'ctime exists' );
    $this->assertArrayHasKey( 'blksize', $stat, 'blksize exists' );
    $this->assertArrayHasKey( 'blocks',  $stat, 'blocks exists' );
    unlink( $tmpfile );
  }

  /**
   * test func
   */
  public function testGetDirUsedSpace() {
    $tmpfile = $this->getTempFile();
    file_put_contents( $tmpfile, 'new content' );
    $r = IWorxFileSys::get_dir_used_space( $this->getTempDir() );
    $this->assertTrue( is_numeric( $r ) );
    unlink( $tmpfile );
  }

  /**
   * test func
   */
  public function testGetMountmap() {
    $r = IWorxFileSys::get_mountmap();
    $this->assertGreaterThan( 0, count( $r ) );
  }

  /**
   * test func
   */
  public function testEregGlob() {
    $r = IWorxFileSys::preg_glob( '/tmp', '/^.*$/' );
    $this->assertGreaterThanOrEqual( 2, count( $r ) );
  }

  /**
   * test func
   */
  public function testGetFstab() {
    $r = IWorxFileSys::getFstab();
    $this->assertGreaterThan( 0, count( $r ) );
  }

  /**
   * test func
   */
  public function testGetQuota() {
    $r = IWorxFileSys::get_quota( 'root' );
    $this->assertTrue( is_numeric( $r ) );
  }

  /**
   * test func
   */
  public function testGetRawQuotaData() {
    $r = IWorxFileSys::get_raw_quota_data();
    $this->assertGreaterThan( 4, count( $r ) );
  }

  /**
   * test func
   */
  public function testSearchReplace() {
    $tempfile = $this->getTempFile();
    file_put_contents( $tempfile, 'you like it' );
    $r = IWorxFileSys::search_replace( $tempfile, 'like', 'really like' );
    $this->assertTrue( $r );
    $content = file_get_contents( $tempfile );
    unlink( $tempfile );
    $this->assertEquals( 'you really like it', $content );
  }

  /**
   * test func
   */
  public function testDirectoryIsEmpty() {
    $dir = $this->getTempDir();
    @mkdir( "$dir/newdir" );
    $this->assertTrue( IWorxFileSys::directoryIsEmpty( "$dir/newdir" ) );
    $this->assertFalse( IWorxFileSys::directoryIsEmpty( $dir ) );
    rmdir( "$dir/newdir" );
  }

  /**
   * test func
   */
  public function testRemoveEmptyDirectoriesFrom() {
    $dir = $this->getTempDir();
    @mkdir( "$dir/newdir" );
    @mkdir( "$dir/newdir/boo" );
    $this->assertTrue( IWorxFileSys::removeEmptyDirectoriesFrom( "{$dir}/newdir" ) );
  }

  /**
   * Tests the IWorxFileSys::getIdenticalFiles function
   */
  public function testGetIdenticalFiles() {
    $dir = $this->getTempDir();

    $dir1 = "$dir/testdir1";
    $dir2 = "$dir/testdir2";

    IWorxFileSys::mkdir( $dir1 );
    IWorxFileSys::mkdir( $dir2 );

    $content = 'This is a dummy file.';
    IWorxFileSys::smart_barf( "$dir1/dummyfile.txt", $content );
    IWorxFileSys::smart_barf( "$dir2/dummyfile.txt", $content );

    $files = IWorxFileSys::getIdenticalFiles( $dir1, $dir2 );
    $this->assertTrue( in_array( "$dir1/dummyfile.txt", $files ) );
    unset( $files );

    // now the bad file.
    IWorxFileSys::smart_barf( "$dir2/dummyfile2.txt", "$content and this is bad" );

    $files = IWorxFileSys::getIdenticalFiles( $dir1, $dir2 );

    IWorxFileSys::rm( $dir1, $r = true );
    IWorxFileSys::rm( $dir2, $r = true );

    $this->asserttrue( !empty( $files ) );
    $this->assertTrue( in_array( "$dir1/dummyfile.txt", $files ) );
    $this->assertTrue( !in_array( "$dir1/dummyfile2.txt", $files ) );
  }
}
?>
