<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    May 19, 2009
 * @id $Id: IWorxRailsTest.php,v 1.4 2009/08/21 02:30:01 poehler Exp $
 */

/**
 * IWorx Rails Test
 *
 * @package    InterWorx
 * @subpackage Rails
 *
 */
class IWorxRailsTest extends IWorx_Framework_TestCase {

  const APP_NAME = 'testrailsapp';
  const PROD     = 'production';
  const DEV      = 'development';

  /**
   * testCreateRailsApp
   */
  public function testCreateRailsApp() {
    $this->addSWAccountIfNecessary();
    $this->_createRailsApp( self::APP_NAME, self::PROD );

    $SiteWorx   = new SiteWorx( self::SW_DOMAIN );
    $domainroot = $SiteWorx->getDomainRoot( self::SW_DOMAIN );
    $app_name   = self::APP_NAME;

    $expected_path = "{$domainroot}/rails/{$app_name}";
    $this->assertTrue( is_dir( $expected_path ) );

    IWorxFileSys::rm( $expected_path, true, true );
    $this->deleteSWAccountIfNecessary();
  }

  /**
   * tests the getRailsApps function
   */
  public function testGetRailsApps() {
    $this->addSWAccountIfNecessary();
    $this->_createRailsApp( self::APP_NAME, self::PROD );

    $apps = IWorxRails::getRailsApps( self::SW_DOMAIN );

    $found = false;
    foreach( $apps as $app ) {
      if( $app['app_name'] == self::APP_NAME &&
          $app['environment'] == self::PROD ) {

        $found = true;
        break;
      }
    }
    $this->assertTrue( $found === true );
  }

  /**
   * testValidateRailsApplication
   */
  public function testValidateRailsApplication() {
    $this->addSWAccountIfNecessary();
    $this->_createRailsApp( self::APP_NAME, self::PROD );

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $domain   = $SiteWorx->getMasterDomainName();
    $app_name = self::APP_NAME;

    $ok = IWorxRails::validateRailsApplication( self::SW_DOMAIN, $app_name );

    $this->assertTrue( $ok );

    $domainroot = $SiteWorx->getDomainRoot( self::SW_DOMAIN );

    $expected_path = "{$domainroot}/rails/{$app_name}";
    IWorxFileSys::rm( $expected_path, true, true );

    $this->deleteSWAccountIfNecessary();
  }

  /**
   * testGetEnvironment
   */
  public function testGetEnvironemnt() {
    $this->addSWAccountIfNecessary();
    $this->_createRailsApp( self::APP_NAME, self::PROD );

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $domain   = $SiteWorx->getMasterDomainName();
    $app_name = self::APP_NAME;

    $environment   = IWorxRails::getEnvironment( $domain, $app_name );
    $possible_envs = array( self::DEV, self::PROD );

    $this->assertTrue( is_string( $environment ) );
    $this->assertTrue( in_array( $environment, $possible_envs ) === true );
    $this->assertTrue( $environment == self::PROD );
  }

  /**
   * testRestartApplication
   */
  public function testRestartApplication() {
    $this->addSWAccountIfNecessary();
    $this->_createRailsApp( self::APP_NAME, self::PROD );

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $domain   = $SiteWorx->getMasterDomainName();
    $app_name = self::APP_NAME;

    try {
      IWorxRails::restartApplication( $domain, $app_name );
      $this->assertTrue( true );
    } catch( IWorx_Exception $e ) {
      $this->assertTrue( false, 'Failed to restart rails application' );
    }
  }

  /**
   * private function, creates a test rails app
   *
   * @param string $app_name
   * @param string $environment
   * @return boolean
   */
  private function _createRailsApp( $app_name, $environment ) {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $domain   = $SiteWorx->getMasterDomainName();

    try {
      IWorxRails::createRailsApp( $domain, $environment, $app_name );
    } catch( IWorx_Exception $e ) {
      try {
        IWorxRails::deleteRailsApp( $domain, $app_name );
        IWorxRails::createRailsApp( $domain, $environment, $app_name );
      } catch( IWorx_Exception $e ) {
        // something must have gone really wrong at this point.
        $this->assertTrue( false, $e->getMessage() );
      }
    }

    return true;
  }
}
?>