<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @id $Id: IWorxValidateTest.php,v 1.29 2009/10/28 04:43:17 poehler Exp $
 */

require_once '/home/interworx/TESTS/unittests.conf.php';
require_once 'PHPUnit/Framework.php';

/**
 * Test class for IWorxValidate.
 * Generated by PHPUnit on 2008-07-29 at 17:36:44.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxValidateTest extends PHPUnit_Framework_TestCase
{
  /**
   * Runs the test methods of this class.
   *
   * @access public
   * @static
   */
  public static function main() {
    require_once 'PHPUnit/TextUI/TestRunner.php';

    $suite  = new PHPUnit_Framework_TestSuite('IWorxValidateTest');
    $result = PHPUnit_TextUI_TestRunner::run( $suite );
  }

  /* data providers start */
  /**
   * Data provider for valid integer input
   *
   * @return array
   */
  public function goodIntegerData() {
    return( array ( array( 1 ),
                    // This will pass because of the extra params
                    array( -1, -10, 0 ),
                    array( 5 ),
                    array( 10 ),
                    array( '1' ),
                    // This will pass because PHP evaluates it to 2 first
                    array( 2.0 ),
                    array( 0 ),
                    array( '0' ),
                    // This will pass because PHP evaluates it to an int first
                    array( 0x1234 ),
                    array( 9999999999, 0, 99999999999 ),
                    array( 9999999999.0, 0, 99999999999 )
                    )
            );
  }

  /**
   * Data provider for invalid integer input
   *
   * @return array
   */
  public function badIntegerData() {
    return( array ( array( 'iworx' ),
                    array( '' ),
                    array( ' ' ),
                    // This will fail because the default is to disallow negative
                    array( -1 ),
                    // This will fail because of the range set
                    array( 7, -10, 5 ),
                    array( '010X' ),
                    array( 3.01 ),
                    array( '2.0'),
                    array( array(1) ),
                    array( ';1 SELECT foo FROM bar;' ),
                    array( '.55' ),
                    array( '0xabcd' )
                    )
            );
  }

  /**
   * Data provider
   *
   * @return array
   */
  public function goodStringIsIntData() {
    return array(
                 array( '1' ),
                 array( 1 ),
                 array( '1234' ),
                 array( 1234 ),
                 array( '278934908' ),
                 array( 278934908 ),
                 array( '0' ),
                 array( '-24000' ),
                 array( -1 ),
                 array( '0711' ),
                 array( '999999999' ),
                 );
  }

  /**
   * Data provider
   *
   * @return array
   */
  public function badStringIsIntData() {
    return array(
                 array( '99999999999999999999' ), // Way too large
                 array( 'one' ),
                 array( '1 of these' ),
                 array( '0.45' ),
                 array( 'booo 14' ),
                 array( '13attempts' ),
                 array( '-14.69' ),
                 );
  }

  /**
   * Data provider for good directory names
   *
   * @return array
   */
  public function goodDirectoryNames() {
    return( array ( array( 'iworx' ),
                    array( 'includes/node_worx/whatever' ),
                    array( 'some-data' ),
                    array( '/root/desktop' ),
                    array( '_folder' )
                    )
            );
  }

  /**
   * Data provider for naughty directory names
   *
   * @return array
   */
  public function badDirectoryNames() {
    return( array ( array( 'iworx*' ),
                    array( '../includes/node_worx/whatever' ),
                    array( 'C:\whatever' ),
                    array( '..' ),
                    array( '.folder' ),
                    array( '/folder/with/OMG SPACES/' ),
                    array( '' ),
                    array( array( 'NO ARRAYS!' ) )
                    )
            );
  }

  /**
   * Data provider of good nicknames
   *
   * @return array
   */
  public function goodNicknames() {
    return( array ( array( 'iworx' ),
                    array( 'tim@iworx' ),
                    array( 'Paul (is beside me)' ),
                    array( '..' ),
                    array( '.folder' ),
                    array( 'Iñtérñátionál Chárs' ),
                    array( '阿爾及利亞 阿尔及利亚' ),
                    array( '우리는 행복합니다' ),
                    array( '%uC6B0%uB9AC%uB294 %uD589%uBCF5%uD569%uB2C8%uB2E4' ),
                    array( '/forward/Slashes/are/OK/' ),
                    array( str_repeat( 'a', 255 ) ),
                    array( 'weirdcharacterstotheright' . chr( 129 ) . chr( 254 ) )
                    )
            );
  }

  /**
   * Data provider of bad nicknames
   *
   * @return array
   */
  public function badNicknames() {
    return( array ( array( "iworx\t" ),
                    array( 'tim@iworx;' ),
                    array( 'Paul (is b:eside me)' ),
                    array( 'Backslashes\are\bad' ),
                    array( '\阿爾及利亞 阿尔及利亚' ),
                    array( '' ),
                    array( array( 'No ARRAYS!' ) ),
                    array( '<script>alert("GOTCHA BITCH!")</script>.folder' ),
                    array( 'Commas, are bad, and, overused' ),
                    // Too long, suck it
                    array( str_repeat( 'a', 256 ) ),
                    array( 'ASCII255isnot allowed' . chr( 255 ) . chr( 254 ))
                    )
            );
  }

  /**
   * Data provider of good filenames
   *
   * @return array
   */
  public function goodFilenames() {
    return( array ( array( 'iworx' ),
                    array( 'tim_iworx' ),
                    array( 'Paul-iworx' ),
                    array( 'chris9' ),
                    array( 'file.txt' )
                    )
            );
  }

  /**
   * Data provider of bad filenames
   *
   * @return array
   */
  public function badFilenames() {
    return( array ( array( 'iwo:rx' ),
                    array( 'tim iworx' ),
                    array( 'Paul-i+worx' ),
                    array( '/file.txt' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodUnixusers() {
    return( array ( array( 'iworx' ),
                    array( 'iworx2' ),
                    array( '25or6to4' ),
                    array( 'iworx-horde' ),
                    array( 'c' ),
                    array( str_repeat( 'a', 32 ) )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badUnixusers() {
    return( array ( array( 'iworx 1' ),
                    array( 'iworx.2' ),
                    array( '' ),
                    array( array( 'naughty' ) ),
                    array( str_repeat( 'a', 33 ) )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodFtpusers() {
    return( array ( array( 'iworx' ),
                    array( 'iworx2' ),
                    array( '25or6-4' ),
                    array( 'c' ),
                    array( 'tcollege@interworx.com' ),
                    array( 'tcollege%interworx.com' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badFtpusers() {
    return( array ( array( 'iworx 1' ),
                    array( 'iworx;2' ),
                    array( '' ),
                    array( array( 'naughty' ) ),
                    array( '<script>alert("BOO")</script>' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodAppnames() {
    return( array( array( 'redmine' ),
                   array( 'redmine2' ),
                   array( 'red-mine' ),
                   array( 'red-mine2' ),
                   array( 'red_mine2' ),
                   array( 'internal_application-no1' )
                   )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badAppnames() {
    return( array( array( '*()&*$#' ),
                   array( 'afd*5r7893 ffffffdf   4' ),
                   array( '5890f df f f      5' ),
                   array( '../../appname' ) ) );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodRakeCommands() {
    return( array( array( 'db:migrate' ),
                   array( 'db:dostuff' ),
                   array( 'goBuyVodka' ),
                   array( 'makeRailsSuckLess' )
                   )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badRakeCommands() {
    return( array( array( '*()&*$#' ),
                   array( 'afd*5r7893 ffffffdf   4' ),
                   array( '5890f df f f      5' ),
                   array( '../../appname' ) ) );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodMySQLusers() {
    return( array ( array( 'iworx' ),
                    array( 'iworx2' ),
                    array( '25or6' ),
                    array( '1234567' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badMySQLusers() {
    return( array ( array( '' ),
                    array( '*2' ),
                    array( 'c[]' ),
                    array( 'one.com' ),
                    array( 'tcol`x' ),
                    // Too long
                    array( '12345678' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodMySQLDatabases() {
    return( array ( array( 'iworx' ),
                    array( 'iworx2' ),
                    array( '25_or_6_to_4' ),
                    array( str_repeat( 'a', 64 ) ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badMySQLDatabases() {
    return( array ( array( 'iworx*' ),
                    array( 'iworx2/' ),
                    array( '25_or_6._to_4' ),
                    array( str_repeat( 'a', 65 ) ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodEmailAddresses() {
    return( array ( array( 'iworx@interworx.com' ),
                    array( 'tim.college@interworx.com' ),
                    array( 'tim_college@inter-worx.com' ),
                    array( 'tim_college@xn--asdf.com' ),
                    // 32-characters is the max for the user part
                    array( 'areasonablylongnamecanbeusedhere@interworx.com' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badEmailAddresses() {
    $really_long_name =
      'areasonablylongnamecanbeusedheresothereforethisistoolong@interworx.com';
    return( array ( array( 'iworx@inter@worx.com' ),
                    // Doesn't start with a letter or number
                    array( '_tim.college@interworx.com' ),
                    array( '@inter-worx.com' ),
                    // 32-characters is the max for the user part
                    array( $really_long_name ),
                    array( 'no spaces@interworx.com'),
                    array( 'nodomain@' ),
                    array( 'noTLD@interworx' ),
                    array( 'back`ticks@interworx.com'),
                    array( 'only{kids}shouldhavebraces@asdf.net')
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodDomains() {
    return( array ( array( 'interworx.com' ),
                    array( 'www.interworx.com' ),
                    array( 'subdomain.inter-worx.com' ),
                    array( 'sub.domain.inter-worx.com' ),
                    array( 'asdf.asdf.asfd.asfd.com' ),
                    array( 'xn--asdf.com' ),
                    array( str_repeat( 'a', 60 ) . '.com' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badDomains() {
    return( array ( array( 'interworxcom' ),
                    array( 'www.inter!worx.com' ),
                    array( 'www.InterWorx.com' ),
                    array( 'subdomain.inter-worx.c!om' ),
                    array( 'xn-\-asdf.com' ),
                    array( 'xn-\-asdf.c\om' ),
                    array( 'domain.123' ),
                    array( 'domain_.com'),
                    array( str_repeat( 'a', 61 ) . '.com' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodPrefixes() {
    return( array ( array( 'prefix' ),
                    // Only works with the second parameter true
                    array( '*', true ),
                    array( 'pre-fix' ),
                    array( str_repeat( 'a', 63 ) ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badPrefixes() {
    return( array ( array( '.prefix' ),
                    array( 'pre#fix' ),
                    // Only works with the second parameter true
                    array( '*' ),
                    array( 'pre_fix' ),
                    array( 'pre....fix' ),
                    array( 'pre&fix' ),
                    array( 'xn--asdf' ),
                    array( str_repeat( 'a', 64 ) ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodHostnames() {
    return( array ( array( 'domain.com' ),
                    array( '*.domain.com', true ),
                    array( 'domain.com', true ),
                    array( 'sub.domain.com' ),
                    array( 'localhost' ),
                    array( 'localhost.localdomain' ),
                    array( 'sub.sub.domain.com' ),
                    array( '1.dns-template.com' ),
                    array( '123.dns-template.com' ),
                    array( 'x10' ),
                    array( '2nd.com' ),
                    array( 'bl.spamcop.net' ),
                    array( 'ce-303.com' ),
                    array( '1234e1234.5431a12345.com' ),
                    array( '2nd.2nd.com', false, '2nd.com' ),
                    array( 'x-10' ),
                    array( 'x10.domain.com', false, 'domain.com' ),
                    array( 'x-1.domain.com', false, 'domain.com' ),
                    array( 'x-1-0.domain.com', false, 'domain.com' ),
                    array( 'domain.com', false, 'domain.com' ),
                    array( 'domain.com', true, 'domain.com' ),
                    array( '*.domain.com', true, 'domain.com' ),
                    array( '*.sub.domain.com', true, 'domain.com' ),
                    array( 'sub.sub.domain.com', false, 'domain.com' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badHostnames() {
    return( array ( array( '*domain.com' ),
                    array( '*.domain.com' ),
                    array( '.domain.com' ),
                    array( 'www..domain.com' ),
                    array( '*d.domain.com', true ),
                    array( 'sub.domain.12' ),
                    array( 'domain.12' ),
                    array( 'domain._' ),
                    array( 'domain.' ),
                    array( 'domain_.com' ),
                    array( 'domain.12' ),
                    array( '' ),
                    array( '*' ),
                    array( '', false, '' ),
                    array( '*', true, '*' ),
                    array( 'domain_.12', false, 'domain_.12' ),
                    array( 'domain.12', false, 'domain.12' ),
                    array( 'domain_.com', false, 'domain_.com' ),
                    array( 'sub.sub.domain.com/something' ),
                    array( 'sub.*.domain.com' ),
                    // Still fails, because this is bogus
                    array( 'cm._domainkey.domain.com', false, 'domain.com' ),
                    array( 'sub.*.domain.com', true ),
                    array( 'sub.sub.domain.com', false, 'otherdomain.com' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodURIs() {
    return( array ( array( 'http://domain.com', array('http', 'ftp') ),
                    array( 'ftp://domain.com', array('http', 'ftp') ),
                    array( 'http://domain.com/', array('http') ),
                    array( 'http://domain.com/path_to/the/file.txt', array('http') ),
                    array( '/absolute/file123/path/' ),
                    array( 'http://localhost', array( 'http' ) ),
                    array( 'ftp://localhost.localdomain', array( 'ftp' ) ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badURIs() {
    return( array ( array( '*domain.com' ),
                    array( '*.domain.com' ),
                    array( '.domain.com' ),
                    array( 'www..domain.com' ),
                    array( 'www.domain.com' ),
                    array( 'sub.domain.com' ),
                    array( '*d.domain.com' ),
                    array( 'localhost' ),
                    array( 'localhost.localdomain' ),
                    array( 'sub.sub.domain.com' ),
                    array( 'sub.domain.12' ),
                    array( 'domain.12' ),
                    array( 'domain._' ),
                    array( 'domain.' ),
                    array( 'domain_.com' ),
                    array( 'http://www.domain.com' ),
                    array( '' ),
                    array( '*' ),
                    array( 'http://', array( 'http' ) ),
                    array( 'http://www.domain.com', array( 'ftp' ) ),
                    array( 'www.domain.com', array( 'http' ) ),
                    array( 'relative/file123/path/to_file' ),
                    array( 'relative/file123/path/to_file.ext' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodExtensions() {
    return( array ( array( '.php' ),
                    array( '.html' ),
                    array( '.txt' ),
                    array( '.TEXT' ),
                    array( '._txt' ),
                    array( '._' ),
                    array( '.doc' ),
                    array( '.ta2' ),
                    array( '.tar.gz' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badExtensions() {
    return( array ( array( 'php' ),
                    array( '.*' ),
                    array( 'file.txt' ),
                    array( '?' ),
                    array( '.' ),
                    array( '' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodIps() {
    return( array ( array( '127.0.0.1' ),
                    array( '192.168.0.1' ),
                    array( '255.255.255.0' ),
                    array( '69.12.123.4' ),
                    array( '1.1.1.1' ),
                    array( '0.0.0.0' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badIps() {
    return( array ( array( '' ),
                    array( '1.*.1.1' ),
                    array( '192.168.0.256' ),
                    array( '999.1.1.1' ),
                    array( '192.168.1' ),
                    array( '0.1.2.3'),
                    array( '1111.123.123.123' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodMaskedIps() {
    return( array ( array( '127.0.0.1' ),
                    array( '192.168.0.1' ),
                    array( '255.255.255.0' ),
                    array( '69.12.123.4' ),
                    array( '1.1.1.1' ),
                    array( '10.1.123.' ),
                    array( '10.1.' ),
                    array( '10.' ),
                    array( '1.1.1.1/1', true ),
                    array( '0.0.0.0' ),
                    array( '127.0.0.1/2' ),
                    array( '192.168.0.1/24' ),
                    array( '255.255.255.0/4' ),
                    array( '1.1.1.1/32' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badMaskedIps() {
    return( array ( array( '' ),
                    array( '1.*.1.1' ),
                    array( '192.168.0.256' ),
                    array( '999.1.1.1' ),
                    array( '192.168.1' ),
                    array( '0.1.2.3'),
                    array( '1111.123.123.123' ),
                    array( '1.*.1.1/' ),
                    array( '192.168.0.256' ),
                    array( '999.1.1.1/33' ),
                    // Technically valid, but disallowed.  /0 is too broad
                    array( '69.12.123.4/0' ),
                    array( '0.0.0.0/0' ),
                    array( '192.168.1/27' ),
                    array( '0.1.2.3/4'),
                    array( '1111.123.123.123/12/' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodPorts() {
    return( array ( array( 1 ),
                    array( 21 ),
                    array( '8088' ),
                    array( '2443' ),
                    array( '65535' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badPorts() {
    return( array ( array( '' ),
                    array( -1 ),
                    array( 0 ),
                    array( 65536 ),
                    array( '0' ),
                    array( 'Frank' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodMd5Passes() {
    return( array ( array( crypt( 'boogers' ) ),
                    array( crypt( 'I like traffic lights!' ) ),
                    array( crypt( 'jkaghet0q3745928375-12 *(&)&*#% ieytr a' ) ),
                    array( crypt( 'boogers\asdasdasda' ) ),
                    array( crypt( 'InterWorx' ) ),
                    array( crypt( 'NodeWorx' ) ),
                    array( crypt( 'NOTWorx' ) ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badMd5Passes() {
    return( array ( array( '' ),
                    array( '$2$.dA3/2sd$aserLH.2342' ),
                    array( '1$.dA3/2sd$aserLH.2342' ),
                    array( '$1$.*A3/2sd$aserLH.2342' ),
                    array( '$1$.dA3/2sdw$aserLH.2342' ),
                    array( '$1$.d_A3/2sd$aserLH.2342' ),
                    array( '$1$.dA3\2sd$aserLH.2342' )
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodTruthyData() {
    $object           = ( object ) null;
    $object->property = 'A non-empty property';
    $emptyObject      = ( object ) null;

    return( array ( array( true ),
                    array( 1 ),
                    array( -1 ),
                    array( '0.0' ),
                    array( 'A non-empty string' ),
                    array( array( 'A non-empty array' ) ),
                    array( $object ),
                    array( 0.0 ),
                    array( $emptyObject ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function goodFalseyData() {
    $object = ( object ) null;
    return( array ( array( false ),
                    array( 0 ),
                    array( '0' ),
                    array( null ),
                    array( array( ) ),
                    array( '' ),
                    )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badNodeWorxIds() {
    return( array(
                  array( '' ),
                  array( false ),
                  array( 0 ),
                  array( null ),
                  array( -2 ),
                  array( '0; select' ),
                  // This one isn't guaranteed bad, but unlikely to exist
                  array( 200000 ),
                  )
            );
  }

  /**
   * Data Provider
   *
   * @return array
   */
  public function badNodeWorxUserIds() {
    return( array(
                  array( '' ),
                  array( false ),
                  array( 0 ),
                  array( null ),
                  array( -2 ),
                  array( '0; select' ),
                  // This one isn't guaranteed bad, but unlikely to exist
                  array( 200000 ),
                  )
            );
  }


  /* data providers end */
  /**
   * Tests validate integer
   *
   * @param integer $i
   * @param integer $min
   * @param integer $max
   *
   * @dataProvider goodIntegerData()
   */
  public function testIntegerReturnsTrueForIntegers( $i, $min = 0, $max = null ) {
    $this->assertTrue( IWorxValidate::integer( $i, $min, $max ) );
  }

  /**
   * Tests validate integer
   *
   * @param integer $i
   * @param integer $min
   * @param integer $max
   *
   * @dataProvider badIntegerData()
   */
  public function testIntegerReturnsFalseForNonIntegers( $i,
                                                         $min = 0,
                                                         $max = null ) {
    $this->assertFalse( IWorxValidate::integer( $i, $min, $max ) );
  }

  /**
   * Test validate string is integer
   *
   * @param string $i
   * @dataProvider goodStringIsIntData
   */
  public function testStringIsIntReturnsTrueForGoodData( $i ) {
    $this->assertTrue( IWorxValidate::stringIsInt( $i ) );
  }

  /**
   * Test validate string is integer
   *
   * @param string $i
   * @dataProvider badStringIsIntData
   */
  public function testStringIsIntReturnsFalseForBadData( $i ) {
    $this->assertFalse( IWorxValidate::stringIsInt( $i ) );
  }

  /**
   * This is stupid.  The whereClause function just returns true always.
   */
  public function testWhereClause() {
    $this->assertTrue( IWorxValidate::whereClause( 'bullshit', 'bullshit' ) );
  }

  /**
   * Tests directory names
   *
   * @param string $name
   * @dataProvider goodDirectoryNames
   */
  public function testDirectoryNameReturnsTrueForGoodNames( $name ) {
    $this->assertTrue( IWorxValidate::directoryName( $name ) );
  }

  /**
   * Tests directory names
   *
   * @param string $name
   * @dataProvider badDirectoryNames
   */
  public function testDirectoryNameReturnsFalseForBadNames( $name ) {
    $this->assertFalse( IWorxValidate::directoryName( $name ) );
  }

  /**
   * Tests nicknames
   *
   * @param string $nickname
   * @dataProvider goodNicknames
   */
  public function testNicknameReturnsTrueForGoodNicknames( $nickname ) {
    $this->assertTrue( IWorxValidate::nickname( $nickname ) );
  }

  /**
   * Tests nicknames
   *
   * @param string $nickname
   * @dataProvider badNicknames
   */
  public function testNicknameReturnsFalseForBadNicknames( $nickname ) {
    $this->assertFalse( IWorxValidate::nickname( $nickname ) );
  }

  /**
   * Tests filenames
   *
   * @param string $name
   * @dataProvider goodFilenames
   */
  public function testFilenameReturnsTrueForGoodFilenames( $name ) {
    $this->assertTrue( IWorxValidate::filename( $name ) );
  }

  /**
   * Tests filenames
   *
   * @param string $name
   * @dataProvider badFilenames
   */
  public function testFilenameReturnsFalseForBadFilenames( $name ) {
    $this->assertFalse( IWorxValidate::filename( $name ) );
  }

  /**
   * tests rails appname
   *
   * @param string $name
   * @dataProvider goodAppnames
   */
  public function testRailsappReturnsTrueForGoodApnames( $name ) {
    $this->assertTrue( IWorxValidate::railsapp( $name ) );
  }

  /**
   * tests rails appname
   *
   * @param string $name
   * @dataProvider badAppnames
   */
  public function testRailsappReturnsFalseForBadAppnames( $name ) {
    $this->assertFalse( IWorxValidate::railsapp( $name ) );
  }

  /**
   * Tests unixnames
   *
   * @param string $name
   * @dataProvider goodUnixusers
   */
  public function testUnixuserReturnsTrueForGoodUsernames( $name ) {
    $this->assertTrue( IWorxValidate::unixuser( $name ) );
  }

  /**
   * Tests unix user names
   *
   * @param string $name
   * @dataProvider badUnixusers
   */
  public function testUnixuserReturnsFalseForBadUsernames( $name ) {
    $this->assertFalse( IWorxValidate::unixuser( $name ) );
  }

  /**
   * Tests ftp user names
   *
   * @param string $name
   * @dataProvider goodFtpusers
   */
  public function testFtpuserReturnsTrueForGoodUsernames( $name ) {
    $this->assertTrue( IWorxValidate::ftpuser( $name ) );
  }

  /**
   * Tests ftp user names
   *
   * @param string $name
   * @dataProvider badFtpusers
   */
  public function testFtpuserReturnsFalseForBadUsernames( $name ) {
    $this->assertFalse( IWorxValidate::ftpuser( $name ) );
  }

  /**
   * Tests mysql user names
   *
   * @param string $name
   * @dataProvider goodMySQLusers
   */
  public function testMysqlDbuserReturnsTrueForGoodUsernames( $name ) {
    $this->assertTrue( IWorxValidate::Mysql_dbuser( $name ) );
  }

  /**
   * Tests mysql user names
   *
   * @param string $name
   * @dataProvider badMySQLusers
   */
  public function testMysqlDbuserReturnsFalseForBadUsernames( $name ) {
    $this->assertFalse( IWorxValidate::Mysql_dbuser( $name ) );
  }

  /**
   * Tests mysql database names
   *
   * @param string $name
   * @dataProvider goodMySQLDatabases
   */
  public function testMysqlDbuserReturnsTrueForGoodDbNames( $name ) {
    $this->assertTrue( IWorxValidate::Mysql_dbname( $name ) );
  }

  /**
   * Tests mysql database names
   *
   * @param string $name
   * @dataProvider badMySQLDatabases
   */
  public function testMysqlDbuserReturnsFalseForBadDbNames( $name ) {
    $this->assertFalse( IWorxValidate::mysql_dbname( $name ) );
  }

  /**
   * Tests email addresses
   *
   * @param string $name
   * @dataProvider goodEmailAddresses
   */
  public function testEmailReturnsTrueForGoodNames( $name ) {
    $this->assertTrue( IWorxValidate::email( $name ) );
  }

  /**
   * Tests email addresses
   *
   * @param string $name
   * @dataProvider badEmailAddresses
   */
  public function testEmailReturnsFalseForBadNames( $name ) {
    $this->assertFalse( IWorxValidate::email( $name ) );
  }

  /**
   * Tests domains
   *
   * @param string $name
   * @dataProvider goodDomains
   */
  public function testDomainReturnsTrueForGoodDomains( $name ) {
    $this->assertTrue( IWorxValidate::domain( $name ) );
  }

  /**
   * Tests domains
   *
   * @param string $name
   * @dataProvider badDomains
   */
  public function testDomainReturnsFalseForBadDomains( $name ) {
    $this->assertFalse( IWorxValidate::domain( $name ) );
  }

  /**
   * Tests host name prefixes
   *
   * @param string $name
   * @param string $wildcard
   * @dataProvider goodPrefixes
   */
  public function testHostnamePrefixReturnsTrueForGoodPrefixes( $name,
                                                                $wildcard = false ) {
    $this->assertTrue( IWorxValidate::hostname_prefix( $name, $wildcard ) );
  }

  /**
   * Tests hostname prefixes
   *
   * @param string $name
   * @param string $wildcard
   * @dataProvider badPrefixes
   */
  public function testHostnamePrefixReturnsFalseForBadPrefixes( $name,
                                                                $wildcard = false ) {
    $this->assertFalse( IWorxValidate::hostname_prefix( $name, $wildcard ) );
  }

  /**
   * Tests hostnames
   *
   * @param string  $name
   * @param boolean $wildcard
   * @param string  $base
   * @dataProvider goodHostnames
   */
  public function testHostnameReturnsTrueForGoodHostnames( $name,
                                                           $wildcard = false,
                                                           $base = false ) {
    $this->assertTrue( IWorxValidate::hostname( $name, $wildcard, $base ) );
  }

  /**
   * Tests hostnames
   *
   * @param string  $name
   * @param boolean $wildcard
   * @param string  $base
   * @dataProvider badHostnames
   */
  public function testHostnameReturnsFalseForBadHostnames( $name,
                                                           $wildcard = false,
                                                           $base = false ) {
    $this->assertFalse( IWorxValidate::hostname( $name, $wildcard, $base ) );
  }

  /**
   * Tests Uris
   *
   * @param string        $uri
   * @param array|boolean $protocols
   * @dataProvider goodURIs
   */
  public function testUriReturnsTrueForGoodUris( $uri, $protocols = false ) {
    $this->assertTrue( IWorxValidate::uri( $uri, $protocols ) );
  }

  /**
   * Tests uris
   *
   * @param string        $uri
   * @param array|boolean $protocols
   * @dataProvider badURIs
   */
  public function testUriReturnsFalseForBadUris( $uri, $protocols = false ) {
    $this->assertFalse( IWorxValidate::uri( $uri, $protocols ) );
  }

  /**
   * Tests file extensions
   *
   * @param string $ext
   * @dataProvider goodExtensions
   */
  public function testExtensionReturnsTrueForGoodExtensions( $ext ) {
    $this->assertTrue( IWorxValidate::extension( $ext ) );
  }

  /**
   * Tests file extensions
   *
   * @param string $ext
   * @dataProvider badExtensions
   */
  public function testExtensionReturnsFalseForBadExtensions( $ext ) {
    $this->assertFalse( IWorxValidate::extension( $ext ) );
  }

  /**
   * Tests IPs
   *
   * @param string $ip
   * @dataProvider goodIps
   */
  public function testIpReturnsTrueForGoodIps( $ip ) {
    $this->assertTrue( IWorxValidate::ip( $ip ) );
  }

  /**
   * Tests IPs
   *
   * @param string $ip
   * @dataProvider badIps
   */
  public function testIpReturnsFalseForBadIps( $ip ) {
    $this->assertFalse( IWorxValidate::ip( $ip ) );
  }

  /**
   * Tests masked IPs
   *
   * @param string  $a
   * @param boolean $b
   * @dataProvider goodMaskedIps
   */
  public function testMaskedIpReturnsTrueForGoodIps( $a, $b = false ) {
    $this->assertTrue( IWorxValidate::masked_ip( $a, $b ) );
  }

  /**
   * Tests masked IPs
   *
   * @param string  $a
   * @param boolean $b
   * @dataProvider badMaskedIps
   */
  public function testMaskedIpReturnsFalseForBadIps( $a, $b = false ) {
    $this->assertFalse( IWorxValidate::masked_ip( $a, $b ) );
  }

  /**
   * Tests MD5 Crypt
   *
   * @param string $input
   * @dataProvider goodMd5Passes
   */
  public function testMd5CryptPasswdReturnsTrueForGoodPasswd( $input ) {
    $this->assertTrue( IWorxValidate::md5_crypt_passwd( $input ) );
  }

  /**
   * Tests D5 Crypt
   *
   * @param string $input
   * @dataProvider badMd5Passes
   */
  public function testMd5CryptPasswdReturnsFalseForBadPasswd( $input ) {
    $this->assertFalse( IWorxValidate::md5_crypt_passwd( $input ) );
  }

  /**
   * Tests ports
   *
   * @param integer $input
   * @dataProvider goodPorts
   */
  public function testPortReturnsTrueForGoodPorts( $input ) {
    $this->assertTrue( IWorxValidate::port( $input ) );
  }

  /**
   * Tests ports
   *
   * @param integer $input
   * @dataProvider badPorts
   */
  public function testPortReturnsFalseForBadPorts( $input ) {
    $this->assertFalse( IWorxValidate::port( $input ) );
  }

  /**
   * Tests truthy data
   *
   * @param mixed $input
   * @dataProvider goodTruthyData
   */
  public function testTruthyReturnsTrueForGoodData( $input ) {
    $this->assertTrue( IWorxValidate::truthy( $input ) );
  }

  /**
   * Tests falsey data
   *
   * @param mixed $input
   * @dataProvider goodFalseyData
   */
  public function testFalseyReturnsTrueForGoodData( $input ) {
    $this->assertTrue( IWorxValidate::falsey( $input ) );
  }

  /**
   * Tests truthy data
   *
   * @param mixed $input
   * @dataProvider goodFalseyData
   */
  public function testTruthyReturnsFalseForBadData( $input ) {
    $this->assertFalse( IWorxValidate::truthy( $input ) );
  }

  /**
   * Tests falsey data
   *
   * @param mixed $input
   * @dataProvider goodTruthyData
   */
  public function testFalseyReturnsFalseForBadData( $input ) {
    $this->assertFalse( IWorxValidate::falsey( $input ) );
  }

  /**
   * Tests NodeWorx Ids
   *
   * @dataProvider goodTruthyData
   */
  public function testNodeWorxIdReturnsTrueForGoodData() {
    // NodeWorx Id 1 will always exist, so we test that
    $this->assertTrue( IWorxValidate::nodeWorxId( 1 ) );
  }

  /**
   * Tests NodeWorx Ids
   *
   * @param mixed $input
   * @dataProvider badNodeWorxIds
   */
  public function testNodeWorxIdReturnsFalseForBadData( $input ) {
    $this->assertFalse( IWorxValidate::nodeWorxId( $input ) );
  }

  /**
   * Tests NodeWorx user IDs
   */
  public function testNodeWorxUserIdReturnsTrueForGoodData() {
    // NodeWorx User Id 1 will always exist, so we test that
    $this->assertTrue( IWorxValidate::nodeWorxUserId( 1 ) );
  }

  /**
   * Tests nodeworx user IDs
   *
   * @param mixed $input
   * @dataProvider badNodeWorxUserIds
   */
  public function testNodeWorxUserIdReturnsFalseForBadData( $input ) {
    $this->assertFalse( IWorxValidate::nodeWorxUserId( $input ) );
  }
}
?>
