<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2008 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Apr 20, 2009
 * @id $Id: ServerTest.php,v 1.3 2009/08/21 02:29:46 poehler Exp $
 */

/**
 * Tests for the MySQLServer object
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class MySQL_ServerTest extends IWorx_Framework_TestCase {

  /**
   * tests if the test SiteWorx account has a MySQL Server
   */
  public function testSiteWorxHasMySQLServer() {
    self::addSWAccountIfNecessary();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Server   = $SiteWorx->getMySQLServer();

    $this->assertTrue( $Server->exists() === true );
  }

  /**
   * tests if the accessors are working
   */
  public function testAccessorsAreWorking() {
    self::addSWAccountIfNecessary();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Server   = $SiteWorx->getMySQLServer();

    $this->assertTrue( $Server->getDsn() != null );
    $this->assertTrue( $Server->getActive() != null );
    $this->assertTrue( $Server->getNickname() != null );
  }

  /**
   * tests if the connection to the server is working
   */
  public function testConnectionIsWorking() {
    self::addSWAccountIfNecessary();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Server   = $SiteWorx->getMySQLServer();

    $ok = test_dsn( $Server->getDsn() );

    $this->assertTrue( $ok === true );
  }

  /**
   * Tests to see if the mysql server save function is working properly
   */
  public function testSaveIsWorking() {
    self::addSWAccountIfNecessary();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Server   = $SiteWorx->getMySQLServer();

    $old_nickname = $Server->getNickname();

    $Server->setNickname( 'NewNickname' );
    $ok = $Server->save();
    $this->assertTrue( $ok === true );

    $Server->setNickname( $old_nickname );
    $ok = $Server->save();
    $this->assertTrue( $ok === true );

    $this->assertEquals( $old_nickname, $Server->getNickname() );
  }

  /**
   * Tests to make sure the getHost function works
   */
  public function testHostIsOk() {
    self::addSWAccountIfNecessary();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Server   = $SiteWorx->getMySQLServer();

    $host = $Server->getHost();

    $this->assertTrue( !empty( $host ) === true );
  }

  /**
   * tests the function which creates the IWorx user dsn on the MySQL Server
   */
  public function testGetIWorxUserDsn() {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Server   = $SiteWorx->getMySQLServer();

    $new_dsn = MySQLServer::createIWorxUserDsn( $Server->getDsn() );

    try {
      $conn = IWorxDbUtil::db_connect( $new_dsn );
      IWorxDbUtil::db_disconnect( $conn );
      $ok = true;
      $msg = null;
    } catch( Exception $e ) {
      $msg = $e->getMessage();
      $ok = false;
    }

    /*
     * BEGIN CLEANUP
     *
     */
    // delete the mysql user
    $db = IWorxDbUtil::db_connect( $Server->getDsn() );

    $p      = Db::parseDSN( $new_dsn );
    $q_fqun = q( $p['username'] );

    $sql = "DELETE FROM mysql.user
      WHERE User = $q_fqun";

    IWorxDbUtil::db_query( $db, $sql );

    $sql = "DELETE FROM mysql.db
      WHERE User = $q_fqun";

    IWorxDbUtil::db_query( $db, $sql );

    // finally, flush the mysql perms
    $sql = 'FLUSH PRIVILEGES';

    IWorxDbUtil::db_query( $db, $sql );

    /*
     * END CLEANUP
     */

    $this->assertTrue( $ok === true, $msg );
  }

  /**
   * Tests the getServers function
   */
  public function testGetServers() {
    $NodeWorx = new NodeWorx( self::NW_USER );
    $Servers  = MySQLServer::getServers( $NodeWorx );

    $this->assertTrue( !empty( $Servers ) === true );
    foreach( $Servers as $Server ) {
      $this->assertTrue( $Server->exists() === true );
    }
  }

  /**
   * gets the server count function
   */
  public function testGetServerCount() {
    $NodeWorx = new NodeWorx( self::NW_USER );
    $count    = MySQLServer::getServerCount( $NodeWorx );

    $this->assertTrue( is_numeric( $count ) );
  }

  /**
   * Gets the local server and make sure it was created properly
   */
  public function testGetLocalServer() {
    $LocalServer = MySQLServer::getLocalServer();

    $this->assertTrue( !is_null( $LocalServer ) === true );
    $this->assertTrue( $LocalServer->isLocal() );
  }
}
?>