/*jslint undef: true, widget: true, browser: true, onevar: true, eqeqeq: true */
/*global YAHOO iw_message iw_lib iw_form */
/**
 * Singleton object (sorta) set up to handle communications. Specifically
 * designed to communicate with xhr.php and receive JSON_Replys
 */

var iw_xhr = function() {
  var _caching = true,
      _url     = '/xhr.php',
      _registered_transactions,
      _h;

  /**
   * Object for keeping track of currently running  transactions
   *
   * format is basically a hash table, with trans_{id_number} as the index
   */
  _registered_transactions = {};

  /**
   * Returns an immutable object containing the reply
   */
  function _reply( c, t, d ) {
    d = d || null;
    return {
      getReplyCode: function() {
        return c;
      },
      getReplyText: function() {
        return t;
      },
      getReplyData: function() {
        return d;
      },
      getReplySeries: function() {
        return Math.floor( c / 100 ) * 100;
      }
    };
  }

  /**
   * Default handler.  Put functions here if you need to handle a specific
   * response globally, such as not being logged in, or permissions issues.
   */
  _h = {
    handle200series: function( r ) {
      iw_lib.iw_message( 'A 200 Series reply was recieved, but the handler did not ' +
                  'do anything with it - perhaps a handle2xx function was forgotten?',
                  'Unexpected 200 series reply' );
    },
    handle401: function( r ) {
      iw_lib.iw_message( '##LG_JSONREPLY401DETAIL##', '##LG_JSONREPLY401##' );
    },
    handle403: function( r ) {
      iw_lib.iw_message( '##LG_JSONREPLY403DETAIL##', '##LG_JSONREPLY403##' );
    }
  };

  /**
   * Hands off the reply to the handler, if possible
   * @param object h Handler object
   * @param object r Output of _reply()
   * @return boolean
   */
  function _dispatchHandler( h, r, args ) {
    // STEP 1:  look for a function specific to the replyCode
    // EXAMPLE: handle201, handle603, etc.
    if( YAHOO.lang.isFunction( h['handle' + r.getReplyCode()] ) ) {
      h['handle' + r.getReplyCode()]( r, args );
      return true;
    // STEP 2:  look for a function that handles an entire series
    // EXAMPLE: handle200series, handle600series
    } else if( YAHOO.lang.isFunction( h['handle' +
                                      r.getReplySeries() +
                                      'series'] )) {
      h['handle' + r.getReplySeries() + 'series']( r, args );
      return true;
    // STEP 3: nothing more specific was defined, so use the default handler
    } else {
      return false;
    }
  }

  /**
   * Handles successful response from the server (whether that response was an
   * error or not is irrelevant)
   * @param object o Response object from YUI
   */
  function _handleSuccess( o ) {
    var temp, r, h, args;
    try {
      iw_xhr.unregisterTransaction( o.tId );
      temp   = YAHOO.lang.JSON.parse( o.responseText );
      temp.d = temp.d || null;

      r    = _reply( temp.c, temp.t, temp.d );
      h    = o.argument.h;
      args = o.argument.args;

      if( _dispatchHandler( h, r, args ) ) {
        return;
      } else if( _dispatchHandler( _h, r, args ) ) {
        return;
      } else {
        iw_xhr.handleError( r );
      }
    } catch( e ) {
      if( YAHOO.lang.isFunction( o.argument ) ) {
        iw_xhr.handleError( _reply( 501, '##LG_JSONUNABLETOPARSE## JSONReply' ) );
      } else {
        iw_xhr.handleError( _reply( 600, '<br/>Uncaught Exception' +
                                         '<br/>' +
                                         'File: ' + e.fileName +
                                         '<br/>' +
                                         'Line: ' + e.lineNumber +
                                         '<br/>' +
                                         e.name + ': ' + e.message ) );
      }
    }
  }

  /**
   * Triggered if the xhr.php script fails to respond completely
   * @param object Response object from YUI
   */
  function _handleFailure( o ) {
    if( o.status === -1 ) {
      alert( o.tId );
      return;
    }
    iw_xhr.handleError( _reply( 504, '##LG_JSONREPLY504##' ) );
  }

  /**
   * Dynamically provides the callback handler for YUI
   * @param object h Handler object
   * @return object
   */
  function _callback( h, args ) {
    return  { success: _handleSuccess,
              failure: _handleFailure,
              cache: _caching,
              argument: { 'h': h, 'args': args }
    };
  }

  // Return an object containing the public methods and properties
  return {
    /**
     * Gets the response from the server and handles it using the handler
     *
     * @param object h     Handler object with functions for dealing with results
     * @param mixed  input Input for the JSON_Responder
     * @param mixed  args  Arguments to pass back to handler function
     */
    getReply: function( h, input, args, form ) {
      var i, query_string, trans;
      input = input || null;
      args  = args || null;
      form  = form || null;
      if( YAHOO.lang.isUndefined( h.responder ) ) {
        throw "h.responder was not a defined";
      }
      i = {r: h.responder, i: input};
      i = YAHOO.lang.JSON.stringify( i );
      i = escape( i );

      if( !YAHOO.lang.isNull( form ) ) {
        YAHOO.util.Connect.setForm( form );
      }
      trans = YAHOO.util.Connect.asyncRequest( 'POST', _url, _callback( h, args ), 'i=' + i );
      iw_xhr.registerTransaction( trans );
    },

    /**
     * Makes a _reply object
     */
    makeReply: function( c, t, d ) {
      return _reply( c, t, d );
    },

    /**
     *
     */
    abortTransactions: function( e ) {
      var trans;
      for( trans in _registered_transactions ) {
        if( _registered_transactions.hasOwnProperty( trans ) ) {
          YAHOO.util.Connect.abort( _registered_transactions[ trans ] );
        }
      }
    },
    /**
     * Central error handler for default responses
     * @param object r Output of _reply()
     */
    handleError: function( r ) {
      var dump = 'replyCode: ' + r.getReplyCode() + "\n" +
                 'replyText: ' + r.getReplyText() + "\n" +
                 'replyData: ' + "\n" + r.getReplyData();

      iw_lib.iw_message( dump );
    },

    /**
     *
     */
    registerTransaction: function( obj ) {
     // alert( 'registered ' + obj.tId );
      _registered_transactions['trans_' + obj.tId ] = obj;
    },

    /**
     *
     */
    unregisterTransaction: function( tId ) {
      //alert( 'unregistering ' + tId );
      delete _registered_transactions['trans_' + tId ];
    },

    /**
     * Sets whether or not the service will cache results
     * @param boolean bool
     */
    setCaching: function( bool ) {
      _caching = bool;
    }
  };
}();

/**
 * A special handler that is used to handle responses from a Controller/Action
 * combination xhr request.
 */
var iw_ctrl_handler = {
  responder: 'Controller',

  /**
   * A 201 is a successful response
   */
  handle200: function( r, callback ) {
    if( YAHOO.lang.isFunction( callback.callback ) ) {
      callback.callback( r, true, callback.args );
    }
  },

  /**
   * A 201 is a successful response
   */
  handle201: function( r, callback ) {
    iw_ctrl_handler.handle200( r, callback );
  },

  /**
   * A 204 means somethign went wrong
   */
  handle204: function( r, callback ) {
    if( YAHOO.lang.isFunction( callback.callback ) ) {
      callback.callback( r, false, callback.args );
    }
  },

  handle500series: function( r, callback ) {
    iw_ctrl_handler.handle700series( r, callback );
  },

  handle700series: function( r, callback ) {
    iw_form.clearStatus( callback.form.submit );
    callback.form.submit.disabled = 0;
    iw_xhr.handleError( r );
  },

  /**
   * Fire off the request to iw_xhr
   *
   * Callback can have two forms - the first form is to just be a function itself.
   * In this form, the callback function is called with two arguments; the reply,
   * and a boolean value indicating true or false.
   *
   * The second form is an object, like this
   *
   * {
   *   callback: function(),
   *   args: mixed
   * }
   *
   * In this second form, the args parameter is simply passed along to the
   * callback function as the third parameter.  Doesn't matter what it is.
   *
   * @param element         form     HTML form element
   * @param function|object callback see above
   * @param string|null     ctrl     controller path
   * @param string|null     action   action function name
   * @param string|null     input    additional input to the controller action
   */
  execute: function( form, callback, ctrl, action, input ) {
    var new_callback = {};

    if( YAHOO.lang.isFunction( callback ) ) {
      new_callback.callback = callback;
      new_callback.args     = null;
      new_callback.form     = form;
    } else {
      new_callback.callback = callback.callback;
      new_callback.args     = callback.args;
      new_callback.form     = form;
    }

    input = input || null;

    if( YAHOO.lang.isNull( input ) ) {
      input = {};
    }

    if( !YAHOO.lang.isNull( ctrl ) && !YAHOO.lang.isUndefined( ctrl ) ) {
      input.c = ctrl;
      input.a = action;
    } else {
      input.c = form.iw_ctrl_name.value;
      input.a = form.iw_action_name.value;
    }
    iw_xhr.getReply( iw_ctrl_handler, input, new_callback, form );
  }
};

var links = document.getElementsByTagName('a');
YAHOO.util.Event.on( window, 'beforeunload', iw_xhr.abortTransactions );
