<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2009 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Sep 24, 2009
 * @id $Id: Simplescripts.php,v 1.9 2009/11/04 16:33:19 tcollege Exp $
 */

/**
 * SimpleScripts Plugin
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_Simplescripts extends Plugin_SiteworxFeature {
  const DEFAULT_HOST_KEY     = 'bjom0npWGOPk';
  const DEFAULT_HOST_API_KEY = 'PvhajTAy9D9MK4t0Ax8Ay57qzye6A5Qf';
  const CLIENT_LOGIN         = 'https://www.simplescripts.com/api/client_login.php';

  protected $_sw_menu_text = 'SimpleScripts';

  protected $_sw_menu_url = '/siteworx/simplescripts';

  //protected $_sw_menu_parent = 'iw-menu-features';

  //protected $_sw_menu_class = 'iw-i-plugin';

  protected $_sw_menu_style =
  'background: #fff url(/plugins/images/simplescripts/simplescripts.gif)';

  //protected $_sw_menu_location_after = 'iw-menu-features-ssl';

  protected $_sw_menu_id = 'menu-simplescripts';

  protected $_sw_account_group_name = 'simplescripts';

  protected $_sw_account_group_label = 'SimpleScripts Plugin Options';

  protected $_sw_account_option_name = 'simplescripts';

  //protected $_sw_account_option_default = 1;

  protected $_sw_account_option_label = 'Access to SimpleScripts';

  protected $_sw_user_perm_name = 'simplescripts';

  protected $_sw_user_perm_label = 'SimpleScripts';

  /**
   * get category
   *
   * @return string
   */
  public function getCategory() {
    return Plugin_Category::SCRIPT_INSTALLERS;
  }

  /**
   * get priority
   *
   * @return integer
   */
  public function getPriority() {
    return 30;
  }

  /**
   * Integrate into the nodeworx menu
   *
   * @param IWorxMenuManager $MenuMan
   */
  public function updateNodeworxMenu( IWorxMenuManager $MenuMan ) {
    $new_data = array( 'text' => 'Simple Scripts',
                       'url' => '/nodeworx/simplescripts',
                       'parent' => 'iw-menu-sw',
                       'class' => 'iw-i-plugin',
                       'style' => $this->_sw_menu_style );

    if( $this->getDatastoreVar( 'reseller_config' ) === 'use_main' ) {
      $new_data['disabled_for_reseller'] = '1';
    }
    $MenuMan->addMenuItemAfter( 'iw-menu-sw-import',
                                'menu-simplescripts',
                                $new_data );
  }

  /**
   * Launch simplescripts
   *
   * Note: if no error, DOES NOT RETURN
   *
   * @return string
   */
  public function launchSimpleScripts() {
    $reseller_config = $this->getDatastoreVar( 'reseller_config' );

    if( $reseller_config === 'use_main' ) {
      $nw_id = NodeWorx::MASTER_ID;
    } else {
      $nw_id = IW::SW()->getParentId();
    }

    $host_key     = $this->getDatastoreVar( "host_key:{$nw_id}" );
    $host_api_key = $this->getDatastoreVar( "host_api_key:{$nw_id}" );

    if( $host_key === '' && $host_api_key === '' ) {
      $host_key     = self::DEFAULT_HOST_KEY;
      $host_api_key = self::DEFAULT_HOST_API_KEY;
    }

    $uniqname = IW::SW()->getUniqname();
    $domain   = IW::SW()->getWorkingDomainName();
    $ip       = IW::SW()->getIPObj()->getDNSIP();
    $email    = IW::SW()->getWorkingUser()->getEmail();
    $ftpuser  = "simplescripts@{$domain}";
    $ftppass  = IWorxUtil::generate_password( 15 );
    $cp_sess  = session_id();

    $Ftp = IW::SW()->getFTPDriver();
    if( $Ftp->exists( 'simplescripts' ) ) {
      $Ftp->update( 'simplescripts', $ftppass, IW::SW()->getPackageRoot() );
    } else {
      $Ftp->add( 'simplescripts', $ftppass, IW::SW()->getPackageRoot() );
    }

    $url_host = parse_url( $_SERVER['HTTP_REFERER'], PHP_URL_HOST );
    $my_request = array(
                        'host_key'      => $host_key,
                        'host_api_key'  => $host_api_key,
                        'user_unique'   => $uniqname,
                        'user_domain'   => $domain,
                        'user_ip'       => $ip,
                        'user_email'    => $email,
                        'user_username' => $ftpuser,
                        'user_password' => $ftppass,
                        'user_cppass'   => $cp_sess,
                        'user_cprefer'  => $url_host
                        );

    return $this->_sendClientLogin( $my_request );
  }

  /**
   * send client login helper
   *
   * @param array $my_request
   * @return array only if error
   */
  private function _sendClientLogin( array $my_request ) {
    $my_request_str = IWorxHTTP::http_build_query( $my_request );

    $ch = curl_init( self::CLIENT_LOGIN );
    curl_setopt( $ch, CURLOPT_POSTFIELDS, $my_request );
    curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
    curl_setopt( $ch, CURLOPT_TIMEOUT, 60 );
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );

    $response = curl_exec( $ch );

    $parts = explode( '|', $response );
    if( $parts[ 0 ] != '1' ) {
      $stripped = strip_tags( $parts[ 1 ] );
      $errors = array( 'Something went wrong connecting to simple scripts. ' .
                       'Please refresh this page.',
                       "Response was: \"{$stripped}\"" );
      return $errors;
    } else {
      header( 'Location: ' . $parts[ 1 ] );
      exit;
    }
  }
}
?>